@@ -1,6 +1,6 @@
-/* $Id: configure.c,v 3.30 1994/03/30 16:53:02 bert Exp $
+/* $Id: configure.c,v 3.42 1995/02/02 09:51:28 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -49,10 +49,9 @@
  *      should be given as NULL.
  *   f) Add one line to the Config_save() routine to have the option saved.
  * 7: Document your option in the manual page for the client.
- * 8: Mail a context diff of your changes to xpilot@cs.uit.no.
+ * 8: Mail a context diff (diff -c old new) of your changes to xpilot@cs.uit.no.
  */
 
-#include <X11/Xproto.h>
 #include <X11/Xlib.h>
 #include <X11/Xos.h>
 #include <X11/Xutil.h>
@@ -75,19 +74,22 @@
 #include <limits.h>
 
 #include "version.h"
-#include "client.h"
+#include "config.h"
+#include "const.h"
 #include "paint.h"
-#include "draw.h"
 #include "xinit.h"
 #include "bit.h"
+#include "keys.h"
 #include "netclient.h"
 #include "widget.h"
 #include "configure.h"
 #include "setup.h"
 #include "error.h"
 
+char configure_version[] = VERSION;
+
 #ifndef PATH_MAX
-#define PATH_MAX	1024
+#define PATH_MAX	1023
 #endif
 
 static int Config_create_power(int widget_desc, int *height);
@@ -107,6 +109,8 @@ static int Config_create_fuelWarning(int widget_desc, int *height);
 static int Config_create_fuelCritical(int widget_desc, int *height);
 static int Config_create_fuelGauge(int widget_desc, int *height);
 static int Config_create_outlineWorld(int widget_desc, int *height);
+static int Config_create_filledWorld(int widget_desc, int *height);
+static int Config_create_texturedWalls(int widget_desc, int *height);
 static int Config_create_slidingRadar(int widget_desc, int *height);
 static int Config_create_showItems(int widget_desc, int *height);
 static int Config_create_showItemsTime(int widget_desc, int *height);
@@ -118,6 +122,10 @@ static int Config_create_toggleShield(int widget_desc, int *height);
 static int Config_create_sparkProb(int widget_desc, int *height);
 static int Config_create_shotSize(int widget_desc, int *height);
 static int Config_create_teamShotSize(int widget_desc, int *height);
+static int Config_create_hudColor(int widget_desc, int *height);
+static int Config_create_hudLockColor(int widget_desc, int *height);
+static int Config_create_wallColor(int widget_desc, int *height);
+static int Config_create_maxFPS(int widget_desc, int *height);
 #ifdef SOUND
 static int Config_create_maxVolume(int widget_desc, int *height);
 #endif
@@ -147,6 +155,7 @@ static int Config_update_turnSpeed(int widget_desc, void *data, float *val);
 static int Config_update_sparkProb(int widget_desc, void *data, float *val);
 static int Config_update_charsPerSecond(int widget_desc, void *data, int *val);
 static int Config_update_toggleShield(int widget_desc, void *data, bool *val);
+static int Config_update_maxFPS(int widget_desc, void *data, int *val);
 
 static int Config_close(int widget_desc, void *data, char **strptr);
 static int Config_next(int widget_desc, void *data, char **strptr);
@@ -203,6 +212,8 @@ static int		(*config_creator[])(int widget_desc, int *height) = {
     Config_create_fuelCritical,
     Config_create_fuelGauge,
     Config_create_outlineWorld,
+    Config_create_filledWorld,
+    Config_create_texturedWalls,
     Config_create_slidingRadar,
     Config_create_showItems,
     Config_create_showItemsTime,
@@ -215,6 +226,10 @@ static int		(*config_creator[])(int widget_desc, int *height) = {
     Config_create_toggleShield,
     Config_create_shotSize,
     Config_create_teamShotSize,
+    Config_create_hudColor,
+    Config_create_hudLockColor,
+    Config_create_wallColor,
+    Config_create_maxFPS,
 #ifdef SOUND
     Config_create_maxVolume,
 #endif
@@ -593,10 +608,10 @@ static int Config_create_altTurnResistance(int widget_desc, int *height)
 static int Config_create_showMessages(int widget_desc, int *height)
 {
     return Config_create_bool(widget_desc, height, "showMessages",
-                            BIT(instruments, SHOW_MESSAGES)
-                                ? true : false,
-                            Config_update_instruments,
-                            (void *) SHOW_MESSAGES);
+			    BIT(instruments, SHOW_MESSAGES)
+				? true : false,
+			    Config_update_instruments,
+			    (void *) SHOW_MESSAGES);
 }
 
 static int Config_create_showHUD(int widget_desc, int *height)
@@ -679,6 +694,24 @@ static int Config_create_outlineWorld(int widget_desc, int *height)
 			      (void *) SHOW_OUTLINE_WORLD);
 }
 
+static int Config_create_filledWorld(int widget_desc, int *height)
+{
+    return Config_create_bool(widget_desc, height, "filledWorld",
+			      BIT(instruments, SHOW_FILLED_WORLD)
+				  ? true : false,
+			      Config_update_instruments,
+			      (void *) SHOW_FILLED_WORLD);
+}
+
+static int Config_create_texturedWalls(int widget_desc, int *height)
+{
+    return Config_create_bool(widget_desc, height, "texturedWalls",
+			      BIT(instruments, SHOW_TEXTURED_WALLS)
+				  ? true : false,
+			      Config_update_instruments,
+			      (void *) SHOW_TEXTURED_WALLS);
+}
+
 static int Config_create_slidingRadar(int widget_desc, int *height)
 {
     if (Client_wrap_mode() == 0) {
@@ -701,19 +734,19 @@ static int Config_create_backgroundPointDist(int widget_desc, int *height)
 static int Config_create_showItems(int widget_desc, int *height)
 {
     return Config_create_bool(widget_desc, height, "showItems",
-                            BIT(instruments, SHOW_ITEMS)
-                                ? true : false,
-                            Config_update_instruments,
-                            (void *) SHOW_ITEMS);
+			    BIT(instruments, SHOW_ITEMS)
+				? true : false,
+			    Config_update_instruments,
+			    (void *) SHOW_ITEMS);
 }
 
 static int Config_create_showItemsTime(int widget_desc, int *height)
 {
     return Config_create_float(widget_desc, height,
-                             "showItemsTime", &showItemsTime,
-                             MIN_SHOW_ITEMS_TIME,
-                             MAX_SHOW_ITEMS_TIME,
-                             NULL, NULL);
+			     "showItemsTime", &showItemsTime,
+			     MIN_SHOW_ITEMS_TIME,
+			     MAX_SHOW_ITEMS_TIME,
+			     NULL, NULL);
 }
 
 static int Config_create_backgroundPointSize(int widget_desc, int *height)
@@ -758,17 +791,41 @@ static int Config_create_toggleShield(int widget_desc, int *height)
 static int Config_create_shotSize(int widget_desc, int *height)
 {
     return Config_create_int(widget_desc, height,
-                           "shotSize", &shot_size,
-                           MIN_SHOT_SIZE, MAX_SHOT_SIZE,
-                           NULL, NULL);
+			   "shotSize", &shot_size,
+			   MIN_SHOT_SIZE, MAX_SHOT_SIZE,
+			   NULL, NULL);
 }
 
 static int Config_create_teamShotSize(int widget_desc, int *height)
 {
     return Config_create_int(widget_desc, height,
-                           "teamShotSize", &teamshot_size,
-                           MIN_TEAMSHOT_SIZE, MAX_TEAMSHOT_SIZE,
-                           NULL, NULL);
+			   "teamShotSize", &teamshot_size,
+			   MIN_TEAMSHOT_SIZE, MAX_TEAMSHOT_SIZE,
+			   NULL, NULL);
+}
+
+static int Config_create_hudColor(int widget_desc, int *height)
+{
+    return Config_create_int(widget_desc, height,
+			   "hudColor", &hudColor,
+			   1, maxColors - 1,
+			   NULL, NULL);
+}
+
+static int Config_create_hudLockColor(int widget_desc, int *height)
+{
+    return Config_create_int(widget_desc, height,
+			   "hudLockColor", &hudLockColor,
+			   1, maxColors - 1,
+			   NULL, NULL);
+}
+
+static int Config_create_wallColor(int widget_desc, int *height)
+{
+    return Config_create_int(widget_desc, height,
+			   "wallColor", &wallColor,
+			   1, maxColors - 1,
+			   NULL, NULL);
 }
 
 #ifdef SOUND
@@ -780,6 +837,13 @@ static int Config_create_maxVolume(int widget_desc, int *height)
 }
 #endif
 
+static int Config_create_maxFPS(int widget_desc, int *height)
+{
+    return Config_create_int(widget_desc, height,
+			     "maxFPS", &maxFPS, 0, 30,
+			     Config_update_maxFPS, NULL);
+}
+
 static int Config_create_showShipName(int widget_desc, int *height)
 {
     return Config_create_bool(widget_desc, height, "showShipName",
@@ -867,7 +931,7 @@ static int Config_create_markingLights(int widget_desc, int *height)
 			      markingLights,
 			      Config_update_bool, &markingLights);
 }
-    
+
 
 static int Config_create_save(int widget_desc, int *height)
 {
@@ -911,21 +975,32 @@ static int Config_update_bool(int widget_desc, void *data, bool *val)
 
 static int Config_update_instruments(int widget_desc, void *data, bool *val)
 {
+    long		old_instruments = instruments;
+    long		bit = (long) data;
+    long		outline_mask = SHOW_OUTLINE_WORLD
+				     | SHOW_FILLED_WORLD
+				     | SHOW_TEXTURED_WALLS;
+
     if (*val == false) {
-	CLR_BIT(instruments, (long)data);
+	CLR_BIT(instruments, bit);
     } else {
-	SET_BIT(instruments, (long)data);
+	SET_BIT(instruments, bit);
     }
-    if ((long)data == SHOW_SLIDING_RADAR) {
+    if (bit == SHOW_SLIDING_RADAR) {
 	Paint_sliding_radar();
     }
-    if ((long)data == SHOW_OUTLINE_WORLD) {
-	Map_blue();
+    if (BIT(bit, outline_mask)) {
+	/* only do the map recalculations if really needed. */
+	if (!BIT(old_instruments, outline_mask)
+	     != !BIT(instruments, outline_mask)) {
+	    Map_restore(0, 0, Setup->x, Setup->y);
+	    Map_blue(0, 0, Setup->x, Setup->y);
+	}
     }
-    if ((long)data == SHOW_PACKET_DROP_METER
-	|| (long)data == SHOW_PACKET_LOSS_METER) {
+    if (BIT(bit, SHOW_PACKET_DROP_METER | SHOW_PACKET_LOSS_METER)) {
 	Net_init_measurement();
     }
+
     return 0;
 }
 
@@ -996,6 +1071,12 @@ static int Config_update_toggleShield(int widget_desc, void *data, bool *val)
     return 0;
 }
 
+static int Config_update_maxFPS(int widget_desc, void *data, int *val)
+{
+    Check_client_fps();
+    return 0;
+}
+
 static void Config_save_failed(char *reason, char **strptr)
 {
     if (config_save_confirm_desc != NO_WIDGET) {
@@ -1058,7 +1139,11 @@ static void Xpilotrc_end(FILE *fp)
 	return;
     }
     for (i = 0; i < num_xpilotrc; i++) {
-	fprintf(fp, "%s", xpilotrc_ptr[i].line);
+	/* a bug in 3.2.8 saved maxFPS, which is wrong!  don't save maxFPS! */
+	if (strncmp(xpilotrc_ptr[i].line + 7, "maxFPS:",
+		    xpilotrc_ptr[i].size + 1) != 0) {
+	    fprintf(fp, "%s", xpilotrc_ptr[i].line);
+	}
 	free(xpilotrc_ptr[i].line);
     }
     free(xpilotrc_ptr);
@@ -1129,16 +1214,16 @@ static int Config_save(int widget_desc, void *button_str, char **strptr)
 			*res,
 			*base = ".xpilotrc",
 			buf[512],
-			oldfile[PATH_MAX],
-			newfile[PATH_MAX];
+			oldfile[PATH_MAX + 1],
+			newfile[PATH_MAX + 1];
 
     *strptr = "Saving...";
     Widget_draw(widget_desc);
     Client_flush();
 
-    if ((home = getenv("HOME")) == NULL
+    if (((home = getenv("HOME")) == NULL
 	&& ((pwent = getpwuid(getuid())) == NULL
-	|| (home = pwent->pw_dir)[0] == '\0')
+	    || (home = pwent->pw_dir)[0] == '\0'))
 	|| access(home, 0) == -1) {
 	Config_save_failed("Can't find home directory.", strptr);
 	return 1;
@@ -1186,6 +1271,7 @@ static int Config_save(int widget_desc, void *button_str, char **strptr)
     Config_save_bool(fp, "showItems", BIT(instruments, SHOW_ITEMS));
     Config_save_float(fp, "showItemsTime", showItemsTime);
     Config_save_bool(fp, "outlineWorld", BIT(instruments, SHOW_OUTLINE_WORLD));
+    Config_save_bool(fp, "filledWorld", BIT(instruments, SHOW_FILLED_WORLD));
     Config_save_bool(fp, "clock", BIT(instruments, SHOW_CLOCK));
     Config_save_int(fp, "backgroundPointDist", map_point_distance);
     Config_save_int(fp, "backgroundPointSize", map_point_size);
@@ -1193,6 +1279,9 @@ static int Config_save(int widget_desc, void *button_str, char **strptr)
     Config_save_float(fp, "sparkProb", spark_prob);
     Config_save_int(fp, "shotSize", shot_size);
     Config_save_int(fp, "teamShotSize", teamshot_size);
+    Config_save_int(fp, "hudColor", hudColor);
+    Config_save_int(fp, "hudLockColor", hudLockColor);
+    Config_save_int(fp, "wallColor", wallColor);
     Config_save_int(fp, "receiveWindowSize", receive_window_size);
     Config_save_int(fp, "charsPerSecond", charsPerSecond);
     Config_save_bool(fp, "markingLights", markingLights);
@@ -1200,6 +1289,7 @@ static int Config_save(int widget_desc, void *button_str, char **strptr)
 #if SOUND
     Config_save_int(fp, "maxVolume", maxVolume);
 #endif
+    /* don't save this one: Config_save_int(fp, "maxFPS", maxFPS); */
     buf[0] = '\0';
     for (i = 0, prev_key = KEY_DUMMY; i < maxKeyDefs; i++, prev_key = key) {
 	ks = keyDefs[i].keysym;
@@ -1296,7 +1386,7 @@ void Config_destroy(void)
 
 void Config_resize(void)
 {
-    bool		mapped = config_created;
+    bool		mapped = config_mapped;
 
     if (config_created == true) {
 	Config_destroy();
