@@ -1,6 +1,6 @@
-/* $Id: event.c,v 3.43 1994/04/14 08:13:15 bert Exp $
+/* $Id: event.c,v 3.57 1995/01/11 19:28:38 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -23,18 +23,24 @@
 
 #define SERVER
 #include <stdlib.h>
+#include <stdio.h>
+
+#include "version.h"
+#include "config.h"
+#include "const.h"
 #include "global.h"
+#include "proto.h"
 #include "score.h"
 #include "map.h"
-#include "robot.h"
-#include "keys.h"
 #include "saudio.h"
 #include "bit.h"
 #include "netserver.h"
 
+char event_version[] = VERSION;
+
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: event.c,v 3.43 1994/04/14 08:13:15 bert Exp $";
+    "@(#)$Id: event.c,v 3.57 1995/01/11 19:28:38 bert Exp $";
 #endif
 
 #define SWAP(_a, _b)	    {float _tmp = _a; _a = _b; _b = _tmp;}
@@ -58,13 +64,16 @@ static void Refuel(int ind)
 
     CLR_BIT(pl->used, OBJ_REFUEL);
     for (i=0; i<World.NumFuels; i++) {
-	l = Wrap_length(pl->pos.x - World.fuel[i].pos.x, 
-			pl->pos.y - World.fuel[i].pos.y);
-	if (BIT(pl->used, OBJ_REFUEL) == 0
-	    || l < dist) {
-	    SET_BIT(pl->used, OBJ_REFUEL);
-	    pl->fs = i;
-	    dist = l;
+	if (World.block[World.fuel[i].blk_pos.x]
+		       [World.fuel[i].blk_pos.y] == FUEL) {
+	    l = Wrap_length(pl->pos.x - World.fuel[i].pix_pos.x,
+			    pl->pos.y - World.fuel[i].pix_pos.y);
+	    if (BIT(pl->used, OBJ_REFUEL) == 0
+		|| l < dist) {
+		SET_BIT(pl->used, OBJ_REFUEL);
+		pl->fs = i;
+		dist = l;
+	    }
 	}
     }
 }
@@ -105,7 +114,7 @@ int Player_lock_closest(int ind, int next)
 
     if (!next)
 	CLR_BIT(pl->lock.tagged, LOCK_PLAYER);
-	
+
     if (BIT(pl->lock.tagged, LOCK_PLAYER)) {
 	lock = GetInd[pl->lock.pl_id];
 	dist = Wrap_length(Players[lock]->pos.x - pl->pos.x,
@@ -133,11 +142,12 @@ int Player_lock_closest(int ind, int next)
     pl->lock.pl_id = Players[newpl]->id;
     return 1;
 }
-	
+
 
 void Pause_player(int ind, int onoff)
 {
     player		*pl = Players[ind];
+    int			i;
 
     if (onoff) {
 	if (!BIT(pl->status, PAUSE)) { /* Turn pause mode on */
@@ -150,16 +160,42 @@ void Pause_player(int ind, int onoff)
 	}
     } else {
 	if (pl->count <= 0) {
+	    bool toolate = false;
+
 	    CLR_BIT(pl->status, PAUSE);
 	    updateScores = true;
-	    if (!BIT(pl->mode, LIMITED_LIVES)) {
-		pl->mychar = ' ';
-		Go_home(ind);
-		SET_BIT(pl->status, PLAYING);
-	    } else {
+	    if (BIT(pl->mode, LIMITED_LIVES)) {
+		for (i = 0; i < NumPlayers; i++) {
+		    /* If a non-team member has lost a life,
+		     * then it's too late to join. */
+		    if (i == ind) {
+			continue;
+		    }
+		    if (Players[i]->life < World.rules->lives && !TEAM(ind, i)) {
+			toolate = true;
+			break;
+		    }
+		}
+	    }
+	    if (toolate) {
 		pl->life = 0;
 		pl->mychar = 'W';
 		SET_BIT(pl->status, GAME_OVER);
+	    } else {
+		pl->mychar = ' ';
+		Go_home(ind);
+		SET_BIT(pl->status, PLAYING);
+		if (BIT(pl->mode, LIMITED_LIVES)) {
+		    pl->life = World.rules->lives;
+		}
+	    }
+	    if (BIT(World.rules->mode, TIMING)) {
+		pl->round = 0;
+		pl->check = 0;
+		pl->time = 0;
+		pl->best_lap = 0;
+		pl->last_lap = 0;
+		pl->last_lap_time = 0;
 	    }
 	}
     }
@@ -200,7 +236,6 @@ int Handle_keyboard(int ind)
 	    case KEY_LOCK_PREV:
 	    case KEY_LOCK_CLOSE:
 	    case KEY_LOCK_NEXT_CLOSE:
-	    case KEY_ID_MODE:
 	    case KEY_TOGGLE_NUCLEAR:
 	    case KEY_TOGGLE_CLUSTER:
 	    case KEY_TOGGLE_IMPLOSION:
@@ -210,13 +245,16 @@ int Handle_keyboard(int ind)
 	    case KEY_TOGGLE_POWER:
 	    case KEY_TOGGLE_LASER:
 	    case KEY_TOGGLE_COMPASS:
-	    case KEY_TOGGLE_OWNED_ITEMS:
-	    case KEY_TOGGLE_MESSAGES:
 	    case KEY_CLEAR_MODIFIERS:
 	    case KEY_LOAD_MODIFIERS_1:
 	    case KEY_LOAD_MODIFIERS_2:
 	    case KEY_LOAD_MODIFIERS_3:
 	    case KEY_LOAD_MODIFIERS_4:
+	    case KEY_LOAD_LOCK_1:
+	    case KEY_LOAD_LOCK_2:
+	    case KEY_LOAD_LOCK_3:
+	    case KEY_LOAD_LOCK_4:
+	    case KEY_REPROGRAM:
 	    case KEY_SWAP_SETTINGS:
 	    case KEY_INCREASE_POWER:
 	    case KEY_DECREASE_POWER:
@@ -253,20 +291,32 @@ int Handle_keyboard(int ind)
 	    case KEY_LOCK_NEXT:
 	    case KEY_LOCK_PREV:
 		j = i = GetInd[pl->lock.pl_id];
+		if (!BIT(pl->lock.tagged, LOCK_PLAYER)
+		    || j < 0 || j >= NumPlayers) {
+		    /* better jump to KEY_LOCK_CLOSEST.... */
+		    (void) Player_lock_closest(ind, 0);
+		    break;
+		}
 		do {
-		    if (key == KEY_LOCK_PREV)
-			i--;
-		    else
-			i++;
-		    i = mod(i, NumPlayers);
+		    if (key == KEY_LOCK_PREV) {
+			if (--i < 0) {
+			    i = NumPlayers - 1;
+			}
+		    } else {
+			if (++i >= NumPlayers) {
+			    i = 0;
+			}
+		    }
 		    if (i == j)
 			break;
 		} while (i == ind || BIT(Players[i]->status, GAME_OVER));
-		pl->lock.pl_id = Players[i]->id;
-		if (i == ind)
+		if (i == ind) {
 		    CLR_BIT(pl->lock.tagged, LOCK_PLAYER);
-		else
+		}
+		else {
+		    pl->lock.pl_id = Players[i]->id;
 		    SET_BIT(pl->lock.tagged, LOCK_PLAYER);
+		}
 		break;
 
 	    case KEY_TOGGLE_COMPASS:
@@ -323,7 +373,7 @@ int Handle_keyboard(int ind)
 		    }
 		    for (i=0; i<NumPlayers; i++)
 			if (i != ind
-			    && Players[i]->robot_mode != RM_OBJECT
+			    && !IS_TANK_IND(i)
 			    && pl->home_base == Players[i]->home_base) {
 			    Pick_startpos(i);
 			    sprintf(msg, "%s has taken over %s's home base.",
@@ -336,7 +386,7 @@ int Handle_keyboard(int ind)
 		    for (i = 0; i < NumPlayers; i++) {
 			if (Players[i]->conn != NOT_CONNECTED) {
 			    Send_base(Players[i]->conn,
-				      pl->id, 
+				      pl->id,
 				      pl->home_base);
 			}
 		    }
@@ -356,31 +406,31 @@ int Handle_keyboard(int ind)
 		break;
 
 	    case KEY_FIRE_SHOT:
-		if (!BIT(pl->used, OBJ_SHIELD|OBJ_FIRE)
-		    && BIT(pl->have, OBJ_FIRE)) {
+		if (!BIT(pl->used, OBJ_SHIELD|OBJ_SHOT)
+		    && BIT(pl->have, OBJ_SHOT)) {
 		    Fire_normal_shots(ind);
 		}
-		SET_BIT(pl->used, OBJ_FIRE);
+		SET_BIT(pl->used, OBJ_SHOT);
 		break;
 
 	    case KEY_FIRE_MISSILE:
-		if (pl->missiles > 0)
+		if (pl->item[ITEM_MISSILE] > 0)
 		    Fire_shot(ind, OBJ_SMART_SHOT, pl->dir);
 		break;
 
 	    case KEY_FIRE_HEAT:
-		if (pl->missiles > 0)
+		if (pl->item[ITEM_MISSILE] > 0)
 		    Fire_shot(ind, OBJ_HEAT_SHOT, pl->dir);
 		break;
 
 	    case KEY_FIRE_TORPEDO:
-		if (pl->missiles > 0)
+		if (pl->item[ITEM_MISSILE] > 0)
 		    Fire_shot(ind, OBJ_TORPEDO, pl->dir);
 
 		break;
 
 	    case KEY_FIRE_LASER:
-		if (pl->lasers > 0 && BIT(pl->used, OBJ_SHIELD) == 0) {
+		if (pl->item[ITEM_LASER] > 0 && BIT(pl->used, OBJ_SHIELD) == 0) {
 		    SET_BIT(pl->used, OBJ_LASER);
 		}
 		break;
@@ -467,12 +517,38 @@ int Handle_keyboard(int ind)
 		CLEAR_MODS(pl->mods);
 		break;
 
+	    case KEY_REPROGRAM:
+		SET_BIT(pl->status, REPROGRAM);
+		break;
+
 	    case KEY_LOAD_MODIFIERS_1:
 	    case KEY_LOAD_MODIFIERS_2:
 	    case KEY_LOAD_MODIFIERS_3:
-	    case KEY_LOAD_MODIFIERS_4:
-		pl->mods = pl->modbank[key - KEY_LOAD_MODIFIERS_1];
+	    case KEY_LOAD_MODIFIERS_4: {
+		modifiers *m = &(pl->modbank[key - KEY_LOAD_MODIFIERS_1]);
+
+		if (BIT(pl->status, REPROGRAM))
+		    *m = pl->mods;
+		else
+		    pl->mods = *m;
 		break;
+	    }
+
+	    case KEY_LOAD_LOCK_1:
+	    case KEY_LOAD_LOCK_2:
+	    case KEY_LOAD_LOCK_3:
+	    case KEY_LOAD_LOCK_4: {
+		int *l = &(pl->lockbank[key - KEY_LOAD_LOCK_1]);
+
+		if (BIT(pl->status, REPROGRAM)) {
+		    if (BIT(pl->lock.tagged, LOCK_PLAYER))
+			*l = pl->lock.pl_id;
+		} else if (*l != NOT_CONNECTED) {
+		    pl->lock.pl_id = *l;
+		    SET_BIT(pl->lock.tagged, LOCK_PLAYER);
+		}
+		break;
+	    }
 
 	    case KEY_TOGGLE_AUTOPILOT:
 		if (BIT(pl->have, OBJ_AUTOPILOT))
@@ -484,6 +560,11 @@ int Handle_keyboard(int ind)
 		    Emergency_thrust(ind,!BIT(pl->used, OBJ_EMERGENCY_THRUST));
 		break;
 
+	    case KEY_EMERGENCY_SHIELD:
+		if (BIT(pl->have, OBJ_EMERGENCY_SHIELD))
+		    Emergency_shield(ind,!BIT(pl->used, OBJ_EMERGENCY_SHIELD));
+		break;
+
 	    case KEY_DROP_MINE:
 		Place_mine(ind);
 		break;
@@ -492,6 +573,10 @@ int Handle_keyboard(int ind)
 		Place_moving_mine(ind);
 		break;
 
+	    case KEY_DETONATE_MINES:
+		Detonate_mines(ind);
+		break;
+
 	    case KEY_TURN_LEFT:
 	    case KEY_TURN_RIGHT:
 		if (BIT(pl->used, OBJ_AUTOPILOT))
@@ -512,27 +597,35 @@ int Handle_keyboard(int ind)
 		break;
 
 	    case KEY_PAUSE:
-		xi = (int)pl->pos.x / BLOCK_SZ;
-		yi = (int)pl->pos.y / BLOCK_SZ;
-		j = World.base[pl->home_base].pos.x;
-		k = World.base[pl->home_base].pos.y;
-		if (j == xi && k == yi) {
-		    minv = 1.0f;
+		if (BIT(pl->status, PAUSE)) {
 		    i = PAUSE;
-		} else {
-		    /*
-		     * Hover pause doesn't work within two squares of the
-		     * players home base, they would want the better pause.
-		     */
-		    if (ABS(j - xi) <= 2 && ABS(k - yi) <= 2)
-			break;
-		    minv = 5.0f;
+		}
+		else if (BIT(pl->status, HOVERPAUSE)) {
 		    i = HOVERPAUSE;
 		}
-		minv += LENGTH(World.gravity[xi][yi].x,
-			       World.gravity[xi][yi].y);
-		if (pl->velocity > minv)
-		    break;
+		else {
+		    xi = (int)pl->pos.x / BLOCK_SZ;
+		    yi = (int)pl->pos.y / BLOCK_SZ;
+		    j = World.base[pl->home_base].pos.x;
+		    k = World.base[pl->home_base].pos.y;
+		    if (j == xi && k == yi) {
+			minv = 1.0f;
+			i = PAUSE;
+		    } else {
+			/*
+			 * Hover pause doesn't work within two squares of the
+			 * players home base, they would want the better pause.
+			 */
+			if (ABS(j - xi) <= 2 && ABS(k - yi) <= 2)
+			    break;
+			minv = 5.0f;
+			i = HOVERPAUSE;
+		    }
+		    minv += LENGTH(World.gravity[xi][yi].x,
+				   World.gravity[xi][yi].y);
+		    if (pl->velocity > minv)
+			break;
+		}
 
 		switch (i) {
 		case PAUSE:
@@ -566,6 +659,9 @@ int Handle_keyboard(int ind)
 			if (BIT(pl->used, OBJ_EMERGENCY_THRUST))
 			    Emergency_thrust(ind, 0);
 
+			if (BIT(pl->used, OBJ_EMERGENCY_SHIELD))
+			    Emergency_shield(ind, 0);
+
 			if (!BIT(pl->used, OBJ_AUTOPILOT))
 			    Autopilot(ind, 1);
 
@@ -575,19 +671,20 @@ int Handle_keyboard(int ind)
 			 * tractor beams.  Other items are allowed (esp.
 			 * cloaking).
 			 */
-			CLR_BIT(pl->used, OBJ_FIRE|OBJ_CONNECTOR
-				|OBJ_REFUEL|OBJ_TRACTOR_BEAM|OBJ_LASER);
+			pl->used &= ~USED_KILL;
 			if (BIT(pl->have, OBJ_SHIELD))
 			    SET_BIT(pl->used, OBJ_SHIELD);
 		    } else if (pl->count <= 0) {
 			Autopilot(ind, 0);
 			CLR_BIT(pl->status, HOVERPAUSE);
-			CLR_BIT(pl->used, OBJ_SHIELD);
+			if (!BIT(pl->have, OBJ_SHIELD)) {
+			    CLR_BIT(pl->used, OBJ_SHIELD);
+			}
 		    }
 		    break;
 		}
 		break;
-		
+
 	    case KEY_SWAP_SETTINGS:
 		if (   BIT(pl->status, HOVERPAUSE)
 		    || BIT(pl->used, OBJ_AUTOPILOT))
@@ -667,7 +764,7 @@ int Handle_keyboard(int ind)
 		break;
 
 	    case KEY_CLOAK:
-		if (pl->cloaks > 0) {
+		if (pl->item[ITEM_CLOAK] > 0) {
 		    sound_play_player(pl, CLOAK_SOUND);
 		    pl->updateVisibility = 1;
 		    TOGGLE_BIT(pl->used, OBJ_CLOAKING_DEVICE);
@@ -679,9 +776,9 @@ int Handle_keyboard(int ind)
 		break;
 
 	    case KEY_TRANSPORTER:
-		if (pl->transporters > 0 && pl->fuel.sum > -ED_TRANSPORTER) {
+		if (pl->item[ITEM_TRANSPORTER] > 0 && pl->fuel.sum > -ED_TRANSPORTER) {
 		    do_transporter(pl);
-		    pl->transporters--;
+		    pl->item[ITEM_TRANSPORTER]--;
 		    Add_fuel(&(pl->fuel), ED_TRANSPORTER);
 		    }
 		break;
@@ -727,7 +824,7 @@ int Handle_keyboard(int ind)
 		break;
 
 	    case KEY_FIRE_SHOT:
-		CLR_BIT(pl->used, OBJ_FIRE);
+		CLR_BIT(pl->used, OBJ_SHOT);
 		break;
 
 	    case KEY_FIRE_LASER:
@@ -740,6 +837,10 @@ int Handle_keyboard(int ind)
 		CLR_BIT(pl->status, THRUSTING);
 		break;
 
+	    case KEY_REPROGRAM:
+		CLR_BIT(pl->status, REPROGRAM);
+		break;
+
 	    default:
 		break;
 	    }
