@@ -1,6 +1,6 @@
-/* $Id: frame.c,v 3.45 1994/03/30 19:41:58 bert Exp $
+/* $Id: frame.c,v 3.59 1995/01/31 18:26:34 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -37,20 +37,27 @@
 #include <time.h>
 
 #define SERVER
-#include "global.h"
 #include "version.h"
+#include "config.h"
+#include "const.h"
+#include "global.h"
+#include "proto.h"
 #include "bit.h"
 #include "netserver.h"
 #include "saudio.h"
 
+char frame_version[] = VERSION;
+
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: frame.c,v 3.45 1994/03/30 19:41:58 bert Exp $";
+    "@(#)$Id: frame.c,v 3.59 1995/01/31 18:26:34 bert Exp $";
 #endif
 
 
 /*
  * Structure for calculating if a pixel is visible by a player.
+ * The following always holds:
+ *	(world.x >= realWorld.x && world.y >= realWorld.y)
  */
 typedef struct {
     position	world;			/* Lower left hand corner is this */
@@ -58,8 +65,6 @@ typedef struct {
     position	realWorld;		/* If the player is on the edge of
 					   the screen, these are the world
 					   coordinates before adjustment... */
-    int		wrappedWorld;		/* Nonzero if we're near the edge
-					   of the world (realWorld != world) */
 } pixel_visibility_t;
 
 /*
@@ -69,7 +74,6 @@ typedef struct {
 typedef struct {
     ipos		world;
     ipos		realWorld;
-    int			wrappedWorld;
 } block_visibility_t;
 
 typedef struct {
@@ -98,48 +102,18 @@ static int		fastshot_num[DEBRIS_TYPES * 2],
 
 static int inview(float x, float y)
 {
-    if (x > pv.world.x
-	&& x < pv.world.x + view_width
-	&& y > pv.world.y
-	&& y < pv.world.y + view_height) {
-	return 1;
-    }
-    if (!pv.wrappedWorld) {
-	return 0;
-    }
-    if ((pv.wrappedWorld & 1) && x > view_width) {
-	x -= World.width;
-    }
-    if ((pv.wrappedWorld & 2) && y > view_height) {
-	y -= World.height;
-    }
-    return (x > pv.realWorld.x
-	&& x < pv.realWorld.x + view_width
-	&& y > pv.realWorld.y
-	&& y < pv.realWorld.y + view_height);
+    return ((x > pv.world.x && x < pv.world.x + view_width)
+	    || (x > pv.realWorld.x && x < pv.realWorld.x + view_width))
+	&& ((y > pv.world.y && y < pv.world.y + view_height)
+	    || (y > pv.realWorld.y && y < pv.realWorld.y + view_height));
 }
 
-static int block_inview(block_visibility_t *pl, int x, int y)
+static int block_inview(block_visibility_t *bv, int x, int y)
 {
-    if (x >= pl->world.x
-	&& x <= pl->world.x + horizontal_blocks
-	&& y >= pl->world.y
-	&& y <= pl->world.y + vertical_blocks) {
-	return 1;
-    }
-    if (!pl->wrappedWorld) {
-	return 0;
-    }
-    if ((pl->wrappedWorld & 1) && x > horizontal_blocks) {
-	x -= World.x;
-    }
-    if ((pl->wrappedWorld & 2) && y > vertical_blocks) {
-	y -= World.y;
-    }
-    return (x >= pl->realWorld.x
-	&& x <= pl->realWorld.x + horizontal_blocks
-	&& y >= pl->realWorld.y
-	&& y <= pl->realWorld.y + vertical_blocks);
+    return ((x > bv->world.x && x < bv->world.x + horizontal_blocks)
+	    || (x > bv->realWorld.x && x < bv->realWorld.x + horizontal_blocks))
+	&& ((y > bv->world.y && y < bv->world.y + vertical_blocks)
+	    || (y > bv->realWorld.y && y < bv->realWorld.y + vertical_blocks));
 }
 
 #define DEBRIS_STORE(xf,yf,color,offset) \
@@ -269,8 +243,8 @@ static int Frame_status(int conn, int ind)
 			lock_id = -1,
 			lock_dist = 0,
 			lock_dir = 0,
-    			i,
-    			showautopilot;
+			i,
+			showautopilot;
 
     /*
      * Don't make lock visible during this frame if;
@@ -297,8 +271,8 @@ static int Frame_status(int conn, int ind)
 		|| inview(Players[lock_ind]->pos.x, Players[lock_ind]->pos.y))
 	    && pl->lock.distance != 0) {
 	    SET_BIT(pl->lock.tagged, LOCK_VISIBLE);
-	    lock_dir = Wrap_findDir(Players[lock_ind]->pos.x - pl->pos.x,
-				    Players[lock_ind]->pos.y - pl->pos.y);
+	    lock_dir = Wrap_findDir((int)(Players[lock_ind]->pos.x - pl->pos.x),
+				    (int)(Players[lock_ind]->pos.y - pl->pos.y));
 	    lock_dist = pl->lock.distance;
 	}
     }
@@ -310,41 +284,6 @@ static int Frame_status(int conn, int ind)
     else
 	showautopilot = 0;
 
-    n = Send_self(conn,
-	(int) (pl->pos.x + 0.5),
-	(int) (pl->pos.y + 0.5),
-	(int) pl->vel.x,
-	(int) pl->vel.y,
-	pl->dir,
-	pl->power,
-	pl->turnspeed,
-	pl->turnresistance,
-	lock_id,
-	lock_dist,
-	lock_dir,
-	pl->check,
-	pl->cloaks,
-	pl->sensors,
-	pl->mines,
-	pl->missiles,
-	pl->ecms,
-	pl->transporters,
-	pl->extra_shots,
-	pl->back_shots,
-	pl->afterburners,
-	pl->lasers,
-	pl->emergency_thrusts,
-	pl->tractor_beams,
-	pl->autopilots,
-	showautopilot,
-	pl->fuel.num_tanks,
-	pl->fuel.current,
-	pl->fuel.sum,
-	pl->fuel.max, Players[GetInd[Get_player_id(conn)]]->status);
-    if (n <= 0) {
-	return 0;
-    }
-
     /*
      * Don't forget to modify Receive_modifier_bank() in netserver.c
      */
@@ -383,12 +322,26 @@ static int Frame_status(int conn, int ind)
 	mods[i++] = (BIT(pl->mods.laser, STUN) ? 'S' : 'B');
     }
     mods[i] = '\0';
-    Send_modifiers(conn, mods);
+    n = Send_self(conn,
+		  pl,
+		  lock_id,
+		  lock_dist,
+		  lock_dir,
+		  showautopilot,
+		  Players[GetInd[Get_player_id(conn)]]->status,
+		  mods);
+    if (n <= 0) {
+	return 0;
+    }
 
     if (BIT(pl->used, OBJ_EMERGENCY_THRUST))
 	Send_thrusttime(conn,
 			pl->emergency_thrust_left,
 			pl->emergency_thrust_max);
+    if (BIT(pl->used, OBJ_EMERGENCY_SHIELD))
+	Send_shieldtime(conn,
+			pl->emergency_shield_left,
+			pl->emergency_shield_max);
     if (BIT(pl->status, SELF_DESTRUCT) && pl->count > 0) {
 	Send_destruct(conn, pl->count);
     }
@@ -411,31 +364,31 @@ static void Frame_map(int conn, int ind)
     y = pl->pos.y / BLOCK_SZ;
     bv.world.x = x - (horizontal_blocks >> 1);
     bv.world.y = y - (vertical_blocks >> 1);
-    bv.wrappedWorld = 0;
+    bv.realWorld = bv.world;
     if (BIT(World.rules->mode, WRAP_PLAY)) {
-	bv.realWorld = bv.world;
-	if (bv.world.x < 0) {
-	    bv.wrappedWorld |= 1;
+	if (bv.world.x < 0 && bv.world.x + horizontal_blocks < World.x) {
 	    bv.world.x += World.x;
-	} else if (bv.world.x + horizontal_blocks > World.x) {
-	    bv.realWorld.x -= World.x;
-	    bv.wrappedWorld |= 1;
 	}
-	if (bv.world.y < 0) {
-	    bv.wrappedWorld |= 2;
+	else if (bv.world.x > 0 && bv.world.x + horizontal_blocks > World.x) {
+	    bv.realWorld.x -= World.x;
+	}
+	if (bv.world.y < 0 && bv.world.y + vertical_blocks < World.y) {
 	    bv.world.y += World.y;
-	} else if (bv.world.y + vertical_blocks > World.y) {
+	}
+	else if (bv.world.y > 0 && bv.world.y + vertical_blocks > World.y) {
 	    bv.realWorld.y -= World.y;
-	    bv.wrappedWorld |= 2;
 	}
     }
 
     for (i = 0; i < World.NumFuels; i++) {
 	if (BIT(World.fuel[i].conn_mask, conn_bit) == 0) {
-	    if (block_inview(&bv,
-			     World.fuel[i].pos.x / BLOCK_SZ,
-			     World.fuel[i].pos.y / BLOCK_SZ)) {
-		Send_fuel(conn, i, (int) World.fuel[i].fuel);
+	    if (World.block[World.fuel[i].blk_pos.x]
+			   [World.fuel[i].blk_pos.y] == FUEL) {
+		if (block_inview(&bv,
+				 World.fuel[i].blk_pos.x,
+				 World.fuel[i].blk_pos.y)) {
+		    Send_fuel(conn, i, (int) World.fuel[i].fuel);
+		}
 	    }
 	}
     }
@@ -454,13 +407,12 @@ static void Frame_map(int conn, int ind)
     }
 
     for (i = 0; i < World.NumTargets; i++) {
-	if (BIT(World.targets[i].conn_mask, conn_bit) == 0) {
-	    if (block_inview(&bv,
-			     World.targets[i].pos.x,
-			     World.targets[i].pos.y)) {
-		Send_target(conn, i, World.targets[i].dead_time,
-			    World.targets[i].damage);
-	    }
+	target_t *targ = &World.targets[i];
+
+	if (BIT(targ->update_mask, conn_bit)
+	    || (BIT(targ->conn_mask, conn_bit) == 0
+		&& block_inview(&bv, targ->pos.x, targ->pos.y))) {
+	    Send_target(conn, i, targ->dead_time, targ->damage);
 	}
     }
 }
@@ -495,6 +447,13 @@ static void Frame_shots(int conn, int ind)
 		 */
 		break;
 	    }
+	    /*
+	     * The number of colors which the client
+	     * uses for displaying debris is bigger than 2
+	     * then the color used denotes the temperature
+	     * of the debris particles.
+	     * Higher color number means hotter debris.
+	     */
 	    if (debris_colors >= 3) {
 		if (debris_colors > 4) {
 		    if (color == BLUE) {
@@ -508,7 +467,6 @@ static void Frame_shots(int conn, int ind)
 		    } else {
 			color = (shot->life >> 3);
 		    }
-
 		}
 		if (color >= debris_colors) {
 		    color = debris_colors - 1;
@@ -517,7 +475,7 @@ static void Frame_shots(int conn, int ind)
 
 	    debris_store(shot->pos.x - pv.world.x,
 			 shot->pos.y - pv.world.y,
-                         color);
+			 color);
 	    break;
 	case OBJ_SHOT:
 	    if (shot->id != -1
@@ -554,81 +512,34 @@ static void Frame_shots(int conn, int ind)
 	    break;
 	case OBJ_MINE:
 	    {
-	      int id = 0;
-	      int laid_by_team = 0;
-	      int confused = 0;
-	      /* calculate whether ownership of mine can be determined */
-	      if (identifyMines
-		  && (Wrap_length(pl->pos.x-shot->pos.x, pl->pos.y-shot->pos.y)
-		      < (SHIP_SZ + MINE_SENSE_BASE_RANGE
-			 + pl->sensors * MINE_SENSE_RANGE_FACTOR))) {
-		id = shot->id;
-		if (id==-1)
-		  id = EXPIRED_MINE_ID;
-		if (BIT(shot->status, CONFUSED))
-		  confused = 1;
-	      }
-	      laid_by_team = (shot->owner != -1 &&
-			((BIT(shot->status, OWNERIMMUNE)
-			  && shot->owner == pl->id)
-			 || TEAM_IMMUNE(ind, GetInd[shot->owner])));
-	      if (confused) {
-		id = 0;
-		laid_by_team = (rand()/3)%2;
-	      }
-	      Send_mine(conn, x, y, laid_by_team, id);
+		int id = 0;
+		int laid_by_team = 0;
+		int confused = 0;
+		/* calculate whether ownership of mine can be determined */
+		if (identifyMines
+		    && (Wrap_length(pl->pos.x - shot->pos.x,
+				    pl->pos.y - shot->pos.y)
+			< (SHIP_SZ + MINE_SENSE_BASE_RANGE
+			   + pl->item[ITEM_SENSOR] * MINE_SENSE_RANGE_FACTOR))) {
+		    id = shot->id;
+		    if (id==-1)
+			id = EXPIRED_MINE_ID;
+		    if (BIT(shot->status, CONFUSED))
+			confused = 1;
+		}
+		laid_by_team = (shot->owner != -1 &&
+				((BIT(shot->status, OWNERIMMUNE)
+				     && shot->owner == pl->id)
+				 || TEAM_IMMUNE(ind, GetInd[shot->owner])));
+		if (confused) {
+		    id = 0;
+		    laid_by_team = rand() & 0x01;
+		}
+		Send_mine(conn, x, y, laid_by_team, id);
 	    }
 	    break;
-	case OBJ_WIDEANGLE_SHOT:
-	    Send_item(conn, x, y, ITEM_WIDEANGLE_SHOT);
-	    break;
-	case OBJ_AFTERBURNER:
-	    Send_item(conn, x, y, ITEM_AFTERBURNER);
-	    break;
-	case OBJ_BACK_SHOT:
-	    Send_item(conn, x, y, ITEM_BACK_SHOT);
-	    break;
-	case OBJ_ROCKET_PACK:
-	    Send_item(conn, x, y, ITEM_ROCKET_PACK);
-	    break;
-	case OBJ_ENERGY_PACK:
-	    Send_item(conn, x, y, ITEM_ENERGY_PACK);
-	    break;
-	case OBJ_MINE_PACK:
-	    Send_item(conn, x, y, ITEM_MINE_PACK);
-	    break;
-	case OBJ_SENSOR_PACK:
-	    Send_item(conn, x, y, ITEM_SENSOR_PACK);
-	    break;
-	case OBJ_ECM:
-	    Send_item(conn, x, y, ITEM_ECM);
-	    break;
-	case OBJ_TANK:
-	    Send_item(conn, x, y, ITEM_TANK);
-	    break;
-	case OBJ_CLOAKING_DEVICE:
-	    Send_item(conn, x, y, ITEM_CLOAKING_DEVICE);
-	    break;
-	case OBJ_TRANSPORTER:
-	    Send_item(conn, x, y, ITEM_TRANSPORTER);
-	    break;
-	case OBJ_LASER:
-	    Send_item(conn, x, y, ITEM_LASER);
-	    break;
-	case OBJ_EMERGENCY_THRUST:
-
-		Send_item(conn, x, y, ITEM_EMERGENCY_THRUST);
-
-	    break;
-	case OBJ_TRACTOR_BEAM:
-
-		Send_item(conn, x, y, ITEM_TRACTOR_BEAM);
-
-	    break;
-	case OBJ_AUTOPILOT:
-
-		Send_item(conn, x, y, ITEM_AUTOPILOT);
-
+	case OBJ_ITEM:
+	    Send_item(conn, x, y, shot->info);
 	    break;
 	default:
 	    error("Frame_shots: Shot type %d not defined.", shot->type);
@@ -750,15 +661,16 @@ static void Frame_ships(int conn, int ind)
 		pl_i->id,
 		pl_i->dir,
 		BIT(pl_i->used, OBJ_SHIELD) != 0,
-		BIT(pl_i->used, OBJ_CLOAKING_DEVICE) != 0
+		BIT(pl_i->used, OBJ_CLOAKING_DEVICE) != 0,
+		BIT(pl_i->used, OBJ_EMERGENCY_SHIELD) != 0
 	    );
 	}
 	if (BIT(pl_i->used, OBJ_REFUEL)) {
-	    if (inview(World.fuel[pl_i->fs].pos.x,
-		       World.fuel[pl_i->fs].pos.y)) {
+	    if (inview(World.fuel[pl_i->fs].pix_pos.x,
+		       World.fuel[pl_i->fs].pix_pos.y)) {
 		Send_refuel(conn,
-		    (int) (World.fuel[pl_i->fs].pos.x + 0.5),
-		    (int) (World.fuel[pl_i->fs].pos.y + 0.5),
+		    (int) (World.fuel[pl_i->fs].pix_pos.x),
+		    (int) (World.fuel[pl_i->fs].pix_pos.y),
 		    (int) (pl_i->pos.x + 0.5),
 		    (int) (pl_i->pos.y + 0.5));
 	    }
@@ -783,15 +695,14 @@ static void Frame_ships(int conn, int ind)
 
 	    for (j = 0; j < 3; j++) {
 		Send_connector(conn,
-		       (int) (t->pos.x + t->ship->pts[t->dir][j].x + 0.5),
-		       (int) (t->pos.y + t->ship->pts[t->dir][j].y + 0.5),
+		       (int) (t->pos.x + t->ship->pts[j][t->dir].x + 0.5),
+		       (int) (t->pos.y + t->ship->pts[j][t->dir].y + 0.5),
 		       (int) (pl_i->pos.x + 0.5),
 		       (int) (pl_i->pos.y + 0.5), 1);
 	    }
 	}
 
 	if (pl_i->ball != NULL
-	    /* && BIT(pl_i->used, OBJ_CONNECTOR) */
 	    && inview(pl_i->ball->pos.x, pl_i->ball->pos.y)) {
 	    Send_connector(conn,
 		(int) (pl_i->ball->pos.x + 0.5),
@@ -808,7 +719,7 @@ static void Frame_radar(int conn, int ind)
     player		*pl = Players[ind];
     object		*shot;
     float		x, y;
-    
+
 #ifndef NO_SMART_MIS_RADAR
     if (nukesOnRadar) {
 	mask = OBJ_SMART_SHOT|OBJ_TORPEDO|OBJ_HEAT_SHOT|OBJ_MINE;
@@ -817,6 +728,8 @@ static void Frame_radar(int conn, int ind)
 		(OBJ_SMART_SHOT|OBJ_TORPEDO|OBJ_HEAT_SHOT) : 0);
 	mask |= (minesOnRadar) ? OBJ_MINE : 0;
     }
+    if (treasuresOnRadar)
+	mask |= OBJ_BALL;
 
     if (mask) {
 	for (i = 0; i < NumObjs; i++) {
@@ -825,12 +738,19 @@ static void Frame_radar(int conn, int ind)
 		continue;
 
 	    shownuke = (nukesOnRadar && (shot)->mods.nuclear);
+	    if (shownuke && (loops & 2)) {
+		s = 3;
+	    } else {
+		s = 0;
+	    }
 
 	    if (BIT(shot->type, OBJ_MINE)) {
 		if (!minesOnRadar && !shownuke)
 		    continue;
 		if (loops % 8 >= 6)
 		    continue;
+	    } else if (BIT(shot->type, OBJ_BALL)) {
+		s = 2;
 	    } else {
 		if (!missilesOnRadar && !shownuke)
 		    continue;
@@ -840,7 +760,6 @@ static void Frame_radar(int conn, int ind)
 
 	    x = shot->pos.x;
 	    y = shot->pos.y;
-	    s = (((loops & 2) && shownuke) ? 3 : 0);
 	    if (Wrap_length(pl->pos.x - x,
 			    pl->pos.y - y) <= pl->sensor_range) {
 		Send_radar(conn, (int) (x + 0.5), (int) (y + 0.5), s);
@@ -848,11 +767,19 @@ static void Frame_radar(int conn, int ind)
 	}
     }
 #endif
-    if (playersOnRadar) {
+    if (playersOnRadar || BIT(World.rules->mode, TEAM_PLAY)) {
 	for (i = 0; i < NumPlayers; i++) {
-	    if (i == ind
-		|| BIT(Players[i]->status, PLAYING|GAME_OVER) != PLAYING
-		|| !pl->visibility[i].canSee) {
+	    /*
+	     * Don't show on the radar:
+	     *		Ourselves (not necassarily same as who we watch).
+	     *		People who are not playing.
+	     *		People in other teams if;
+	     *			no playersOnRadar or if not visible
+	     */
+	    if (Players[i]->conn == conn
+		|| BIT(Players[i]->status, PLAYING|PAUSE|GAME_OVER) != PLAYING
+		|| (!TEAM(i, ind)
+		    && (!playersOnRadar || !pl->visibility[i].canSee))) {
 		continue;
 	    }
 	    x = Players[i]->pos.x;
@@ -887,23 +814,19 @@ static void Frame_parameters(int conn, int ind)
 
     pv.world.x = (int)(pl->pos.x + 0.5) - view_width / 2;	/* Scroll */
     pv.world.y = (int)(pl->pos.y + 0.5) - view_height / 2;
-    pv.wrappedWorld = 0;
+    pv.realWorld = pv.world;
     if (BIT (World.rules->mode, WRAP_PLAY)) {
-	pv.realWorld = pv.world;
-	if (pv.world.x < 0) {
-	    pv.wrappedWorld |= 1;
+	if (pv.world.x < 0 && pv.world.x + view_width < World.width) {
 	    pv.world.x += World.width;
-	} else if (pv.world.x + view_width >= World.width) {
-	    pv.realWorld.x -= World.width;
-	    pv.wrappedWorld |= 1;
 	}
-
-	if (pv.world.y < 0) {
-	    pv.wrappedWorld |= 2;
+	else if (pv.world.x > 0 && pv.world.x + view_width >= World.width) {
+	    pv.realWorld.x -= World.width;
+	}
+	if (pv.world.y < 0 && pv.world.y + view_height < World.height) {
 	    pv.world.y += World.height;
-	} else if (pv.world.y + view_height >= World.height) {
+	}
+	else if (pv.world.y > 0 && pv.world.y + view_height >= World.height) {
 	    pv.realWorld.y -= World.height;
-	    pv.wrappedWorld |= 2;
 	}
     }
 }
@@ -917,7 +840,7 @@ void Frame_update(void)
     time_t		newTimeLeft = 0;
     static time_t	oldTimeLeft;
     static bool		game_over_called = false;
-    
+
     if (++loops >= LONG_MAX)	/* Used for misc. timing purposes */
 	loops = 0;
 
@@ -951,6 +874,18 @@ void Frame_update(void)
 		continue;
 	    }
 	}
+
+	/*
+	* Reduce frame rate to player's own rate.
+	*/
+	if (pl->player_count > 0) {
+	    pl->player_round++;
+	    if (pl->player_round > pl->player_count) {
+		pl->player_round = 0;
+		continue;
+	    }
+	}
+
 	if (Send_start_of_frame(conn) == -1) {
 	    continue;
 	}
@@ -962,7 +897,7 @@ void Frame_update(void)
 	 * other players 'eyes'.  If PAUSE'd this only works on team members.
 	 * We can't use TEAM() macro as it PAUSE'd players are always on
 	 * equivalent teams.
-	 * 
+	 *
 	 * This is done by using two indexes, one
 	 * determining which data should be used (ind, set below) and
 	 * one determining which connection to send it to (conn).
@@ -1046,4 +981,7 @@ void Set_player_message(player *pl, char *message)
     }
     if (pl->conn != NOT_CONNECTED)
 	Send_message(pl->conn, msg);
+    else if (IS_ROBOT_PTR(pl))
+	Robot_message(GetInd[pl->id], msg);
 }
+
