@@ -1,6 +1,6 @@
-/* $Id: netclient.c,v 3.59 1994/04/05 20:28:40 bert Exp $
+/* $Id: netclient.c,v 3.82 1995/01/29 16:17:44 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -48,6 +48,8 @@
 #include <netdb.h>
 
 #include "version.h"
+#include "config.h"
+#include "const.h"
 #include "error.h"
 #include "net.h"
 #include "netclient.h"
@@ -57,11 +59,14 @@
 #include "paint.h"
 #include "xinit.h"
 #include "pack.h"
-#include "draw.h"
-#include "client.h"
+#include "socklib.h"
 
-#define TALK_RETRY	FPS
+char netclient_version[] = VERSION;
 
+#define TALK_RETRY	2
+
+
+int micro_delay(unsigned usec);
 
 /*
  * Type definitions.
@@ -69,13 +74,14 @@
 typedef struct {
     long		loops;
     sockbuf_t		sbuf;
-} frame_t;
+} frame_buf_t;
 
 /*
  * Exported variables.
  */
 setup_t			*Setup;
 int			receive_window_size;
+long			last_loops;
 
 /*
  * Local variables.
@@ -83,12 +89,11 @@ int			receive_window_size;
 static sockbuf_t	rbuf,
 			cbuf,
 			wbuf;
-static frame_t		*Frames;
+static frame_buf_t	*Frames;
 static int		(*receive_tbl[256])(void),
 			(*reliable_tbl[256])(void);
 static unsigned		magic;
-static long		last_loops,
-			last_keyboard_change,
+static long		last_keyboard_change,
 			last_keyboard_ack,
 			last_keyboard_update,
 			last_send_anything,
@@ -98,7 +103,6 @@ static long		last_loops,
 			talk_last_send;
 static char		talk_str[MAX_CHARS];
 
-extern void usleep(unsigned long usec);
 
 /*
  * Initialize the function dispatch tables.
@@ -142,17 +146,10 @@ static void Receive_init(void)
     receive_tbl[PKT_RADAR]	= Receive_radar;
     receive_tbl[PKT_RELIABLE]	= Receive_reliable;
     receive_tbl[PKT_QUIT]	= Receive_quit;
-    receive_tbl[PKT_SHOT+0]	= Receive_shot;
-    receive_tbl[PKT_SHOT+1]	= Receive_shot;
-    receive_tbl[PKT_SHOT+2]	= Receive_shot;
-    receive_tbl[PKT_SHOT+3]	= Receive_shot;
-    receive_tbl[PKT_TEAMSHOT+0] = Receive_teamshot;
-    receive_tbl[PKT_TEAMSHOT+1] = Receive_teamshot;
-    receive_tbl[PKT_TEAMSHOT+2] = Receive_teamshot;
-    receive_tbl[PKT_TEAMSHOT+3] = Receive_teamshot;
     receive_tbl[PKT_MODIFIERS]  = Receive_modifiers;
     receive_tbl[PKT_FASTSHOT]	= Receive_fastshot;
     receive_tbl[PKT_THRUSTTIME] = Receive_thrusttime;
+    receive_tbl[PKT_SHIELDTIME] = Receive_shieldtime;
     for (i = 0; i < DEBRIS_TYPES; i++) {
 	receive_tbl[PKT_DEBRIS + i] = Receive_debris;
     }
@@ -161,6 +158,7 @@ static void Receive_init(void)
     reliable_tbl[PKT_MESSAGE]	= Receive_message;
     reliable_tbl[PKT_PLAYER]	= Receive_player;
     reliable_tbl[PKT_SCORE]	= Receive_score;
+    reliable_tbl[PKT_TIMING]	= Receive_timing;
     reliable_tbl[PKT_LEAVE]	= Receive_leave;
     reliable_tbl[PKT_WAR]	= Receive_war;
     reliable_tbl[PKT_SEEK]	= Receive_seek;
@@ -247,7 +245,7 @@ static int Uncompress_map(void)
 }
 
 /*
- * Receive the map data and some game parameters from 
+ * Receive the map data and some game parameters from
  * the server.  The map data may be in compressed form.
  */
 int Net_setup(void)
@@ -391,7 +389,7 @@ int Net_setup(void)
  * Send the first packet to the server with our name,
  * nick and display contained in it.
  * The server uses this data to verify that the packet
- * is from the right UDP connection, it already has 
+ * is from the right UDP connection, it already has
  * this info from the ENTER_GAME_pack.
  */
 int Net_verify(char *real, char *nick, char *disp, int my_team)
@@ -421,7 +419,9 @@ int Net_verify(char *real, char *nick, char *disp, int my_team)
 	    }
 	    time(&last);
 #ifndef SILENT
-	    printf("Waiting for verify response\n");
+	    if (retries > 1) {
+		printf("Waiting for verify response\n");
+	    }
 #endif
 	}
 	SetTimeout(1, 0);
@@ -478,7 +478,9 @@ int Net_verify(char *real, char *nick, char *disp, int my_team)
 	break;
     }
 #ifndef SILENT
-    printf("Verified correctly\n");
+    if (retries > 1) {
+	printf("Verified correctly\n");
+    }
 #endif
     return 0;
 }
@@ -523,8 +525,8 @@ int Net_init(char *server, int port)
 	error("Can't set receive buffer size to %d", CLIENT_RECV_SIZE + 256);
     }
 
-    size = receive_window_size * sizeof(frame_t);
-    if ((Frames = (frame_t *) malloc(size)) == NULL) {
+    size = receive_window_size * sizeof(frame_buf_t);
+    if ((Frames = (frame_buf_t *) malloc(size)) == NULL) {
 	error("No memory (%u)", size);
 	return -1;
     }
@@ -564,7 +566,7 @@ int Net_init(char *server, int port)
     return 0;
 }
 
-/* 
+/*
  * Cleanup all the network buffers and close the datagram socket.
  * Also try to send the server a quit packet if possible.
  * Because this quit packet may get lost we send one at the
@@ -582,7 +584,7 @@ void Net_cleanup(void)
 	    GetSocketError(sock);
 	    write(sock, &ch, 1);
 	}
-	usleep(50*1000);
+	micro_delay((unsigned)50*1000);
     }
     if (Frames != NULL) {
 	for (i = 0; i < receive_window_size; i++) {
@@ -607,7 +609,7 @@ void Net_cleanup(void)
 	    GetSocketError(sock);
 	    write(sock, &ch, 1);
 	}
-	usleep(50*1000);
+	micro_delay((unsigned)50*1000);
 	if (write(sock, &ch, 1) != 1) {
 	    GetSocketError(sock);
 	    write(sock, &ch, 1);
@@ -635,6 +637,13 @@ int Net_flush(void)
 	wbuf.ptr = wbuf.buf;
 	return 0;
     }
+    if (last_keyboard_ack != last_keyboard_change) {
+	/*
+	 * Since 3.2.10: just call Key_update to add our keyboard vector.
+	 * Key_update will call Send_keyboard to flush our buffer.
+	 */
+	return Key_update();
+    }
     Send_talk();
     if (Sockbuf_flush(&wbuf) == -1) {
 	return -1;
@@ -677,7 +686,11 @@ int Net_start(void)
 	    if (Send_shape(shipShape) == -1
 		|| Packet_printf(&wbuf, "%c", PKT_PLAY) <= 0
 		|| Client_power() == -1
-	    	|| Sockbuf_flush(&wbuf) == -1) {
+#ifdef SOUND
+		|| Send_audio_request(1) == -1
+#endif
+		|| Client_fps_request() == -1
+		|| Sockbuf_flush(&wbuf) == -1) {
 		error("Can't send start play packet");
 		return -1;
 	    }
@@ -841,8 +854,20 @@ static int Net_packet(void)
 	    error("Got reply packet (%d,%d)", replyto, status);
 	}
 	else if (reliable_tbl[type] == NULL) {
+	    int i;
 	    errno = 0;
-	    error("Received unknown reliable data packet type (%d)", type);
+	    error("Received unknown reliable data packet type (%d,%d,%d)",
+		  type, cbuf.ptr - cbuf.buf, cbuf.len);
+	    printf("\tdumping buffer for debugging:\n");
+	    for (i = 0; i < cbuf.len; i++) {
+		printf("%3d", cbuf.buf[i] & 0xFF);
+		if (i % 20 == 0) {
+		    printf("\n");
+		} else {
+		    printf(" ");
+		}
+	    }
+	    printf("\n");
 	    return -1;
 	}
 	else if ((result = (*reliable_tbl[type])()) <= 0) {
@@ -909,7 +934,7 @@ static void Net_measurement(long loop, int status)
  * we retry to read a packet once more.
  * It's a non-blocking read.
  */
-static int Net_read(frame_t *frame)
+static int Net_read(frame_buf_t *frame)
 {
     int		n;
     long	loop;
@@ -934,6 +959,7 @@ static int Net_read(frame_t *frame)
 	     */
 	    return 1;
 	}
+	/* Peek at the frame loop number. */
 	n = Packet_scanf(&frame->sbuf, "%c%ld", &ch, &loop);
 	frame->sbuf.ptr = frame->sbuf.buf;
 	if (n <= 0) {
@@ -969,7 +995,7 @@ int Net_input(void)
     int		i,
 		j,
 		n;
-    frame_t	*frame,
+    frame_buf_t	*frame,
 		*last_frame,
 		*oldest_frame = &Frames[0],
 		tmpframe;
@@ -1134,12 +1160,12 @@ int Net_input(void)
 
     /*
      * If the server hasn't yet acked our last keyboard change
-     * and we haven't updated it in this time frame already
-     * or we haven't sent anything for a while (keepalive)
+     * and we haven't updated it in the previous time frame
+     * or if we haven't sent anything for a while (keepalive)
      * then we send our current keyboard state.
      */
-    if (last_keyboard_ack != last_keyboard_change
-	&& last_keyboard_update < last_loops
+    if ((last_keyboard_ack != last_keyboard_change
+	    && last_keyboard_update + 1 < last_loops)
 	|| last_loops - last_send_anything > 5 * Setup->frames_per_second) {
 	Key_update();
 	last_send_anything = last_loops;
@@ -1312,9 +1338,6 @@ int Net_ask_for_motd(long offset, long maxlen)
 	error("Can't ask motd");
 	return -1;
     }
-#ifndef SILENT
-    printf("Asking for a motd\n");
-#endif
 
     return 0;
 }
@@ -1329,7 +1352,7 @@ int Receive_self(void)
 {
     int		n;
     short	x, y, vx, vy, lockId, lockDist,
-    		fuelSum, fuelMax;
+		fuelSum, fuelMax;
     u_byte	ch, heading, power, turnspeed, turnresistance,
 		nextCheckPoint, lockDir, autopilotLight, currentTank, stat;
     u_byte	num_items[NUM_ITEMS];
@@ -1351,15 +1374,15 @@ int Receive_self(void)
 		     &power, &turnspeed, &turnresistance,
 		     &lockId, &lockDist, &lockDir, &nextCheckPoint,
 
-		     &(num_items[ITEM_CLOAKING_DEVICE]),
-		     &(num_items[ITEM_SENSOR_PACK]),
-		     &(num_items[ITEM_MINE_PACK]),
-		     &(num_items[ITEM_ROCKET_PACK]),
+		     &(num_items[ITEM_CLOAK]),
+		     &(num_items[ITEM_SENSOR]),
+		     &(num_items[ITEM_MINE]),
+		     &(num_items[ITEM_MISSILE]),
 		     &(num_items[ITEM_ECM]),
 
 		     &(num_items[ITEM_TRANSPORTER]),
-		     &(num_items[ITEM_WIDEANGLE_SHOT]),
-		     &(num_items[ITEM_BACK_SHOT]),
+		     &(num_items[ITEM_WIDEANGLE]),
+		     &(num_items[ITEM_REARSHOT]),
 		     &(num_items[ITEM_AFTERBURNER]),
 		     &(num_items[ITEM_TANK]),
 
@@ -1372,9 +1395,22 @@ int Receive_self(void)
 		     &view_width, &view_height, &debris_colors,
 		     &stat, &autopilotLight
 		     );
+    if (n <= 0) {
+	return n;
+    }
+    if (version < 0x3200) {
+	num_items[ITEM_EMERGENCY_SHIELD] = 0;
+    }
+    else {
+	n = Packet_scanf(&rbuf, "%c",
+			 &(num_items[ITEM_EMERGENCY_SHIELD]));
+	if (n <= 0) {
+	    return n;
+	}
+    }
 
-    if (debris_colors > 8) {
-	debris_colors = 8;
+    if (debris_colors > num_spark_colors) {
+	debris_colors = num_spark_colors;
     }
     if (view_width != draw_width || view_height != draw_height) {
 	Send_display();
@@ -1486,7 +1522,7 @@ int Receive_ball(void)
 
 int Receive_ship(void)
 {
-    int		n, shield, cloak;
+    int		n, shield, cloak, eshield;
     short	x, y, id;
     u_byte	ch, dir, flags;
 
@@ -1498,7 +1534,8 @@ int Receive_ship(void)
     }
     shield = ((flags & 1) != 0);
     cloak = ((flags & 2) != 0);
-    if ((n = Handle_ship(x, y, id, dir, shield, cloak)) == -1) {
+    eshield = ((flags & 4) != 0);
+    if ((n = Handle_ship(x, y, id, dir, shield, cloak, eshield)) == -1) {
 	return -1;
     }
     return 1;
@@ -1529,8 +1566,10 @@ int Receive_item(void)
     if ((n = Packet_scanf(&rbuf, "%c%hd%hd%c", &ch, &x, &y, &type)) <= 0) {
 	return n;
     }
-    if ((n = Handle_item(x, y, type)) == -1) {
-	return -1;
+    if (type < NUM_ITEMS) {
+	if ((n = Handle_item(x, y, type)) == -1) {
+	    return -1;
+	}
     }
     return 1;
 }
@@ -1580,6 +1619,21 @@ int Receive_thrusttime(void)
     return 1;
 }
 
+int Receive_shieldtime(void)
+{
+    int		n;
+    short	count, max;
+    u_byte	ch;
+
+    if ((n = Packet_scanf(&rbuf, "%c%hd%hd", &ch, &count, &max)) <= 0) {
+	return n;
+    }
+    if ((n = Handle_shieldtime(count, max)) == -1) {
+	return -1;
+    }
+    return 1;
+}
+
 int Receive_fastshot(void)
 {
     int			n, r, type;
@@ -1618,43 +1672,6 @@ int Receive_debris(void)
     return (r == -1) ? -1 : 1;
 }
 
-int Receive_shot(void)
-{
-    int		n, color;
-    short	x, y;
-    u_byte	ch;
-
-    if ((n = Packet_scanf(&rbuf, "%c%hd%hd", &ch, &x, &y)) <= 0) {
-	return n;
-    }
-    color = ch - PKT_SHOT;
-    if ((n = Handle_shot(x, y, color)) == -1) {
-	return -1;
-    }
-    return 1;
-}
-
-int Receive_teamshot(void)
-{
-    int		n, color;
-    short	x, y;
-    u_byte	ch;
-
-    if ((n = Packet_scanf(&rbuf, "%c%hd%hd", &ch, &x, &y)) <= 0) {
-	return n;
-    }
-    color = ch - PKT_TEAMSHOT;
-    if (color < 0 || color >= NUM_COLORS) {
-	errno = 0;
-	error("Bad teamshot color %d (%d)", color, ch);
-	return -1;
-    }
-    if ((n = Handle_teamshot(x, y, color)) == -1) {
-	return -1;
-    }
-    return 1;
-}
-
 int Receive_ecm(void)
 {
     int			n;
@@ -1676,13 +1693,13 @@ int Receive_trans(void)
     short		x1, y1, x2, y2;
     u_byte		ch;
 
-    if ((n = Packet_scanf(&rbuf, "%c%hd%hd%hd%hd", 
+    if ((n = Packet_scanf(&rbuf, "%c%hd%hd%hd%hd",
 			  &ch, &x1, &y1, &x2, &y2)) <= 0) {
 	return n;
     }
     if ((n = Handle_trans(x1, y1, x2, y2)) == -1) {
 	return -1;
-    } 
+    }
     return 1;
 }
 
@@ -1710,7 +1727,7 @@ int Receive_radar(void)
     if ((n = Packet_scanf(&rbuf, "%c%hd%hd%c", &ch, &x, &y, &size)) <= 0) {
 	return n;
     }
-	    
+
     if ((n = Handle_radar(x, y, size)) == -1) {
 		    return -1;
     }
@@ -1782,20 +1799,27 @@ int Receive_player(void)
 {
     int			n;
     short		id;
-    u_byte		ch, team, mychar;
+    u_byte		ch, myteam, mychar;
     char		name[MAX_CHARS],
 			real[MAX_CHARS],
 			host[MAX_CHARS],
-			shape[MSG_LEN];
+			shape[2*MSG_LEN],
+			*cbuf_ptr = cbuf.ptr;
 
     if ((n = Packet_scanf(&cbuf,
 			  "%c%hd%c%c" "%s%s%s" "%S",
-			  &ch, &id, &team, &mychar,
+			  &ch, &id, &myteam, &mychar,
 			  name, real, host,
 			  shape)) <= 0) {
 	return n;
     }
-    if ((n = Handle_player(id, team, mychar, name, real, host, shape)) == -1) {
+    if (version > 0x3200) {
+	if ((n = Packet_scanf(&cbuf, "%S", &shape[strlen(shape)])) <= 0) {
+	    cbuf.ptr = cbuf_ptr;
+	    return n;
+	}
+    }
+    if ((n = Handle_player(id, myteam, mychar, name, real, host, shape)) == -1) {
 	return -1;
     }
     return 1;
@@ -1809,7 +1833,7 @@ int Receive_score_object(void)
     char		msg[MAX_CHARS];
     u_byte		ch;
 
-    if ((n = Packet_scanf(&cbuf, "%c%hd%hu%hu%s", 
+    if ((n = Packet_scanf(&cbuf, "%c%hd%hu%hu%s",
 			  &ch, &score, &x, &y, msg)) <= 0) {
 	return n;
     }
@@ -1837,6 +1861,27 @@ int Receive_score(void)
     return 1;
 }
 
+int Receive_timing(void)
+{
+    int			n,
+			check,
+			round;
+    short		id;
+    unsigned short	timing;
+    u_byte		ch;
+
+    n = Packet_scanf(&cbuf, "%c%hd%hu", &ch, &id, &timing);
+    if (n <= 0) {
+	return n;
+    }
+    check = timing % MAX_CHECKS;
+    round = timing / MAX_CHECKS;
+    if ((n = Handle_timing(id, check, round)) == -1) {
+	return -1;
+    }
+    return 1;
+}
+
 int Receive_fuel(void)
 {
     int			n;
@@ -1877,7 +1922,7 @@ int Receive_target(void)
 			damage;
     u_byte		ch;
 
-    if ((n = Packet_scanf(&rbuf, "%c%hu%hu%hu", &ch, 
+    if ((n = Packet_scanf(&rbuf, "%c%hu%hu%hu", &ch,
 			  &num, &dead_time, &damage)) <= 0) {
 	return n;
     }
@@ -2070,10 +2115,20 @@ int Send_keyboard(u_byte *keyboard_vector)
 
 int Send_shape(char *str)
 {
-    if (Packet_printf(&wbuf, "%c%S", PKT_SHAPE,
-		      (str) ? str : "") <= 0) {
+    wireobj		*w;
+    char		buf[MSG_LEN], ext[MSG_LEN];
+
+    w = Convert_shape_str(str);
+    Convert_ship_2_string(w, buf, ext, (version < 0x3200) ? 0x3100 : 0x3200);
+    Free_ship_shape(w);
+    if (Packet_printf(&wbuf, "%c%S", PKT_SHAPE, buf) <= 0) {
 	return -1;
     }
+    if (version > 0x3200) {
+	if (Packet_printf(&wbuf, "%S", ext) <= 0) {
+	    return -1;
+	}
+    }
     return 0;
 }
 
@@ -2217,13 +2272,8 @@ int Send_talk(void)
 
 int Send_display(void)
 {
-    int			num_col;
-
-    num_col = (maxColors == 8) ? 4
-	: (maxColors == 16) ? 8
-	: 0;
     if (Packet_printf(&wbuf, "%c%hd%hd%c%c", PKT_DISPLAY,
-		      draw_width, draw_height, num_col, spark_rand) == -1) {
+		      draw_width, draw_height, num_spark_colors, spark_rand) == -1) {
 	return -1;
     }
     return 0;
@@ -2239,3 +2289,40 @@ int Send_modifier_bank(int bank)
 	return -1;
     return 0;
 }
+
+int Send_pointer_move(int movement)
+{
+    if (version > 0x3201) {
+	if (Packet_printf(&wbuf, "%c%hd", PKT_POINTER_MOVE,
+			  movement) == -1) {
+	    return -1;
+	}
+    }
+    return 0;
+}
+
+int Send_audio_request(int onoff)
+{
+    if (version < 0x3250) {
+	return 0;
+    }
+#ifndef SOUND
+    onoff = 0;
+#endif
+    if (Packet_printf(&wbuf, "%c%c", PKT_REQUEST_AUDIO, (onoff != 0)) == -1) {
+	return -1;
+    }
+    return 0;
+}
+
+int Send_fps_request(int fps)
+{
+    if (version < 0x3280) {
+	return 0;
+    }
+    if (Packet_printf(&wbuf, "%c%c", PKT_ASYNC_FPS, fps) == -1) {
+	return -1;
+    }
+    return 0;
+}
+
