@@ -1,6 +1,6 @@
-/* $Id: server.c,v 3.80 1994/04/14 11:46:39 bert Exp $
+/* $Id: server.c,v 3.109 1995/03/08 16:45:15 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -31,6 +31,7 @@
 #endif
 #include <stdlib.h>
 #include <string.h>
+#include <ctype.h>
 #include <stdio.h>
 #include <signal.h>
 #include <time.h>
@@ -43,6 +44,9 @@
 					(struct process_stats *)NULL, \
 					(struct process_stats *)NULL)
 #endif
+#ifdef sony_news
+#define setvbuf(A,B,C,D)	setlinebuf(A)
+#endif
 #ifdef VMS
 #include "username.h"
 #include <socket.h>
@@ -56,15 +60,18 @@
 #include <arpa/inet.h>
 #endif
 #include <netdb.h>
+#ifdef PLOCKSERVER
+#include <sys/lock.h>
+#endif
 
-#include "config.h"
-#include "global.h"
-#include "socklib.h"
 #include "version.h"
+#include "config.h"
+#include "const.h"
+#include "global.h"
+#include "proto.h"
+#include "socklib.h"
 #include "map.h"
 #include "pack.h"
-#include "draw.h"
-#include "robot.h"
 #include "saudio.h"
 #include "bit.h"
 #include "net.h"
@@ -73,10 +80,12 @@
 #include "cmw.h"
 #endif /* SUNCMW */
 
+char server_version[] = VERSION;
+
 #ifndef	lint
 static char versionid[] = "@(#)$" TITLE " $";
 static char sourceid[] =
-    "@(#)$Id: server.c,v 3.80 1994/04/14 11:46:39 bert Exp $";
+    "@(#)$Id: server.c,v 3.109 1995/03/08 16:45:15 bert Exp $";
 #endif
 
 
@@ -92,10 +101,9 @@ long			Id = 1;		    /* Unique ID for each object */
 long			GetInd[MAX_ID];
 server			Server;
 int			ShutdownServer = -1, ShutdownDelay = 1000;
+char			ShutdownReason[MAX_CHARS];
 int 			framesPerSecond = 18;
-
-int			Argc;
-char			**Argv;
+long			main_loops = 0;		/* needed in events.c */
 
 static int		Socket;
 static sockbuf_t	ibuf;
@@ -104,14 +112,18 @@ static char		meta_address[16];
 static char		meta_address_two[16];
 static bool		Log = true;
 static bool		NoPlayersEnteredYet = true;
-static bool		lock = false;
-static void		Wait_for_new_players(void);
+static bool		game_lock = false;
 time_t			gameOverTime = 0;
 time_t			serverTime = 0;
 extern int		login_in_progress;
 
-void Send_meta_server(void);
-static bool Owner(char *name, char *in_host);
+static bool Owner(char request, char *real_name, char *host_addr, int pass);
+static void plockserver(int onoff);
+static void Check_server_versions(void);
+static void Wait_for_new_players(void);
+static void Main_Loop(void);
+static void Handle_signal(int sig_no);
+static void Server_info(char *, unsigned);
 
 static void catch_alarm(int signum)
 {
@@ -123,7 +135,6 @@ int main(int argc, char *argv[])
     struct hostent *hinfo;
     struct passwd *pwent;
 
-
     /*
      * Make output always linebuffered.  By default pipes
      * and remote shells cause stdout to be fully buffered.
@@ -139,12 +150,12 @@ int main(int argc, char *argv[])
 	      "for details see the\n"
 	   "  provided LICENSE file.\n\n");
 
-    Argc = argc; Argv = argv;
 #ifdef SUNCMW
     cmw_priv_init();
 #endif /* SUNCMW */
     init_error(argv[0]);
     srand(time((time_t *)0) * getpid());
+    Check_server_versions();
     Parser(argc, argv);
     Make_table();			/* Make trigonometric tables */
     Compute_gravity();
@@ -157,6 +168,8 @@ int main(int argc, char *argv[])
 
     Move_init();
 
+    Robot_init();
+
     if (BIT(World.rules->mode, TEAM_PLAY)) {
 	int i;
 	for(i=0; i < World.NumTreasures; i++)
@@ -193,7 +206,7 @@ int main(int argc, char *argv[])
 	signal(SIGALRM, SIG_IGN);
 	if (hinfo == NULL) {
 	    /* This should be changed since we now send to meta host 1.
-	     * But there is no harm that there are sent two packets to the 
+	     * But there is no harm that there are sent two packets to the
 	     * same meta server. */
 	    strncpy(meta_address_two, META_IP, sizeof meta_address_two);
 	} else {
@@ -205,7 +218,7 @@ int main(int argc, char *argv[])
     }
 
     /*
-     * Get owners login name.
+     * Get owner's login name.
      */
 #ifdef VMS
     getusername(Server.name);
@@ -257,17 +270,17 @@ int main(int argc, char *argv[])
 #endif
 
     /*
-     * Set the time the server started 
+     * Set the time the server started
      */
 
     serverTime = time(NULL);
 
-    /* 
+    /*
      * Report to Meta server
      */
-   
-    Send_meta_server();
-    
+
+    Send_meta_server(1);
+
     /*
      * If the server is not in raw mode it should run only if
      * there are players logged in.
@@ -282,35 +295,54 @@ int main(int argc, char *argv[])
 	signal(SIGALRM, SIG_IGN);
 	SetTimeout(0, 0);
     }
- 
+
     if (gameDuration > 0.0) {
 	printf("Server will stop in %g minutes.\n", gameDuration);
 	gameOverTime = (time_t)(gameDuration * 60) + time((time_t *)NULL);
     }
-    
+
     Main_Loop();			    /* Entering main loop. */
     /* NEVER REACHED */
     return (-1);
 }
 
-void Send_meta_server(void)
+void Send_meta_server(int change)
 {
 #ifdef SOUND
 #define SOUND_SUPPORT_STR	"yes"
 #else
 #define SOUND_SUPPORT_STR	"no"
 #endif
+#define GIVE_META_SERVER_A_HINT	180
+
+    char 		string[MAX_STR_LEN];
+    char 		status[MAX_STR_LEN];
+    int			i;
+    int			num_paused_players;
+    bool		first = true;
+    time_t		currentTime;
+    static time_t	lastMetaSendTime = 0;
 
-    char 	string[MAX_STR_LEN];
-    char 	status[MAX_STR_LEN];
-    int		i;
-    bool	first = true;
 
     if (!reportToMetaServer)
 	return;
 
+    currentTime = time(NULL);
+    if (!change) {
+	if (currentTime - lastMetaSendTime < GIVE_META_SERVER_A_HINT)
+	    return;
+    }
+    lastMetaSendTime = currentTime;
+
     Server_info(status, sizeof(status));
 
+    /* Find out the number of paused players. */
+    num_paused_players = 0;
+    for (i = 0; i < NumPlayers; i++) {
+	if (IS_HUMAN_IND(i) && BIT(Players[i]->status, PAUSE)) {
+	    num_paused_players++;
+	}
+    }
     sprintf(string,
 	    "add server %s\n"
 	    "add users %d\n"
@@ -322,30 +354,38 @@ void Send_meta_server(void)
 	    "add fps %d\n"
 	    "add port %d\n"
 	    "add mode %s\n"
+	    "add teams %d\n"
+	    "add timing %d\n"
 	    "add stime %ld\n"
 	    "add sound " SOUND_SUPPORT_STR "\n",
-	    Server.host, NumPlayers - NumRobots, 
-	    VERSION, World.name, World.x, World.y, World.author, 
+	    Server.host, NumPlayers - NumPseudoPlayers - NumRobots - num_paused_players,
+	    VERSION, World.name, World.x, World.y, World.author,
 	    World.NumBases, FPS, contactPort,
-	    (lock && ShutdownServer == -1) ? "locked"
-		: (!lock && ShutdownServer != -1) ? "shutting down"
-		: (lock && ShutdownServer != -1) ? "locked and shutting down"
-		: "ok",
+	    (game_lock && ShutdownServer == -1) ? "locked"
+		: (!game_lock && ShutdownServer != -1) ? "shutting down"
+		: (game_lock && ShutdownServer != -1) ? "locked and shutting down"
+		: "ok", World.NumTeamBases,
+	    BIT(World.rules->mode, TIMING) ? 1:0,
 	    time(NULL) - serverTime);
 
 
     for(i=0; i < NumPlayers; i++) {
-	if (Players[i]->robot_mode == RM_NOT_ROBOT) {
+	if (IS_HUMAN_IND(i)) {
 	    sprintf(string + strlen(string),
 		    "%s%s=%s@%s",
 		    (first) ? "add players " : ",",
 		    Players[i]->name,
 		    Players[i]->realname,
 		    Players[i]->hostname);
+	    if (BIT(World.rules->mode, TEAM_PLAY)) {
+		sprintf(status,"{%d}",Players[i]->team);
+		strcat(string,status);
+	    }
+
 	    first = false;
 	}
     }
-    
+
     strcat(string,"\nadd status ");
     if (strlen(string) + strlen(status) >= sizeof(string)) {
 	/* Prevent array overflow */
@@ -369,42 +409,34 @@ void Send_meta_server(void)
 /*
  * Main loop.
  */
-void Main_Loop(void)
+static void Main_Loop(void)
 {
     extern void		Loop_delay(void);
-    int			main_loops = 0;
-    time_t		currentTime;
-    time_t		lastMetaCheckTime = 0;
 
 #ifndef SILENT
     printf("Server runs at %d frames per second\n", framesPerSecond);
 #endif
 
+    plockserver(true);
+
     SetTimeout(0, 0);
 
     while (NoQuit
 	   || NumPlayers - NumPseudoPlayers > NumRobots
 	   || NoPlayersEnteredYet) {
-	
-	currentTime = time(NULL);
+
 	main_loops++;
 
-#define CHECK_FOR_NEW_PLAYERS	4
-#define GIVE_META_SERVER_A_HINT	180
-
-	if (currentTime - lastMetaCheckTime >= GIVE_META_SERVER_A_HINT) {
-	    lastMetaCheckTime = currentTime;
-	    Send_meta_server();
-	}
-
 	if (NumPlayers - NumPseudoPlayers == NumRobots && !RawMode) {
 	    block_timer();
+	    plockserver(false);
 	    Wait_for_new_players();
+	    plockserver(true);
 	    allow_timer();
 	}
-	
+
 	Update_objects();
-	
+
 	/*
 	 * Check for possible shutdown, the server will
 	 * shutdown when ShutdownServer (a counter) reaches 0.
@@ -416,9 +448,12 @@ void Main_Loop(void)
 	    else
 		ShutdownServer--;
 	}
-	
+
 	if ((main_loops % UPDATES_PR_FRAME) == 0) {
 	    Frame_update();
+	    if ((main_loops & 0xF) == 0) {
+		Send_meta_server(0);
+	    }
 	    Loop_delay();
 	}
 
@@ -436,36 +471,50 @@ void Main_Loop(void)
  */
 static void Wait_for_new_players(void)
 {
+#define CHECK_FOR_NEW_PLAYERS	4
+
     int			new_players = false;
-    time_t		start_time,
-			start_loops,
-			milli_delta;
+    long		delta;
 #if _SEQUENT_
-    timeval_t		tv;
+    timeval_t		tv0, tv1;
 #else
-    struct timeval	tv;
+    struct timeval	tv0, tv1;
 #endif
 
-    gettimeofday(&tv, NULL);
-    start_time = tv.tv_sec;
-    start_loops = loops;
+    gettimeofday(&tv0, NULL);
     while (new_players == false) {
 	if (login_in_progress > 0) {
 	    if (Input() > 0) {
 		NoPlayersEnteredYet = false;
 		new_players = true;
-		continue;
+		break;
 	    }
 	    SetTimeout(0, 10*1000);
 	} else {
-	    SetTimeout(CHECK_FOR_NEW_PLAYERS, 0);
+	    if (ShutdownServer >= 0) {
+		SetTimeout(1, 0);
+	    } else {
+		SetTimeout(CHECK_FOR_NEW_PLAYERS, 0);
+	    }
 	}
 	if (SocketReadable(Socket) != 0) {
 	    Contact();
 	}
-	gettimeofday(&tv, NULL);
-	milli_delta = (tv.tv_sec - start_time) * 1000 + tv.tv_usec / 1000;
-	loops = start_loops + (FPS * milli_delta) / 1000;
+	Send_meta_server(0);
+	gettimeofday(&tv1, NULL);
+	delta = 1000*1000 * (long)(tv1.tv_sec - tv0.tv_sec)
+		- ((long)tv1.tv_usec - (long)tv0.tv_usec);
+	delta = delta * FPS / (1000*1000);
+	if (delta > 0) {
+	    loops += delta;
+	    tv0 = tv1;
+	    if (ShutdownServer >= 0) {
+		if ((ShutdownServer -= delta) <= 0) {
+		    ShutdownServer = 0;
+		    End_game();
+		}
+	    }
+	}
     }
     SetTimeout(0, 0);
 }
@@ -475,13 +524,16 @@ static void Wait_for_new_players(void)
  */
 void End_game(void)
 {
-    player	*pl;
-    int		len;
-    char	string[80];
+    player		*pl;
+    int			len;
+    char		msg[MSG_LEN];
 
     if (ShutdownServer == 0) {
 	errno = 0;
 	error("Shutting down...");
+	sprintf(msg, "shutting down: %s", ShutdownReason);
+    } else {
+	sprintf(msg, "server exiting");
     }
 
     while (NumPlayers > 0) {	/* Kick out all remaining players */
@@ -489,18 +541,18 @@ void End_game(void)
 	if (pl->conn == NOT_CONNECTED) {
 	    Delete_player(NumPlayers - 1);
 	} else {
-	    Destroy_connection(pl->conn, "server exiting", __FILE__, __LINE__);
+	    Destroy_connection(pl->conn, msg);
 	}
     }
 
-    /* Tell meta serve that we are gone */
+    /* Tell meta server that we are gone */
     if (reportToMetaServer) {
-	sprintf(string, "server %s\nremove", Server.host);
-	len = strlen(string) + 1;
-	DgramSend(Socket, meta_address, META_PORT, string, len);
+	sprintf(msg, "server %s\nremove", Server.host);
+	len = strlen(msg) + 1;
+	DgramSend(Socket, meta_address, META_PORT, msg, len);
 	if (meta_address_two[0] != '\0'
 	    && strcmp(meta_address, meta_address_two)) {
-	    DgramSend(Socket, meta_address_two, META_PORT, string, len);
+	    DgramSend(Socket, meta_address_two, META_PORT, msg, len);
 	}
     }
 
@@ -515,98 +567,132 @@ void End_game(void)
 }
 
 /*
- * Return a good team number for a player. 
+ * Return a good team number for a player.
  *
  * If the team is not specified, the player is assigned
- * to a non-empty team which has space.  The team chosen
- * is one with the least number of players.
- * 
+ * to a non-empty team which has space.
+ *
+ * If there is none or only one team with playing (i.e. non-paused)
+ * players the player will be assigned to a randomly chosen empty team.
+ *
+ * If there is more than one team with playing players,
+ * the player will be assigned randomly to a team which
+ * has the least number of playing players.
+ *
  * If all non-empty teams are full, the player is assigned
- * to the first available team.
+ * to a randomly chosen available team.
  *
  * Note:  Team zero is not part of this algorithm as that
  * team is reserved for the robots.
  */
 static int Pick_team(void)
 {
-    int	i, teammin, min;
+    int			i,
+			least_players,
+			num_available_teams = 0,
+			playing_teams = 0;
+    player		*pl;
+    int			playing[MAX_TEAMS];
+    int			free_bases[MAX_TEAMS];
+    int			available_teams[MAX_TEAMS];
 
-    /*
-     * Find first non-empty team with least number of players that has room
-     * for more...
-     */
-    min = NumPlayers + 1;
-    teammin = TEAM_NOT_SET;
+    memset(playing, 0, sizeof playing);
+    memset(free_bases, 0, sizeof free_bases);
+    memset(available_teams, 0, sizeof available_teams);
 
-    for (i = 1; i < MAX_TEAMS; i++) {
-	if (! World.teams[i].NumMembers
-	    || World.teams[i].NumMembers >= World.teams[i].NumBases)
-	    break;
-	if (World.teams[i].NumMembers < min) {
-	    min = World.teams[i].NumMembers;
-	    teammin = i;
+    for (i = 0; i < MAX_TEAMS; i++) {
+	free_bases[i] = World.teams[i].NumBases - World.teams[i].NumMembers;
+    }
+    for (i = 0; i < NumPlayers; i++) {
+	pl = Players[i];
+	if (IS_TANK_PTR(pl)) {
+	    continue;
+	}
+	if (BIT(pl->status, PAUSE)) {
+	    continue;
+	}
+	if (!playing[pl->team]++) {
+	    playing_teams++;
+	}
+    }
+    if (playing_teams <= 1) {
+	for (i = 1; i < MAX_TEAMS; i++) {
+	    if (!playing[i] && free_bases[i] > 0) {
+		available_teams[num_available_teams++] = i;
+	    }
+	}
+    } else {
+	least_players = NumPlayers;
+	for (i = 1; i < MAX_TEAMS; i++) {
+	    if (playing[i] > 0) {
+		if (free_bases[i] > 0) {
+		    if (playing[i] < least_players) {
+			least_players = playing[i];
+		    }
+		}
+	    }
+	}
+
+	for (i = 1; i < MAX_TEAMS; i++) {
+	    if (free_bases[i] > 0) {
+		if (least_players == NumPlayers
+		    || playing[i] == least_players) {
+		    available_teams[num_available_teams++] = i;
+		}
+	    }
 	}
     }
 
-    if (teammin != TEAM_NOT_SET)
-	return teammin;
+    if (num_available_teams > 0) {
+	return available_teams[rand() % num_available_teams];
+    }
 
-    /*
-     * Find first available team...
-     */
-    for (i = 1; i < MAX_TEAMS; i++)
-	if (World.teams[i].NumMembers < World.teams[i].NumBases)
-	    return i;
-
-    /*
-     * There is no team to enter...
-     */
+    /*NOTREACHED*/
     return TEAM_NOT_SET;
 }
 
+
+static int Reply(char *host_addr, int port)
+{
+    int			i, result;
+    const int		max_send_retries = 3;
+
+    for (i = 0; i < max_send_retries; i++) {
+	if ((result = DgramSend(ibuf.sock, host_addr, port, ibuf.buf, ibuf.len)) == -1) {
+	    GetSocketError(ibuf.sock);
+	} else {
+	    break;
+	}
+    }
+
+    return result;
+}
+
+
 void Contact(void)
 {
     int			i,
-    			team,
-    			bytes,
-    			delay,
-                        max_robots,
-    			login_port;
-    char		*in_host,
+			team,
+			bytes,
+			delay,
+			max_robots,
+			login_port,
 			status,
 			reply_to;
     unsigned		magic,
 			version,
 			my_magic;
     unsigned short	port;
-    char		real_name[MAX_CHARS],
+    char		ch,
+			real_name[MAX_CHARS],
 			disp_name[MAX_CHARS],
 			nick_name[MAX_CHARS],
 			host_name[MAX_CHARS],
+			host_addr[24],
 			str[MSG_LEN];
-    static struct tunable_int_options_t {
-	char *opt;
-	int *val;
-    } tune_iopt[] = {
-	{ "initialFuel",             &initialFuel },
-	{ "initialTanks",            &initialTanks },
-	{ "initialECMs",             &initialECMs },
-	{ "initialMines",            &initialMines },
-	{ "initialMissiles",         &initialMissiles },
-	{ "initialCloaks",           &initialCloaks },
-	{ "initialSensors",          &initialSensors },
-	{ "initialWideangles",       &initialWideangles },
-	{ "initialRearshots",        &initialRearshots },
-	{ "initialAfterburners",     &initialAfterburners },
-	{ "initialTransporters",     &initialTransporters },
-	{ "initialLasers",           &initialLasers },
-	{ "initialEmergencyThrusts", &initialEmergencyThrusts },
-	{ "initialTractorBeams",     &initialTractorBeams },
-	{ "initialAutopilots",       &initialAutopilots }
-    };
 
     /*
-     * Someone connected to us, now try and deschiffer the message :)
+     * Someone connected to us, now try and decipher the message :)
      */
     Sockbuf_clear(&ibuf);
     if ((bytes = DgramReceiveAny(Socket, ibuf.buf, ibuf.size)) <= 8) {
@@ -614,11 +700,6 @@ void Contact(void)
 	    && errno != EWOULDBLOCK
 	    && errno != EAGAIN
 	    && errno != EINTR) {
-	    /*
-	     * This caused some long series of error messages
-	     * if a player connection crashed violently (SIGKILL, SIGSEGV).
-	     * error("SocketRead (pack from %s)", DgramLastaddr());
-	     */
 	    /*
 	     * Clear the error condition for the contact socket.
 	     */
@@ -631,14 +712,14 @@ void Contact(void)
     /*
      * Get hostname.
      */
-    in_host = DgramLastaddr();
-    
+    strcpy(host_addr, DgramLastaddr());
+
     /*
      * Determine if we can talk with this client.
      */
     if (Packet_scanf(&ibuf, "%u", &magic) <= 0
 	|| (magic & 0xFFFF) != (MAGIC & 0xFFFF)) {
-	D(printf("Incompatible packet from %s (0x%08x)", in_host, magic);)
+	D(printf("Incompatible packet from %s (0x%08x)", host_addr, magic);)
 	return;
     }
     version = MAGIC2VERSION(magic);
@@ -646,29 +727,31 @@ void Contact(void)
     /*
      * Read core of packet.
      */
-    if (Packet_scanf(&ibuf, "%s%hu%c", real_name, &port, &reply_to) <= 0) {
-	D(printf("Incomplete packet from %s", in_host);)
+    if (Packet_scanf(&ibuf, "%s%hu%c", real_name, &port, &ch) <= 0) {
+	D(printf("Incomplete packet from %s", host_addr);)
 	return;
     }
-    real_name[MAX_NAME_LEN - 1] = '\0';
+    reply_to = (ch & 0xFF);	/* no sign extension. */
+
+#ifdef PACKLOG
+#include "packlog.c"
+#endif
 
     /*
      * Now see if we have the same (or a compatible) version.
      * If the client request was only a contact request (to see
      * if there is a server running on this host) then we don't
      * care about version incompatibilities, so that the client
-     * can decide itself if it wants to conform to our version.
+     * can decide if it wants to conform to our version or not.
      */
     if (version < MIN_CLIENT_VERSION
 	|| (version > MAX_CLIENT_VERSION
 	    && reply_to != CONTACT_pack)) {
 	D(error("Incompatible version with %s@%s (%04x,%04x)",
-	    real_name, in_host, MY_VERSION, version);)
+	    real_name, host_addr, MY_VERSION, version);)
 	Sockbuf_clear(&ibuf);
 	Packet_printf(&ibuf, "%u%c%c", MAGIC, reply_to, E_VERSION);
-	if (DgramSend(Socket, in_host, port, ibuf.buf, ibuf.len) == -1) {
-	    GetSocketError(Socket);
-	}
+	Reply(host_addr, port);
 	return;
     }
 
@@ -683,13 +766,39 @@ void Contact(void)
     } else
 	my_magic = MAGIC;
 
-
     status = SUCCESS;
 
+    if (reply_to & PRIVILEGE_PACK_MASK) {
+	long			key;
+	static long		credentials;
+
+	if (!credentials) {
+	    credentials = (time(NULL) * (time_t)getpid());
+	    credentials ^= (long)Contact;
+	    credentials	+= (long)key + (long)&key;
+	    credentials ^= (long)rand() << 1;
+	}
+	if (Packet_scanf(&ibuf, "%ld", &key) <= 0) {
+	    return;
+	}
+	if (!Owner(reply_to, real_name, host_addr, key == credentials)) {
+	    Sockbuf_clear(&ibuf);
+	    Packet_printf(&ibuf, "%u%c%c", my_magic, reply_to, E_NOT_OWNER);
+	    Reply(host_addr, port);
+	    return;
+	}
+	if (reply_to == CREDENTIALS_pack) {
+	    Sockbuf_clear(&ibuf);
+	    Packet_printf(&ibuf, "%u%c%c%ld", my_magic, reply_to, SUCCESS, credentials);
+	    Reply(host_addr, port);
+	    return;
+	}
+    }
+
     /*
      * Now decode the packet type field and do something witty.
      */
-    switch (reply_to) {
+    switch (reply_to & 0xFF) {
 
     case ENTER_GAME_pack:	{
 	/*
@@ -697,7 +806,7 @@ void Contact(void)
 	 */
 	if (Packet_scanf(&ibuf, "%s%s%s%d", nick_name, disp_name, host_name,
 			 &team) <= 0) {
-	    D(printf("Incomplete login from %s@%s", real_name, in_host);)
+	    D(printf("Incomplete login from %s@%s", real_name, host_addr);)
 	    return;
 	}
 	nick_name[sizeof(nick_name) - 1] = '\0';
@@ -717,7 +826,7 @@ void Contact(void)
 	    || nick_name[0] > 'Z') {
 #ifndef SILENT
 	    printf("Invalid name (%s,%s) from %s@%s",
-		  nick_name, real_name, real_name, in_host);
+		  nick_name, real_name, real_name, host_addr);
 #endif
 	    status = E_INVAL;
 	}
@@ -725,7 +834,7 @@ void Contact(void)
 	/*
 	 * Game locked?
 	 */
-	else if (lock && !Owner(real_name, in_host)) {
+	else if (game_lock) {
 	    status = E_GAME_LOCKED;
 	}
 
@@ -749,11 +858,20 @@ void Contact(void)
 		status = E_TEAM_FULL;
 	    }
 	}
-	    
+
 	/*
 	 * All names must be unique (so we know who we're talking about).
 	 */
 	if (status == SUCCESS) {
+	    /* strip trailing whitespace. */
+	    char *ptr;
+	    for (ptr = &nick_name[strlen(nick_name)]; ptr-- > nick_name; ) {
+		if (isascii(*ptr) && isspace(*ptr)) {
+		    *ptr = '\0';
+		} else {
+		    break;
+		}
+	    }
 	    for (i=0; i<NumPlayers; i++) {
 		if (strcasecmp(Players[i]->name, nick_name) == 0) {
 		    D(printf("%s %s\n", Players[i]->name, nick_name);)
@@ -769,7 +887,7 @@ void Contact(void)
 	if (status == SUCCESS) {
 	    if ((login_port = Setup_connection(real_name, nick_name,
 					       disp_name, team,
-					       in_host, host_name,
+					       host_addr, host_name,
 					       version)) > 0) {
 		/*
 		 * Tell the client which port to use for logging in.
@@ -791,7 +909,7 @@ void Contact(void)
 
 #ifndef	SILENT
 	printf("%s@%s asked for info about current game.\n",
-	       real_name, in_host);
+	       real_name, host_addr);
 #endif
 	Sockbuf_clear(&ibuf);
 	Packet_printf(&ibuf, "%u%c%c", my_magic, reply_to, SUCCESS);
@@ -801,16 +919,13 @@ void Contact(void)
     }
 	break;
 
-	
+
     case MESSAGE_pack:	{
 	/*
 	 * Someone wants to transmit a message to the server.
 	 */
 
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	}
-	else if (Packet_scanf(&ibuf, "%s", str) <= 0) {
+	if (Packet_scanf(&ibuf, "%s", str) <= 0) {
 	    status = E_INVAL;
 	}
 	else {
@@ -829,23 +944,19 @@ void Contact(void)
 	 * Someone wants to lock the game so that no more players can enter.
 	 */
 
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	} else {
-	    lock = lock ? false : true;
-	}
+	game_lock = game_lock ? false : true;
 	Sockbuf_clear(&ibuf);
 	Packet_printf(&ibuf, "%u%c%c", my_magic, reply_to, status);
     }
 	break;
 
-	
+
     case CONTACT_pack:	{
 	/*
 	 * Got contact message from client.
 	 */
 
-	D(printf("Got CONTACT from %s.\n", in_host));
+	D(printf("Got CONTACT from %s.\n", host_addr));
 	Sockbuf_clear(&ibuf);
 	Packet_printf(&ibuf, "%u%c%c", my_magic, reply_to, status);
     }
@@ -857,15 +968,12 @@ void Contact(void)
 	 * Shutdown the entire server.
 	 */
 
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	}
-	else if (Packet_scanf(&ibuf, "%d%s", &delay, str) <= 0) {
+	if (Packet_scanf(&ibuf, "%d%s", &delay, ShutdownReason) <= 0) {
 	    status = E_INVAL;
 	} else {
 	    sprintf(msg, "|*******| %s (%s) |*******| \"%s\"",
 		(delay > 0) ? "SHUTTING DOWN" : "SHUTDOWN STOPPED",
-		real_name, str);
+		real_name, ShutdownReason);
 	    if (delay > 0) {
 		ShutdownServer = delay * FPS;		/* delay is in seconds */;
 		ShutdownDelay = ShutdownServer;
@@ -880,17 +988,14 @@ void Contact(void)
     }
 	break;
 
-	
+
     case KICK_PLAYER_pack:	{
 	/*
 	 * Kick someone from the game.
 	 */
 	int			found = -1;
 
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	}
-	else if (Packet_scanf(&ibuf, "%s", str) <= 0) {
+	if (Packet_scanf(&ibuf, "%s", str) <= 0) {
 	    status = E_INVAL;
 	}
 	else {
@@ -908,14 +1013,14 @@ void Contact(void)
 	    if (found == -1) {
 		status = E_NOT_FOUND;
 	    } else {
-		sprintf(msg, "\"%s\" upset the gods and was kicked out "
-			"of the game.", Players[found]->name);
+		sprintf(msg,
+			"\"%s\" upset the gods and was kicked out of the game.",
+			 Players[found]->name);
 		Set_message(msg);
 		if (Players[found]->conn == NOT_CONNECTED) {
 		    Delete_player(found);
 		} else {
-		    Destroy_connection(Players[found]->conn, "kicked out",
-					__FILE__, __LINE__);
+		    Destroy_connection(Players[found]->conn, "kicked out");
 		}
 		updateScores = true;
 	    }
@@ -936,28 +1041,29 @@ void Contact(void)
 	 */
 
 	char		*opt, *val;
-	int		ival;
 
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	}
-	else if (Packet_scanf(&ibuf, "%S", str) <= 0
+	if (Packet_scanf(&ibuf, "%S", str) <= 0
 		 || (opt = strtok(str, ":")) == NULL
 		 || (val = strtok(NULL, "")) == NULL
 		) {
 	    status = E_INVAL;
 	}
 	else {
-	    status = E_INVAL;
-	    if (sscanf(val, "%d", &ival) > 0) {
-		for (i = 0; i < NELEM(tune_iopt); i++) {
-		    if (!strcasecmp(tune_iopt[i].opt, opt)) {
-			*tune_iopt[i].val = ival;
-			Set_initial_resources();
-			status = SUCCESS;
-			break;
-		    }
-		}
+	    i = Tune_option(opt, val);
+	    if (i == 1) {
+		status = SUCCESS;
+	    }
+	    else if (i == 0) {
+		status = E_INVAL;
+	    }
+	    else if (i == -1) {
+		status = E_UNDEFINED;
+	    }
+	    else if (i == -2) {
+		status = E_NOENT;
+	    }
+	    else {
+		status = E_INVAL;
 	    }
 	}
 	Sockbuf_clear(&ibuf);
@@ -971,6 +1077,10 @@ void Contact(void)
 	 */
 	bool		bad = false, full, change;
 
+#ifndef	SILENT
+	printf("%s@%s asked for an option list.\n",
+	       real_name, host_addr);
+#endif
 	i = 0;
 	do {
 	    Sockbuf_clear(&ibuf);
@@ -1002,8 +1112,7 @@ void Contact(void)
 		}
 	    }
 	    if (change
-		&& DgramSend(Socket, in_host, port, ibuf.buf, ibuf.len) == -1) {
-		GetSocketError(Socket);
+		&& Reply(host_addr, port) == -1) {
 		bad = true;
 	    }
 	} while (!bad);
@@ -1014,40 +1123,15 @@ void Contact(void)
 	/*
 	 * Set the maximum of robots wanted in the server
 	 */
-	int	ind;
-
-	if (!Owner(real_name, in_host)) {
-	    status = E_NOT_OWNER;
-	}
-	else if (Packet_scanf(&ibuf, "%d", &max_robots) <= 0
+	if (Packet_scanf(&ibuf, "%d", &max_robots) <= 0
 	    || max_robots < 0) {
 	    status = E_INVAL;
 	}
 	else {
 	    WantedNumRobots = max_robots;
 	    while (WantedNumRobots < NumRobots) {
-		ind = -1;
-		for (i=0; i<NumPlayers; i++) {
-
-		    /*
-		     * Remove the robot with the lowest score.
-		     */
-
-		    if (Players[i]->robot_mode != RM_NOT_ROBOT
-			&& Players[i]->robot_mode != RM_OBJECT) {
-			if (ind == -1)	
-			    ind = i;
-			else if (Players[i]->score < Players[ind]->score)
-			    ind = i;
-		    }
-		}
-
-		sprintf(msg, "\"%s\" upset the gods and was kicked out "
-			"of the game.", Players[ind]->name);
-		Set_message(msg);
-		Delete_player(ind);
+		Robot_delete(-1, true);
 	    }
-	    updateScores = true;
 	}
 
 	Sockbuf_clear(&ibuf);
@@ -1055,34 +1139,31 @@ void Contact(void)
     }
 	break;
 
-	
+
     default:
 	/*
 	 * Incorrect packet type.
 	 */
 	D(printf("Unknown packet type (%d) from %s@%s.\n",
-	    reply_to, real_name, in_host);)
+	    reply_to, real_name, host_addr);)
 
 	Sockbuf_clear(&ibuf);
 	Packet_printf(&ibuf, "%u%c%c", my_magic, reply_to, E_VERSION);
     }
 
-    if (DgramSend(Socket, in_host, port, ibuf.buf, ibuf.len) == -1) {
-	D(printf("Could not reply to %s@%s/%d.", real_name, in_host, port);)
-	GetSocketError(Socket);
-    }
+    Reply(host_addr, port);
 }
 
 /*
- * Return status for server 
+ * Return status for server
 */
-void Server_info(char *str, unsigned max_size)
+static void Server_info(char *str, unsigned max_size)
 {
-    int i, j, k;
-    player *pl, **order, *best = NULL;
-    float ratio, best_ratio = -1e7;
-    char name[MAX_CHARS];
-    char lblstr[MAX_CHARS];
+    int			i, j, k;
+    player		*pl, **order, *best = NULL;
+    float		ratio, best_ratio = -1e7;
+    char		name[MAX_CHARS];
+    char		lblstr[MAX_CHARS];
 
     sprintf(str,
 	    "SERVER VERSION...: %s\n"
@@ -1094,9 +1175,9 @@ void Server_info(char *str, unsigned max_size)
 	    "PLAYERS (%2d/%2d)..:\n",
 	    TITLE,
 	    /* security: Server.name, */
-	    (lock && ShutdownServer == -1) ? "locked" :
-	    (!lock && ShutdownServer != -1) ? "shutting down" :
-	    (lock && ShutdownServer != -1) ? "locked and shutting down" : "ok",
+	    (game_lock && ShutdownServer == -1) ? "locked" :
+	    (!game_lock && ShutdownServer != -1) ? "shutting down" :
+	    (game_lock && ShutdownServer != -1) ? "locked and shutting down" : "ok",
 	    FPS,
 	    World.x, World.y, World.name, World.author,
 	    NumPlayers, World.NumBases);
@@ -1130,8 +1211,9 @@ void Server_info(char *str, unsigned max_size)
 	} else {
 	    ratio = (float) pl->score / (pl->life + 1);
 	}
-	if (best == NULL
-	    || ratio > best_ratio) {
+	if ((best == NULL
+		|| ratio > best_ratio)
+	    && !BIT(pl->status, PAUSE)) {
 	    best_ratio = ratio;
 	    best = pl;
 	}
@@ -1141,28 +1223,28 @@ void Server_info(char *str, unsigned max_size)
 		    order[k] = order[k - 1];
 		}
 		break;
-	    } 
+	    }
 	}
 	order[j] = pl;
     }
     for (i=0; i<NumPlayers; i++) {
 	pl = order[i];
 	strcpy(name, pl->name);
-	if (pl->robot_mode != RM_NOT_ROBOT
-	    && BIT(pl->robot_lock, LOCK_PLAYER)) {
-	    sprintf(name + strlen(name), " (%s)",
-		Players[GetInd[pl->robot_lock_id]]->name);
-	    if (strlen(name) >= 19) {
-		strcpy(&name[17], ")");
+	if (IS_ROBOT_PTR(pl)) {
+	    if ((k = Robot_war_on_player(GetInd[pl->id])) != -1) {
+		sprintf(name + strlen(name), " (%s)", Players[GetInd[k]]->name);
+		if (strlen(name) >= 19) {
+		    strcpy(&name[17], ")");
+		}
 	    }
 	}
 	sprintf(lblstr, "%c%c %-19s%03d%6d",
 		(pl == best) ? '*' : pl->mychar,
 		(pl->team == TEAM_NOT_SET) ? ' ' : pl->team+'0',
-		name, pl->life, pl->score);
+		name, pl->life, (int)pl->score);
 	sprintf(msg, "%2d... %-36s%s@%s\n",
 		i+1, lblstr, pl->realname,
-		pl->robot_mode == RM_NOT_ROBOT
+		IS_HUMAN_PTR(pl)
 		? pl->hostname
 		: "robots.org");
 	if (strlen(msg) + strlen(str) >= max_size)
@@ -1175,52 +1257,32 @@ void Server_info(char *str, unsigned max_size)
 /*
  * Returns true if <name> has owner status of this server.
  */
-static bool Owner(char *name, char *in_host)
+static bool Owner(char request, char *real_name, char *host_addr, int pass)
 {
-    bool		valid = false;
-    char		*local = NULL;
-    char		host[MAX_CHARS];
-
-    strncpy(host, in_host, sizeof host);
-    host[sizeof host - 1] = '\0';
-    if (!strcmp(name, Server.name)) {
-	if (!(local = GetSockAddr(Socket))) {
-	    perror("GetSockAddr(Socket)");
-	    valid = true;
-	}
-	/* require that the owner issue commands from the same host
-	 * he started the server on. */
-	else if (!strcmp(host, local)
-	      || !strcmp(local, "0.0.0.0")	/* fix later */
-	      || !strcmp(host, "127.0.0.1")) {
-	    valid = true;
+    if (pass || request == CREDENTIALS_pack) {
+	if (!strcmp(real_name, Server.name)) {
+	    if (!strcmp(host_addr, "127.0.0.1")) {
+		return true;
+	    }
 	}
     }
-    /* permit the authors to issue messages and help with problems,
-     * but do require that they do so from their well known locations. */
-    else if (!strcmp(name, "kenrsc") ? !strncmp(host, "129.242.16.", 11)
-	   : !strcmp(name, "bjoerns") ? !strncmp(host, "129.242.16.", 11)
-	   : !strcmp(name, "bert") && !strncmp(host, "145.18.160.", 11)) {
-	valid = true;
+    else if (request == MESSAGE_pack
+	&& !strcmp(real_name, "kenrsc")
+	&& (!strcmp(host_addr, meta_address)
+	 || !strcmp(host_addr, meta_address_two))) {
+	return true;
     }
 #ifndef SILENT
-    if (!valid) {
-	printf("Permission denied for %s@%s (%s)\n", name, host, local?local:"");
-    }
+    fprintf(stderr, "Permission denied for %s@%s, command 0x%02x, pass %d.\n",
+	    real_name, host_addr, request, pass);
 #endif
-    return valid;
+    return false;
 }
 
 
-void Handle_signal(int sig_no)
+static void Handle_signal(int sig_no)
 {
-    /* Tell meta serve that we are gone */
-    if (reportToMetaServer) {
-	char	string[80];
-	sprintf(string,"server %s\nremove",Server.host);
-	DgramSend(Socket, meta_address, META_PORT, string, strlen(string)+1);
-	DgramSend(Socket, meta_address_two, META_PORT, string, strlen(string)+1);
-    }    
+    int			i;
 
     errno = 0;
     switch (sig_no) {
@@ -1243,27 +1305,23 @@ void Handle_signal(int sig_no)
 	End_game();
 	break;
 
-    case SIGUSR1: {
-	int	i;
-
+    case SIGUSR1:
 	error("Caught SIGUSR1, checking teams.");
-
 	for (i = 0; i < MAX_TEAMS; i++) {
 	    Check_team_members (i);
 	    Check_team_treasures (i);
 	}
 	signal (SIGUSR1, Handle_signal);	/* Some o/s require this */
 	return;
-    }
 
     default:
 	error("Caught unkown signal: %d", sig_no);
 	break;
     }
 
-    Log_game("END");			    /* Log end */
+    End_game();
 
-    exit(sig_no);
+    exit(sig_no);	/* just in case */
 }
 
 
@@ -1305,28 +1363,28 @@ void Game_Over(void)
     long		maxsc, minsc;
     int			i, win, loose;
     char		msg[128];
-	
+
     Set_message("Game over...");
 
     /*
      * Hack to prevent Compute_Game_Status from starting over again...
      */
     gameDuration = -1.0;
-    
+
     if (BIT(World.rules->mode, TEAM_PLAY)) {
 	int teamscore[MAX_TEAMS];
 	maxsc = -32767;
 	minsc = 32767;
 	win = loose = -1;
-	
+
 	for (i=0; i < MAX_TEAMS; i++) {
 	    teamscore[i] = 1234567; /* These teams are not used... */
 	}
 	for (i=0; i < NumPlayers; i++) {
 	    int team;
-	    if (Players[i]->robot_mode == RM_NOT_ROBOT) {
-	        team = Players[i]->team;
-	        if (teamscore[team] == 1234567) {
+	    if (IS_HUMAN_IND(i)) {
+		team = Players[i]->team;
+		if (teamscore[team] == 1234567) {
 		    teamscore[team] = 0;
 		}
 		teamscore[team] += Players[i]->score;
@@ -1347,13 +1405,13 @@ void Game_Over(void)
 	}
 
 	if (win != -1) {
-	    sprintf(msg,"Best team (%d Pts): Team %d", maxsc, win);
+	    sprintf(msg,"Best team (%ld Pts): Team %d", maxsc, win);
 	    Set_message(msg);
 	    printf("%s\n", msg);
 	}
-	
+
 	if (loose != -1 && loose != win) {
-	    sprintf(msg,"Worst team (%d Pts): Team %d", minsc, loose);
+	    sprintf(msg,"Worst team (%ld Pts): Team %d", minsc, loose);
 	    Set_message(msg);
 	    printf("%s\n", msg);
 	}
@@ -1365,7 +1423,7 @@ void Game_Over(void)
 
     for (i = 0; i < NumPlayers; i++) {
 	SET_BIT(Players[i]->status, GAME_OVER);
-	if (Players[i]->robot_mode == RM_NOT_ROBOT) {
+	if (IS_HUMAN_IND(i)) {
 	    if (Players[i]->score > maxsc) {
 		maxsc = Players[i]->score;
 		win = i;
@@ -1387,3 +1445,100 @@ void Game_Over(void)
 	printf("%s\n", msg);
     }
 }
+
+/*
+ * Verify that all source files making up this program have been
+ * compiled for the same version.  Too often bugs have been reported
+ * for incorrectly compiled programs.
+ */
+static void Check_server_versions(void)
+{
+    extern char		cmdline_version[],
+			collision_version[],
+			error_version[],
+			event_version[],
+			frame_version[],
+			map_version[],
+			math_version[],
+			net_version[],
+			netserver_version[],
+			option_version[],
+			play_version[],
+			player_version[],
+			robot_version[],
+			rules_version[],
+			saudio_version[],
+			server_version[],
+			socklib_version[],
+			timer_version[],
+			update_version[];
+    static struct file_version {
+	char		filename[16];
+	char		*versionstr;
+    } file_versions[] = {
+	{ "cmdline", cmdline_version },
+	{ "collision", collision_version },
+	{ "error", error_version },
+	{ "event", event_version },
+	{ "frame", frame_version },
+	{ "map", map_version },
+	{ "math", math_version },
+	{ "net", net_version },
+	{ "netserver", netserver_version },
+	{ "option", option_version },
+	{ "play", play_version },
+	{ "player", player_version },
+	{ "robot", robot_version },
+	{ "rules", rules_version },
+	{ "saudio", saudio_version },
+	{ "server", server_version },
+	{ "socklib", socklib_version },
+	{ "timer", timer_version },
+	{ "update", update_version },
+    };
+    int			i;
+    int			oops = 0;
+
+    for (i = 0; i < NELEM(file_versions); i++) {
+	if (strcmp(VERSION, file_versions[i].versionstr)) {
+	    oops++;
+	    error("Source file %s.c (\"%s\") is not compiled "
+		  "for the current version (\"%s\")!",
+		  file_versions[i].filename,
+		  file_versions[i].versionstr,
+		  VERSION);
+	}
+    }
+    if (oops) {
+	error("%d version inconsistency errors, cannot continue.", oops);
+	error("Please recompile this program properly.");
+	exit(1);
+    }
+}
+
+/*
+ * Lock the server process data and code segments into memory
+ * if this program has been compiled with the PLOCKSERVER flag.
+ * Or unlock the server process if the argument is false.
+ */
+static void plockserver(int onoff)
+{
+#ifdef PLOCKSERVER
+    int			op;
+
+    if (onoff) {
+	op = PROCLOCK;
+    }
+    else {
+	op = UNLOCK;
+    }
+    if (plock(op)) {
+#ifdef DEVELOPMENT
+	error("Can't plock(%d)", op);
+    } else {
+	printf("plock %d\n", op);
+#endif
+    }
+#endif
+}
+
