@@ -14,414 +14,7 @@
  *
  * This software is provided "as is" without any express or implied warranty.
  *
- * 1993/05/22 bert
- * Changed SocketReadable to check for return value of select()
- * and changed unused fd sets to NULL pointers.
- *
- * Revision so-and-so, Id: bla-bla, Code: too.  Bert Gsbers April/Mai 93
- * Added several interface calls to get/setsockopt/ioctl calls for use
- * in the client/server version of XPilot.
- * Also, (Please Note!) I removed the linger stuff from the SocketAccept()
- * function into a separate function call.  When a socket is non-blocking
- * then lingering on close didn't seem like a good idea to me.
- *
- * RCS:      $Id: socklib.c,v 3.31 1994/04/14 09:44:19 bert Exp $
- * Log:      $Log: socklib.c,v $
- * Revision 3.31  1994/04/14  09:44:19  bert
- * Commented out _res usage for Solaris.
- *
- * Revision 3.30  1994/04/13  15:17:52  bert
- * Changed conditional compilation for SetSocketNonBlocking.
- *
- * Revision 3.29  1994/04/11  12:22:26  bert
- * Added another (the fourth!) way to make a socket non-blocking.
- *
- * Revision 3.28  1994/04/10  13:20:38  bert
- * Extended the way sockets are made non-blocking by trying three different
- * possibilities depending upon which system include file constants are defined.
- * Now fcntl(O_NDELAY), fcntl(FNDELAY) and ioctl(FIONBIO) are all tried until
- * one of them succeeds.
- * Improved GetLocalHostName for suns.
- *
- * Revision 3.27  1994/03/23  08:46:21  bert
- * Fixed GetLocalHostName by adding a size parameter.
- *
- * Revision 3.26  1994/03/22  19:01:40  bert
- * Fixed MAXHOSTNAMELEN dependencies.
- *
- * Revision 3.25  1994/02/06  18:18:38  bert
- * Fixed typo.
- *
- * Revision 3.24  1994/02/04  16:11:30  bjoerns
- * Minor.
- *
- * Revision 3.23  1994/01/27  22:28:38  bert
- * Added a new call GetLocalHostName() to get the fully qualified local hostname.
- *
- * Revision 3.22  1994/01/23  14:15:23  bert
- * setsockopt needs a (void *) argument, not (char *).
- * Added a SetSocketBroadcast() function.
- *
- * Revision 3.21  1994/01/20  21:20:09  bert
- * Changes to get the prototypes when compiling with C++.
- * Small change for C++ functionprototypes and the select system call.
- *
- * Revision 3.20  1993/12/23  12:29:46  bert
- * Applied a patch from Stig Bakken for Solaris compatibility ifdefs.
- *
- * Revision 3.19  1993/12/19  19:04:32  bert
- * Simplified a comment.
- *
- * Revision 3.18  1993/11/16  22:40:09  bert
- * Added an #ifdef TCP_NODELAY for the use of TCP_NODELAY to prevent any more
- * silly patches for it from people not having TCP_NODELAY.
- * Hardcoded the printing of the source filename for Suns.  *sigh*.
- *
- * Revision 3.17  1993/10/24  22:33:09  bert
- * Changed a __FILE__ expression to "socklib.c" as Suns don't seem to support
- * the __FILE__ construct.  Sigh.
- * Added a DgramReply() routine, not used yet.
- *
- * Revision 3.16  1993/10/21  11:08:54  bert
- * VMS patch from Curt Hjorring.
- * Removed Optimize_map() from the server.
- * Made toggleShield a new client option.
- *
- * Revision 3.15  1993/10/01  20:47:15  bjoerns
- * Added Thrusted Solaris CMW 1.0 patch by Steve Marsden.
- *
- * Revision 3.14  1993/09/20  18:46:17  bert
- * Changes to have XPilot compiled by g++, both under Linux and HP-UX 9.01.
- * Added new entries in Makefile.std for C++ compilation.
- * Several include files added.  Missing prototypes were added to include
- * files.  Several casts to shut up the compiler.  Changes to socklib.c
- * for bad function pointers.  Some obscure functions were still written
- * in K&R style.  A few small changes to make some statements valid C++ code.
- *
- * Revision 3.13  1993/08/24  12:17:30  bjoerns
- * Summary provided by Bert:
- *
- * 1) Lasers.
- * #define MAX_LASERS              8
- * #define MIN_LASER_LEN           100
- * #define MAX_LASER_LEN           350
- * #define EXTRA_LASER_LEN         ((MAX_LASER_LEN-MIN_LASER_LEN)/(MAX_LASERS-1))
- * #define LASER_LENGTH(lasers)    (MIN_LASER_LEN + ((lasers)-1)*EXTRA_LASER_LEN)
- * #define LASER_COST(lasers)      (ED_LASER*LASER_LENGTH(lasers)/MAX_LASER_LEN)
- * #define ED_LASER                (-80.0*FUEL_SCALE_FACT)
- * #define ED_LASER_HIT            (-120.0*FUEL_SCALE_FACT)
- * Lasers costs quite a lot of fuel and it remains to be seen if they
- * are useful at all.  But I didn't want to risk introducing a weapon
- * which was too heavy.  So rather start low profile then.
- * Robots defend against lasers and use them if possible.
- * The default key for firing a laser is the slash key.
- *
- * 2) Add player info to the Player packet for new to be developed
- * player info window.
- *
- * 3) Sliding radar is now also selectable from the configure window.
- *
- * 4) Fix for player mutual crash scores.  The scores for the second player
- * were disadvantaged by the scores just calculated for the first player.
- * This depended upon the order of the players in the Players[] array.
- *
- * 5) Removed all pl->fuel.count = FUEL_NOTIFY; statements from the server
- * as this is a client only business.
- *
- * 6) Moved the robot war declarations from collision.c to robot.c in a
- * new function because it was also needed in LaserCollision().
- *
- * 7) The configuration window stuff is now also savable to $(HOME)/.xpilotrc.
- * It checks to only replace resources that it understands and will add
- * resources that were missing.  The screen related resources are not
- * saved because they may have been derived from a X display particular
- * resource file.
- *
- * 8) Changed most of the `double buffer' names into 'color switch'
- * were appropriate.
- *
- * 9) Changed PACK version into 0x3041 to differentiate from the 0x3040 versions.
- *
- * 10) Changed About window to create room for the new Laser item info.
- * The window is now also somewhat smaller to fit into 1024x768 screens.
- *
- * 11) All the sliding radar X stuff is now move into a separate subroutine
- * in paint.c.
- *
- * 12) The rand() calls in Place_item for gravity items wasn't doing anything.
- * It read: (((rand()&7)-3)/4) which always evaluates to zero.
- *
- * 13) ECMs also damage lasers.
- *
- * 14) Fixed a score update bug introduced by a previous 3.0.4 patch.
- *
- * 15) Increased the robot manouvring lookahead a little, because robots
- * crashed into the walls too often after the previous lookahead limitation.
- *
- * 16) Some of the robot idle mode manouvring compensates for gravity a little.
- *
- * 17) New easy to use confirmation popup widget for short messages.
- *
- * 18) Changes to the wideangle item to have three streams of bullets.
- * Otherwise it looked too much the same as the laser item.
- *
- * 19) When installing a new colormap try to allocate the first 16 colors
- * the same as the default colormap to reduce the chance that the window
- * manager decorations and other colors will flicker because of the
- * color switching.
- *
- * 20) If the default visual is DirectColor or TrueColor then automatically.
- * try to use a PseudoColor visual.
- *
- * 21) Fix for the CONFIG/SCORE label in the menu.  The bug was pointed
- * out by Fred Hucht.
- *
- * 22) The client accepts now more then one server on the command line.
- *
- * Revision 3.12  1993/08/19  07:35:25  kenrsc
- * Added patch from bert (3f4changes)
- *
- * Revision 3.11  1993/08/16  17:03:42  kenrsc
- * Changes by Bert (see patch 3a4 for more info
- *
- * Revision 3.10  1993/08/15  00:33:34  kenrsc
- * Changed back soem of the code for the meta server. Fixed a small bug.
- *
- * Revision 3.9  1993/08/07  02:23:45  kenrsc
- * Made the extra fields reported to the meta server.
- * Made the readable address available to the meta server and the server.
- * Made the server report to the meta server when player numbers changes.
- * Changed the field called dispname to hostname since it is not a dispname
- * but a host name :)
- *
- * Revision 3.8  1993/08/06  21:21:00  bjoerns
- * 1) Some tweaks to the new client option code like a -xrm option.
- * 2) Correction for maximum string size Get_resource.
- * 3) Fix for the /usr/X11/lib/app-defaults resource path.
- * 4) Fix for NULL pointer dereference if neither XUSERFILESEARCHPATH
- * or XAPPLRESDIR were defined in the environment.
- * Thanks to welch@xcf.Berkeley.EDU (Sean N. Welch).
- * 5) Limit the receiveWindowSize.
- * 6) Destroying the resource database after use (memory reuse).
- * 7) Fix for definition of the BLUE_... bits in paint.c.
- * 8) Fix for the first empty score entry in paint.c.
- *
- * Revision 3.7  1993/08/02  12:55:37  bjoerns
- * Patchlevel 3.
- *
- * Revision 3.6  1993/08/02  12:51:18  bjoerns
- * Patchlevel 2.
- *
- * Revision 3.5  1993/08/02  12:41:41  bjoerns
- * Patchlevel 1.
- *
- * Revision 3.4  1993/06/28  20:54:22  bjoerns
- * Added an Imakefile and renamed Makefile to Makefile.std.
- * Made Bert's last name 8-bit.
- *
- * Revision 3.3  1993/06/28  11:43:36  bjoerns
- * Applied PL8 which was handled completely by Bert Gsbers.
- *
- *
- * SUMMARY OF CHANGES
- *
- * Andy Skinner contributed a new targetKillTeam option.
- * When turned on it will kill all players in the team that just
- * lost their target.  The default value is off/inactive.
- *
- * Any remaining problems  with the login sequence should be fixed
- * due to a thorough revision/rewrite of the receive-window-size code.
- * E.g., the `ufo-problem' reported by Andy should be gone.
- *
- * Player info is now transmitted and stored in a less bugprone way.
- * All problems with player names and radar visibility should
- * now be fixed.
- *
- * The flashing of the 'F' in fuelstations should now be gone.
- * The solution was to use the proper pre-xored pixel value as
- * foreground for xor operations.
- *
- * Drawing the new score objects now also work across edge wrap.
- *
- * Shots are now killed if they hit a cannon.
- *
- * In Move_smart_shot() `min' wasn't allways initialized before use.
- *
- * The PacketDrawMeter is removed (was overkill).
- *
- * The mineDraw resource is removed (was overkill).
- *
- * Mines are now drawn like they already appeared in the new
- * items and HUD instead of a small-blue-almost-invisible circle.
- *
- * Several functions have now a (void) prototype when no arguments
- * to shut up the compiler.
- *
- * The backgroundPointDist resource code is now checking for division
- * by zero.
- *
- * The printing of duplicate reliable data packet info is commented out.
- *
- * The drawing of fuelstations is buffered and optimised to reduce
- * changing of GCs.
- *
- * The '#ifdef SOUND' statement in sound.c is now on top of the sound.c file
- * like suggested in alt.games.xpilot.
- *
- * DrawShadowText() wasn't always returning a value.
- *
- * Small changes to pixmaps for Tank, Transporter and Cloak items.
- *
- * Revision 3.2  1993/06/12  20:35:37  bjoerns
- * Applied yet anoter patch from Bert, this time he has implemented clustering of objects (sparks for the moment).
- *
- * Revision 3.1  1993/05/28  18:04:02  bjoerns
- * Applied new giant patch from Bert.  Here's a summary I got from him:
- *
- * changes to Makefile:
- * 	some AIX flags
- * 	new include file dependencies
- * client.c:
- * 	Target test.
- * 	player info bug fix (already mailed to you).
- * 	More accurate RadarHeight calculation.
- * client.h:
- * 	"self_visible" boolean flag in client structure.
- * 	new prototype for Client_flush().
- * cmdline.c:
- * 	playerStartsShielded flag for PlayerShielding=false mode
- * 	playersOnRadar if players are visible on radar (default=true)
- * 	missilesOnRadar if missiles are visible on radar (default=true)
- * collision.c:
- * 	new interesting experiment with collision detection for objects,
- * 	now only test objects that have already passed the ObjectCollision
- * 	function this timeframe (simple change, should be an improvement I hope)
- * const.h:
- * 	Experiment with hypot() in LENGTH() macro, untested yet.
- * 	TARGET_DEAD_TIME is defined in some amount of FPS.
- * 	A little higher amount of TARGET_DAMAGE and TARGET_REPAIR_PER_FRAME.
- * event.c:
- * 	Changes to  Refuel() and Player_lock_closest(), I did these because
- * 	I hadn't your changes available.  Might be skipped.
- * 	Removed pl->control_count stuff as these are only needed in the client.
- * frame.c:
- * 	Added new visibility detection stuff for map data.
- * 	It didn't work in PL0 at the edges for cannons and fuelstations.
- * 	This one works perfect for cannons and fuelstations, but has a tough
- * 	to find bug for target info.  Target info is only correctly displayed
- * 	if the player ship is above the target, not below or equal.  Strange!
- * 	Changed radar stuff to incorporate the new playersOnRadar and
- * 	missilesOnRadar options.
- * global.h:
- * 	New declarations for the new commandline options.
- * join.c:
- * 	Flushing of display stuff, probably not necessary.
- * 	But David had some kind of strange problem.
- * 	May be removed.
- * net.c:
- * 	Changed ifdefs for memmove as AIX doesn't have that either.
- * 	Also, AIX does not use EWOULDBLOCK but EAGAIN instead.
- * netclient.c:
- * 	Removed Check_packet_type().  Not needed anymore.
- * 	Changed Kernel network buffer sizes ifdefs as AIX has smaller
- * 	socket buffers by default.
- * 	Added stuff for PKT_PLAY to also handle premature frames.
- * 	May evetually not be needed or in a different way.
- * netserver.c:
- * 	Big changes in states a new connection can be in.
- * 	Removed duplicate code.  Other cleanups.
- * 	A little better retransmission code + bug fix which you already have.
- * 	AIX doesn't have strdup().
- * 	More different types of packet dispatch tables.
- * 	Removed the CONN_ACCEPTED state.  Added CONN_DRAIN and CONN_READY
- * 	to wait until client has acked all reliable data.
- * 	Added war stuff for new players.
- * 	Made an upper limit of 5 for maximum input packet reads per frame.
- * 	Some functions now are static.
- * netserver.h:
- * 	See netserver.c.
- * object.h:
- * 	Removed control_count.
- * 	Added shield_time which gives the time a player still has shields left
- * 	after restart if mode is playerShielding=false and
- * 	playerStartsShielded=true.
- * paint.c:
- * 	Add drawing of transporter lines together with drawing of refuel lines
- * 	and connector lines.
- * 	Added proper control_count decrease.
- * 	New radar stuff.  Improved accuracy.  And better radar map drawing.
- * 	Changed drawing color of target damage, but I don't like it.  Damage lines
- * 	are now drawn in opposite color.  See if you dislike it as much as I do.
- * 	No more direction line of self on radar when dead, like Bjoern disliked.
- * 	I experimented somewhat with the drawing of a cross instead, but
- * 	removed it also again.  Now nothing is drawn when dead, just like 2.0.
- * play.c:
- * 	Small fix to remove AIX compiler complaint, it improves code readability
- * 	too I think.  Unimportant.
- * player.c:
- * 	New playerShielding code for playerStartsShielded option.
- * 	Bugfix for race mode when only one player is playing.
- * robot.c:
- * 	New playerShielding code for playerStartsShielded option.
- * server.c:
- * 	Removed RadarHeight stuff as this is only needed in the client.
- * 	Bugfix for Wait_for_new_players() to disable the real-time interval
- * 	timer when noone is playing.  Otherwise all selects() would have been
- * 	interrupted at a rate of FPS.
- * socklib.c:
- * 	Changed SocketReadable() to check for select returning -1.
- * timer.c:
- * 	Dummy routines if defined BUSYLOOP for block_timer() and allow_timer().
- * update.c:
- * 	New transport to home stuff after death.  I really like this and
- * 	am curious to know if you do too.  You now are transported back to
- * 	home with a constant acceleration and deacceleration.  Try it!
- * 	The old code is preserved with an ifdef.
- * xinit.c:
- * 	I think I fixed two syntax bugs, but please have a careful look yourself.
- * 	There were two statements like: if (a = b), instead of: if (a == b).
- *
- * That's all sofar.
- *
- * Bert
- *
- * Revision 3.0  1993/05/21  18:36:39  bjoerns
- * New client server release.
- *
- * Revision 1.3  1993/05/20  17:33:53  kenrsc
- * Bert changes from M -> R version !!!!!
- *
- * Revision 1.2  1993/05/18  16:49:30  kenrsc
- * Berts few changes !
- *
- * Revision 1.1  1993/04/22  10:21:32  bjoerns
- * Moved socklib from lib to src.
- *
- * Revision 1.3  1993/04/14  16:50:41  bjoerns
- * Applied two patches from Bert, one was really big and fixed all known
- * wrap-around problems, and added some timing and robot optimisations.
- * Also fixed other small bugs.
- *
- * Revision 1.2  1993/03/24  03:24:56  bjoerns
- * Patch by Bert, mainly casting to right type.
- *
- * Revision 1.1  1993/03/09  14:33:25  kenrsc
- * Hopefully we won't have a corrupted CVS directory anymore.
- *
- * Revision 1.1.1.1  1993/02/27  14:47:46  bjoerns
- * XPilot v2.0
- *
- * Revision 1.1.1.1  1993/01/19  17:19:59  bjoerns
- * XPilot v1.4
- *
- * Revision 1.4  1992/08/26  19:36:35  bjoerns
- * Incorporated NCD patch.
- *
- * Revision 1.3  1992/06/25  04:31:37  bjoerns
- * Added source id to socklib.
- *
- * Revision 1.2  1992/05/19  21:11:39  bjoerns
- * Minor.
+ * RCS:      $Id: socklib.c,v 3.42 1995/02/02 09:03:29 bert Exp $
  *
  * Revision 1.1.1.1  1992/05/11  12:32:34  bjoerns
  * XPilot v1.0
@@ -429,15 +22,15 @@
  * Revision 1.2  91/10/02  08:38:01  08:38:01  arne (Arne Helme)
  * "ANSI C prototypes added.
  * Timeout interface changed."
- * 
+ *
  * Revision 1.1  91/10/02  08:34:45  08:34:45  arne (Arne Helme)
  * Initial revision
- * 
+ *
  */
 
 #ifndef lint
 static char sourceid[] =
-    "@(#)$Id: ";
+    "@(#)$Id: socklib.c,v 3.42 1995/02/02 09:03:29 bert Exp $";
 #endif
 
 /* _SOCKLIB_LIBSOURCE must be defined int this file */
@@ -497,6 +90,7 @@ static char sourceid[] =
 #endif
 
 /* Socklib Includes And Definitions */
+#include "version.h"
 #include "socklib.h"
 #ifdef SUNCMW
 #include "cmw.h"
@@ -505,6 +99,8 @@ static char sourceid[] =
 #define cmw_priv_deassert_netaccess() /* empty */
 #endif /* SUNCMW */
 
+char socklib_version[] = VERSION;
+
 /* Debug macro */
 #ifdef DEBUG
 #define DEB(x) x
@@ -570,8 +166,8 @@ int			sl_broadcast_enabled = 0;
  *
  * Originally coded by Arne Helme
  */
-#ifdef __STDC__
 void
+#ifdef __STDC__
 SetTimeout(int s, int us)
 #else
 SetTimeout(s, us)
@@ -599,8 +195,8 @@ int s, us;
  *	None
  *
  * Return Value
- *	The function returns the socket descriptor, or -1 if 
- *	any errors occured. 
+ *	The function returns the socket descriptor, or -1 if
+ *	any errors occured.
  *
  * Globals Referenced
  *	sl_errno	- if errors occured: SL_ESOCKET, SL_EBIND,
@@ -638,7 +234,7 @@ int	port;
     addr_in.sin_family		= AF_INET;
     addr_in.sin_addr.s_addr	= INADDR_ANY;
     addr_in.sin_port		= htons(port);
-    
+
     retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
     if (retval < 0)
     {
@@ -688,7 +284,7 @@ int	port;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 GetPortNum(int fd)
 #else
@@ -740,7 +336,7 @@ int	fd;
  *
  * Originally coded by Bert Gijsbers
  */
-char * 
+char *
 #ifdef __STDC__
 GetSockAddr(int fd)
 #else
@@ -791,7 +387,7 @@ int	fd;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 SLGetPeerName(int fd, char *name, int namelen)
 #else
@@ -860,7 +456,7 @@ int	namelen;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 CreateClientSocket(char *host, int port)
 #else
@@ -872,11 +468,11 @@ int	port;
     struct sockaddr_in	peer;
     struct hostent	*hp;
     int			fd;
-    
+
     memset((char *)&peer, 0, sizeof(struct sockaddr_in));
     peer.sin_family = AF_INET;
     peer.sin_port   = htons(port);
-    
+
     peer.sin_addr.s_addr = inet_addr(host);
     if (peer.sin_addr.s_addr == (int)-1)
     {
@@ -897,7 +493,7 @@ int	port;
 	return (-1);
     }
 
-    if (connect(fd, (struct sockaddr *)&peer, sizeof(struct sockaddr_in)) < 0) 
+    if (connect(fd, (struct sockaddr *)&peer, sizeof(struct sockaddr_in)) < 0)
     {
 	sl_errno = SL_ECONNECT;
 	(void) close(fd);
@@ -916,7 +512,7 @@ int	port;
  *******************************************************************************
  * Description
  *	This function is called in a TCP/IP server to accept incoming calls.
- *	
+ *
  * Input Parameters
  *	fd		- The listen socket.
  *
@@ -932,13 +528,13 @@ int	port;
  *
  * External Calls
  *	none
- *	
+ *
  * Called By
  *	User applications.
  *
  * Originally coded by Arne Helme.
  */
-int 
+int
 #ifdef __STDC__
 SocketAccept(int fd)
 #else
@@ -964,7 +560,7 @@ int	fd;
  *******************************************************************************
  * Description
  *	This function is called on a stream socket to set the linger option.
- *	
+ *
  * Input Parameters
  *	fd		- The stream socket to set the linger option on.
  *
@@ -979,13 +575,13 @@ int	fd;
  *
  * External Calls
  *	setsockopt
- *	
+ *
  * Called By
  *	User applications.
  *
  * Originally coded by Arne Helme, but moved out of SocketAccept by Bert.
  */
-int 
+int
 #ifdef __STDC__
 SocketLinger(int fd)
 #else
@@ -993,7 +589,7 @@ SocketLinger(fd)
 int	fd;
 #endif /* __STDC__ */
 {
-#if defined(LINUX) || defined(__linux__)
+#if defined(LINUX0) || !defined(SO_LINGER)
     /*
      * As of 0.99.12 Linux doesn't have LINGER stuff.
      */
@@ -1041,7 +637,7 @@ int	fd;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 SetSocketReceiveBufferSize(int fd, int size)
 #else
@@ -1085,7 +681,7 @@ int	size;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 SetSocketSendBufferSize(int fd, int size)
 #else
@@ -1130,7 +726,7 @@ int	size;
  * Originally coded by Bert Gsbers
  */
 #ifdef TCP_NODELAY
-int 
+int
 #ifdef __STDC__
 SetSocketNoDelay(int fd, int flag)
 #else
@@ -1180,7 +776,7 @@ int	flag;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 SetSocketNonBlocking(int fd, int flag)
 #else
@@ -1235,34 +831,36 @@ int	flag;
 # endif
 #endif
 
+    char buf[128];
+
 #ifdef USE_FCNTL_FNDELAY
     if (fcntl(fd, F_SETFL, (flag != 0) ? FNDELAY : 0) != -1)
 	return 0;
-    fprintf(stderr, "fcntl FNDELAY failed in file \"%s\", line %d: %s\n",
-	    __FILE__, __LINE__, strerror(errno));
+    sprintf(buf, "fcntl FNDELAY failed in socklib.c line %d", __LINE__);
+    perror(buf);
 #endif
- 
+
 #ifdef USE_IOCTL_FIONBIO
     if (ioctl(fd, FIONBIO, &flag) != -1)
 	return 0;
-    fprintf(stderr, "ioctl FIONBIO failed in file \"%s\", line %d: %s\n",
-	    __FILE__, __LINE__, strerror(errno));
+    sprintf(buf, "ioctl FIONBIO failed in socklib.c line %d", __LINE__);
+    perror(buf);
 #endif
- 
+
 #ifdef USE_FCNTL_O_NONBLOCK
     if (fcntl(fd, F_SETFL, (flag != 0) ? O_NONBLOCK : 0) != -1)
 	return 0;
-    fprintf(stderr, "fcntl O_NONBLOCK failed in file \"%s\", line %d: %s\n",
-	    __FILE__, __LINE__, strerror(errno));
+    sprintf(buf, "fcntl O_NONBLOCK failed in socklib.c line %d", __LINE__);
+    perror(buf);
 #endif
- 
+
 #ifdef USE_FCNTL_O_NDELAY
     if (fcntl(fd, F_SETFL, (flag != 0) ? O_NDELAY : 0) != -1)
 	return 0;
-    fprintf(stderr, "fcntl O_NDELAY failed in file \"%s\", line %d: %s\n",
-	    __FILE__, __LINE__, strerror(errno));
+    sprintf(buf, "fcntl O_NDELAY failed in socklib.c line %d", __LINE__);
+    perror(buf);
 #endif
- 
+
     return (-1);
 } /* SetSocketNonBlocking */
 
@@ -1297,7 +895,7 @@ int	flag;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 SetSocketBroadcast(int fd, int flag)
 #else
@@ -1340,7 +938,7 @@ int	flag;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 GetSocketError(int fd)
 #else
@@ -1389,7 +987,7 @@ int	fd;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 SocketReadable(int fd)
 #else
@@ -1410,7 +1008,7 @@ int	fd;
 
     if (select(fd + 1, &readfds, NULL, NULL, &timeout) == -1)
 	return ((errno == EINTR) ? 0 : -1);
-    
+
     if (readfds & (1 << fd))
 	return (1);
     return (0);
@@ -1491,7 +1089,7 @@ static inthandler()
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 SocketRead(int fd, char *buf, int size)
 #else
@@ -1502,7 +1100,7 @@ char	*buf;
 {
     int	ret, ret1;
 
-    if (setjmp(env)) 
+    if (setjmp(env))
     {
 	(void) alarm(0);
 	(void) signal(SIGALRM, SIG_DFL);
@@ -1510,7 +1108,7 @@ char	*buf;
     }
     ret = 0;
     cmw_priv_assert_netaccess();
-    while (ret < size) 
+    while (ret < size)
     {
 	(void) signal(SIGALRM, inthandler);
 	(void) alarm(sl_timeout_s);
@@ -1558,7 +1156,7 @@ char	*buf;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 SocketWrite(int fd, char *buf, int size)
 #else
@@ -1612,7 +1210,7 @@ char	*buf;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 SocketClose(int fd)
 #else
@@ -1668,7 +1266,7 @@ int	fd;
  *
  * Originally coded by Arne Helme
  */
-int 
+int
 #ifdef __STDC__
 CreateDgramSocket(int port)
 #else
@@ -1704,6 +1302,78 @@ int	port;
     return (fd);
 } /* CreateDgramSocket */
 
+
+/*
+ *******************************************************************************
+ *
+ *	CreateDgramAddrSocket()
+ *
+ *******************************************************************************
+ * Description
+ *	Creates a UDP/IP datagram socket on a know interface address.
+ *
+ * Input Parameters
+ *	dotaddr		- Pointer to string containing of IP address in dot-format.
+ *	port		- The port number. A value of zero may be specified in
+ *			  clients to assign any available port number.
+ *
+ * Output Parameters
+ *	None
+ *
+ * Return Value
+ *	A UDP/IP datagram socket descriptor, or -1 on error.
+ *
+ * Globals Referenced
+ *	sl_errno	- If any errors occured: SL_ESOCKET, SL_EBIND.
+ *
+ * External Calls
+ *	socket
+ *	memset
+ *	bind
+ *	close
+ *
+ * Called By
+ *	User applications.
+ *
+ * Originally coded by Bert Gijsbers, adapted from CreateDgramSocket().
+ */
+int
+#ifdef __STDC__
+CreateDgramAddrSocket(char *dotaddr, int port)
+#else
+CreateDgramAddrSocket(dotaddr, port)
+char	*dotaddr;
+int	port;
+#endif /* __STDC__ */
+{
+    struct sockaddr_in	addr_in;
+    int			fd;
+    int			retval;
+
+    fd = socket(AF_INET, SOCK_DGRAM, 0);
+    if (fd < 0)
+    {
+	sl_errno = SL_ESOCKET;
+	return (-1);
+    }
+
+    memset((char *)&addr_in, 0, sizeof(struct sockaddr_in));
+    addr_in.sin_family		= AF_INET;
+    addr_in.sin_addr.s_addr	= inet_addr(dotaddr);
+    addr_in.sin_port		= htons(port);
+    retval = bind(fd, (struct sockaddr *)&addr_in, sizeof(struct sockaddr_in));
+    if (retval < 0)
+    {
+	sl_errno = SL_EBIND;
+	retval = errno;
+	(void) close(fd);
+	errno = retval;
+	return (-1);
+    }
+
+    return (fd);
+} /* CreateDgramAddrSocket */
+
 
 /*
  *******************************************************************************
@@ -1717,7 +1387,7 @@ int	port;
  * Input Parameters
  *	fd		- The socket to operate on.
  *	host		- The host name.
- *	port		- The port number. 
+ *	port		- The port number.
  *
  * Output Parameters
  *	None
@@ -1737,7 +1407,7 @@ int	port;
  *
  * Originally coded by Bert Gsbers
  */
-int 
+int
 #ifdef __STDC__
 DgramConnect(int fd, char *host, int port)
 #else
@@ -1762,7 +1432,7 @@ int	port;
 	    return (-1);
 	}
 	else
-	    addr_in.sin_addr.s_addr = 
+	    addr_in.sin_addr.s_addr =
 		((struct in_addr*)(hp->h_addr))->s_addr;
     }
     addr_in.sin_family		= AF_INET;
@@ -1847,7 +1517,7 @@ char	*host, *sbuf;
 		return (-1);
 	    }
 	    else
-		the_addr.sin_addr.s_addr = 
+		the_addr.sin_addr.s_addr =
 		    ((struct in_addr*)(hp->h_addr))->s_addr;
 	}
     }
@@ -1973,7 +1643,7 @@ char	*from, *rbuf;
 		((struct in_addr*)(hp->h_addr))->s_addr;
     }
     retval = DgramReceiveAny(fd, rbuf, size);
-    if (retval == -1 || 
+    if (retval == -1 ||
 	tmp_addr.sin_addr.s_addr != sl_dgram_lastaddr.sin_addr.s_addr)
     {
 	sl_errno = SL_EWRONGHOST;
@@ -2113,7 +1783,7 @@ DgramInthandler()
  *	signal
  *	DgramSend
  *	DgramReceive
- *	
+ *
  * Called By
  *	User applications.
  *
@@ -2131,7 +1801,7 @@ char	*host, *sbuf, *rbuf;
 {
     int		retval = -1;
     int		retry = sl_default_retries;
-    
+
     (void) signal(SIGALRM, DgramInthandler);
     while (retry > 0)
     {
@@ -2338,37 +2008,103 @@ void GetLocalHostName(name, size)
     unsigned		size;
 #endif /* __STDC__ */
 {
-    struct hostent	*he;
+    struct hostent	*he, *xpilot_he, tmp;
+    int			xpilot_len;
+    char		*alias, *dot;
+    char		xpilot_hostname[MAXHOSTNAMELEN];
+    static const char	xpilot[] = "xpilot";
+
+    xpilot_len = strlen(xpilot);
+
+    /* Make a wild guess that a "xpilot" hostname or alias is in this domain */
+    if ((xpilot_he = gethostbyname(xpilot)) != NULL) {
+	strcpy(xpilot_hostname, xpilot_he->h_name);	/* copy data to buffer */
+	tmp = *xpilot_he;
+	xpilot_he = &tmp;
+    }
 
     gethostname(name, size);
     if ((he = gethostbyname(name)) == NULL) {
 	return;
     }
+    strncpy(name, he->h_name, size);
+    name[size - 1] = '\0';
     /*
      * If there are no dots in the name then we don't have the FQDN,
      * and if the address is of the normal Internet type
      * then we try to get the FQDN via the backdoor of the IP address.
      * Let's hope it works :)
      */
+
     if (strchr(he->h_name, '.') == NULL
 	&& he->h_addrtype == AF_INET
 	&& he->h_length == 4) {
 	unsigned long a = 0;
 	memcpy((void *)&a, he->h_addr_list[0], 4);
-	if ((he = gethostbyaddr((char *)&a, 4, AF_INET)) == NULL
-	    || strchr(he->h_name, '.') == NULL) {
-#if defined(__sun__) && !defined(__svr4__) && !defined(SVR4)
-	    /* bah.  play it dirty then, sun asks for it. */
-	    if (_res.defdname[0] != '\0') {
-		sprintf(name + strlen(name), "%s%s",
-			(_res.defdname[0] == '.') ? "" : ".",
-			_res.defdname);
+	if ((he = gethostbyaddr((char *)&a, 4, AF_INET)) != NULL
+	    && strchr(he->h_name, '.') != NULL) {
+	    strncpy(name, he->h_name, size);
+	    name[size - 1] = '\0';
+	}
+	else {
+#if !defined(VMS)
+	    FILE *fp = fopen("/etc/resolv.conf", "r");
+	    if (fp) {
+		char *s, buf[256];
+		while (fgets(buf, sizeof buf, fp)) {
+		    if ((s = strtok(buf, " \t\r\n")) != NULL
+			&& !strcmp(s, "domain")
+			&& (s = strtok(NULL, " \t\r\n")) != NULL) {
+			strcat(name, ".");
+			strcat(name, s);
+			break;
+		    }
+		}
+		fclose(fp);
 	    }
 #endif
 	    return;
 	}
     }
-    strncpy(name, he->h_name, size);
+
+    /*
+     * If a "xpilot" host is found compare if it's this one.
+     * and if so, make the local name as "xpilot.*"
+     */
+    if (xpilot_he != NULL) {               /* host xpilot was found */
+	if (strcmp(he->h_name, xpilot_hostname) == 0) {
+	   /*
+	    * Identical official names. Can they be different hosts after this?
+	    * Find out the name which starts with "xpilot" and use it:
+	    */
+	    xpilot_he = gethostbyname(xpilot); /* read again the aliases info */
+	    if (xpilot_he == NULL)       /* shouldn't happen */
+		return;
+
+	    if (strncmp(xpilot, xpilot_he->h_name, xpilot_len) != 0) {
+		/*
+		 * the official hostname doesn't begin "xpilot"
+		 * so we'll find the alias:
+		 */
+		int i;
+		for (i = 0; xpilot_he->h_aliases[i] != NULL; i++) {
+		    alias = xpilot_he->h_aliases[i];
+		    if (!strncmp(xpilot, alias, xpilot_len)) {
+			strcpy(xpilot_hostname, alias);
+			if (!strchr(alias, '.') && (dot = strchr(name, '.'))) {
+			    strcat(xpilot_hostname + strlen(xpilot_hostname), dot);
+			}
+			strncpy(name, xpilot_hostname, size);
+			return;
+		    }
+		}
+	    } else {
+		strncpy(name, xpilot_he->h_name, size);
+		return;
+	    }
+	}
+	/* NOT REATCHED */
+    }
 } /* GetLocalHostName */
 
 
@@ -2390,10 +2126,3 @@ char *inet_ntoa (struct in_addr in)
 	return ascii;
 }
 #endif
-
-#ifdef __cplusplus
-int select(size_t n, fd_set *r, fd_set *w, fd_set *e, const struct timeval *t)
-{
-    return select(n, (int *)r, (int *)w, (int *)e, t);
-}
-#endif
