@@ -1,6 +1,6 @@
-/* $Id: update.c,v 3.37 1994/03/30 17:07:31 bert Exp $
+/* $Id: update.c,v 3.63 1995/01/11 20:00:31 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -21,19 +21,25 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#define SERVER
 #include <stdlib.h>
+#include <stdio.h>
+
+#define SERVER
+#include "version.h"
+#include "config.h"
+#include "const.h"
 #include "global.h"
+#include "proto.h"
 #include "map.h"
 #include "score.h"
-#include "draw.h"
-#include "robot.h"
 #include "bit.h"
 #include "saudio.h"
 
+char update_version[] = VERSION;
+
 #ifndef	lint
 static char sourceid[] =
-    "@(#)$Id: update.c,v 3.37 1994/03/30 17:07:31 bert Exp $";
+    "@(#)$Id: update.c,v 3.63 1995/01/11 20:00:31 bert Exp $";
 #endif
 
 
@@ -55,7 +61,9 @@ static char msg[MSG_LEN];
 static void Transport_to_home(int ind)
 {
     /*
-     * Transport a corpse from the place where it died back to its homebase.
+     * Transport a corpse from the place where it died back to its homebase,
+     * or if in race mode, back to the last passed check point.
+     * 
      * During the first part of the distance we give it a positive constant
      * acceleration G, during the second part we make this a negative one -G.
      * This results in a visually pleasing take off and landing.
@@ -64,8 +72,19 @@ static void Transport_to_home(int ind)
     float		bx, by, dx, dy,	t, m;
     const int		T = RECOVERY_DELAY;
 
-    bx = World.base[pl->home_base].pos.x * BLOCK_SZ + BLOCK_SZ/2;
-    by = World.base[pl->home_base].pos.y * BLOCK_SZ + BLOCK_SZ/2;
+    if (BIT(World.rules->mode, TIMING) && pl->round) {
+	    int check;
+
+	    if (pl->check)
+		    check = pl->check - 1;
+	    else
+		    check = World.NumChecks - 1;
+	    bx = World.check[check].x * BLOCK_SZ + BLOCK_SZ/2;
+	    by = World.check[check].y * BLOCK_SZ + BLOCK_SZ/2;
+    } else {
+	    bx = World.base[pl->home_base].pos.x * BLOCK_SZ + BLOCK_SZ/2;
+	    by = World.base[pl->home_base].pos.y * BLOCK_SZ + BLOCK_SZ/2;
+    }
     dx = WRAP_DX(bx - pl->pos.x);
     dy = WRAP_DY(by - pl->pos.y);
     t = pl->count + 0.5f;
@@ -91,26 +110,54 @@ void Emergency_thrust (int ind, int on)
 	if (pl->emergency_thrust_left <= 0) {
 	    pl->emergency_thrust_left = emergency_thrust_time;
 	    pl->emergency_thrust_max = emergency_thrust_time;
-	    pl->emergency_thrusts--;
+	    pl->item[ITEM_EMERGENCY_THRUST]--;
 	}
 	SET_BIT(pl->used, OBJ_EMERGENCY_THRUST);
 	sound_play_sensors(pl->pos.x, pl->pos.y, EMERGENCY_THRUST_ON_SOUND);
     } else {
 	CLR_BIT(pl->used, OBJ_EMERGENCY_THRUST);
 	if (pl->emergency_thrust_left <= 0) {
-	    /*
-	     * If we have no emergency thrust left we also turn off autopilot
-	     * as that meant we were doing an emergency brake.
-	     */
-	    if (BIT(pl->used, OBJ_AUTOPILOT))
-		Autopilot(ind, 0);
-	    if (pl->emergency_thrusts <= 0)
+	    if (pl->item[ITEM_EMERGENCY_THRUST] <= 0)
 		CLR_BIT(pl->have, OBJ_EMERGENCY_THRUST);
 	}
 	sound_play_sensors(pl->pos.x, pl->pos.y, EMERGENCY_THRUST_OFF_SOUND);
     }
 }
 
+/*
+ * Turn emergency shield on or off.
+ */
+void Emergency_shield (int ind, int on)
+{
+    player	*pl = Players[ind];
+    const int	emergency_shield_time = 4 * FPS;	/* 8 -> 4 */
+
+    if (on) {
+	if (BIT(pl->have, OBJ_EMERGENCY_SHIELD)
+	    && !BIT(pl->used, OBJ_EMERGENCY_SHIELD)) {
+	    if (pl->emergency_shield_left <= 0) {
+		pl->emergency_shield_left = emergency_shield_time;
+		pl->emergency_shield_max = emergency_shield_time;
+		pl->item[ITEM_EMERGENCY_SHIELD]--;
+	    }
+	    SET_BIT(pl->used, OBJ_EMERGENCY_SHIELD);
+	    SET_BIT(pl->have, OBJ_SHIELD);
+	    sound_play_sensors(pl->pos.x, pl->pos.y, EMERGENCY_SHIELD_ON_SOUND);
+	}
+    } else {
+	CLR_BIT(pl->used, OBJ_EMERGENCY_SHIELD);
+	if (pl->emergency_shield_left <= 0) {
+	    if (pl->item[ITEM_EMERGENCY_SHIELD] <= 0)
+		CLR_BIT(pl->have, OBJ_EMERGENCY_SHIELD);
+	}
+	if (!BIT(DEF_HAVE, OBJ_SHIELD)) {
+	    CLR_BIT(pl->have, OBJ_SHIELD);
+	    CLR_BIT(pl->used, OBJ_SHIELD);
+	}
+	sound_play_sensors(pl->pos.x, pl->pos.y, EMERGENCY_SHIELD_OFF_SOUND);
+    }
+}
+
 /*
  * Turn autopilot on or off.  This always clears the thrusting bit.  During
  * automatic pilot mode any changes to the current power, turnacc, turnspeed
@@ -169,19 +216,19 @@ static void do_Autopilot (player *pl)
      * the autopilot code is used by the pause code.
      */
     delta = auto_pilot_settings_delta;
-    if (pl->autopilots)
-	delta *= pl->autopilots;
+    if (pl->item[ITEM_AUTOPILOT])
+	delta *= pl->item[ITEM_AUTOPILOT];
 
     if (BIT(pl->used, OBJ_EMERGENCY_THRUST)) {
 	afterburners = MAX_AFTERBURNER;
 	if (delta < emergency_thrust_settings_delta)
 	    delta = emergency_thrust_settings_delta;
     } else {
-	afterburners = pl->afterburners;
+	afterburners = pl->item[ITEM_AFTERBURNER];
     }
 
-    ix = pl->pos.x/BLOCK_SZ;
-    iy = pl->pos.y/BLOCK_SZ;
+    ix = (int)(pl->pos.x / BLOCK_SZ);
+    iy = (int)(pl->pos.y / BLOCK_SZ);
     gx = World.gravity[ix][iy].x;
     gy = World.gravity[ix][iy].y;
 
@@ -195,7 +242,7 @@ static void do_Autopilot (player *pl)
 	pl->vel.x = pl->vel.y = vel = 0.0;
 	pl->pos = pl->prevpos;
     }
-	
+
     /*
      * Calculate power needed to change instantaneously to stopped.  We
      * must include gravity here for next time round the update loop.
@@ -204,12 +251,15 @@ static void do_Autopilot (player *pl)
     power = acc * pl->mass;
     if (afterburners)
 	power /= AFTER_BURN_POWER_FACTOR(afterburners);
-    
+
     /*
      * Calculate direction change needed to reduce velocity to zero.
      */
     if (vel == 0.0) {
-	vad = findDir(-gx, -gy);
+	if (gx == 0 && gy == 0)
+	    vad = pl->dir;
+	else
+	    vad = findDir(-gx, -gy);
     } else {
 	vad = findDir(-pl->vel.x, -pl->vel.y);
     }
@@ -302,16 +352,18 @@ static void do_Autopilot (player *pl)
  */
 void do_Tractor_beam (player *pl)
 {
-    player	*to;
-    float	maxdist, maxforce, percent;
-    float	xd, yd, force, mass;
-    long	cost;
-    int		theta;
+    player		*victim;
+    float		maxdist, maxforce, percent;
+    float		xd, yd;
+    float		dvx, dvy;
+    float		force, mass;
+    long		cost;
+    int			theta;
 
     maxdist = TRACTOR_MAX_RANGE(pl);
 
-    if (BIT(pl->lock.tagged, (LOCK_PLAYER|LOCK_VISIBLE)) != (LOCK_PLAYER
-							     |LOCK_VISIBLE)
+    if (BIT(pl->lock.tagged, (LOCK_PLAYER|LOCK_VISIBLE))
+		!= (LOCK_PLAYER|LOCK_VISIBLE)
 	|| pl->lock.distance >= maxdist) {
 	pl->tractor = NULL;
 	return;
@@ -329,25 +381,31 @@ void do_Tractor_beam (player *pl)
     }
 
     sound_play_sensors(pl->pos.x, pl->pos.y,
-		       force < 0 ? TRACTOR_BEAM_SOUND : PRESSOR_BEAM_SOUND);
+		       (force < 0) ? TRACTOR_BEAM_SOUND : PRESSOR_BEAM_SOUND);
 
-    to = pl->tractor = Players[GetInd[pl->lock.pl_id]];
+    victim = pl->tractor = Players[GetInd[pl->lock.pl_id]];
 
     Add_fuel(&(pl->fuel), cost);
 
-    xd = WRAP_DX(pl->pos.x - to->pos.x);
-    yd = WRAP_DY(pl->pos.y - to->pos.y);
+    xd = WRAP_DX(pl->pos.x - victim->pos.x);
+    yd = WRAP_DY(pl->pos.y - victim->pos.y);
 
     theta = findDir(xd, yd);
-    mass = pl->mass + to->mass;
+    mass = pl->mass + victim->mass;
 
-    pl->vel.x += tcos(theta) * force / pl->mass;
-    pl->vel.y += tsin(theta) * force / pl->mass;
-    to->vel.x -= tcos(theta) * force / to->mass;
-    to->vel.y -= tsin(theta) * force / to->mass;
+    dvx = tcos(theta) * (force / pl->mass);
+    dvy = tsin(theta) * (force / pl->mass);
+    pl->vel.x += dvx;
+    pl->vel.y += dvy;
+    Record_shove(pl, victim, loops);
+
+    dvx = -(tcos(theta) * (force / victim->mass));
+    dvy = -(tsin(theta) * (force / victim->mass));
+    victim->vel.x += dvx;
+    victim->vel.y += dvy;
+    Record_shove(victim, pl, loops);
 }
 
-
 /********** **********
  * Updating objects and the like.
  */
@@ -360,7 +418,7 @@ void Update_objects(void)
     /*
      * Update robots.
      */
-    Update_robots();
+    Robot_update();
 
     /*
      * Autorepeat fire, must unfortunately be done here, not in
@@ -370,7 +428,7 @@ void Update_objects(void)
     if (fireRepeatRate > 0) {
 	for (i = 0; i < NumPlayers; i++) {
 	    pl = Players[i];
-	    if (BIT(pl->used, OBJ_FIRE)
+	    if (BIT(pl->used, OBJ_SHOT)
 		&& loops - pl->shot_time >= fireRepeatRate) {
 		Fire_normal_shots(i);
 	    }
@@ -385,7 +443,7 @@ void Update_objects(void)
 	    && World.items[i].chance > 0
 	    && rand()%World.items[i].chance == 0) {
 
-	    Place_item(i, 0);
+	    Place_item(i, (player *)NULL);
 	}
 
     /*
@@ -422,7 +480,7 @@ void Update_objects(void)
 	update_object_speed(obj);
 	Move_object(i);
 
-	if (BIT(Obj[i]->type, OBJ_MINE))
+	if (BIT(obj->type, OBJ_MINE))
 	    Move_mine(i);
 
 	else if (BIT(obj->type, OBJ_SMART_SHOT|OBJ_HEAT_SHOT|OBJ_TORPEDO))
@@ -453,7 +511,7 @@ void Update_objects(void)
 	}
 	World.cannon[i].active = false;
     }
-    
+
     /*
      * Update targets
      */
@@ -463,7 +521,24 @@ void Update_objects(void)
 		World.block[World.targets[i].pos.x][World.targets[i].pos.y]
 		    = TARGET;
 		World.targets[i].conn_mask = 0;
+		World.targets[i].update_mask = (unsigned)-1;
 		World.targets[i].last_change = loops;
+
+		if (targetSync) {
+		    u_short team = World.targets[i].team;
+
+		    for (j = 0; j < World.NumTargets; j++) {
+			if (World.targets[j].team == team) {
+			    World.block[World.targets[j].pos.x]
+				       [World.targets[j].pos.y] = TARGET;
+			    World.targets[j].conn_mask = 0;
+			    World.targets[j].update_mask = (unsigned)-1;
+			    World.targets[j].last_change = loops;
+			    World.targets[j].dead_time = 0;
+			    World.targets[j].damage = TARGET_DAMAGE;
+			}
+		    }
+		}
 	    }
 	    continue;
 	}
@@ -492,7 +567,7 @@ void Update_objects(void)
      */
     for (i=0; i<NumPlayers; i++) {
 #ifdef TURN_FUEL
-        long tf = 0;
+	long tf = 0;
 #endif
 
 	pl = Players[i];
@@ -525,9 +600,6 @@ void Update_objects(void)
 		Throw_items(pl);
 		Kill_player(i);
 		updateScores = true;
-		pl->check = 0;
-		pl->round = 0;
-		pl->time  = 0;
 	    }
 	}
 
@@ -536,10 +608,14 @@ void Update_objects(void)
 
 	if (pl->shield_time > 0) {
 	    if (--pl->shield_time == 0) {
-		CLR_BIT(pl->used, OBJ_SHIELD);
+		if (!BIT(pl->used, OBJ_EMERGENCY_SHIELD)) {
+		    CLR_BIT(pl->used, OBJ_SHIELD);
+		}
 	    }
 	    if (BIT(pl->used, OBJ_SHIELD) == 0) {
-		CLR_BIT(pl->have, OBJ_SHIELD);
+		if (!BIT(pl->have, OBJ_EMERGENCY_SHIELD)) {
+		    CLR_BIT(pl->have, OBJ_SHIELD);
+		}
 		pl->shield_time = 0;
 	    }
 	}
@@ -551,6 +627,13 @@ void Update_objects(void)
 		Emergency_thrust(i, 0);
 	}
 
+	if (BIT(pl->used, OBJ_EMERGENCY_SHIELD)) {
+	    if (pl->fuel.sum > 0
+		&& BIT(pl->used, OBJ_SHIELD)
+		&& --pl->emergency_shield_left <= 0)
+		Emergency_shield(i, 0);
+	}
+
 	/*
 	 * Only do autopilot code if switched on and player is not
 	 * damaged (ie. can see).
@@ -565,23 +648,23 @@ void Update_objects(void)
 	 */
 	pl->turnvel	+= pl->turnacc;
 	pl->turnvel	*= pl->turnresistance;
-        
+
 #ifdef TURN_FUEL
-        tf = pl->oldturnvel-pl->turnvel;
-        tf = TURN_FUEL(tf);
-        if (pl->fuel.sum <= tf) {
-            tf = 0;
-            pl->turnacc = 0.0;
-            pl->turnvel = pl->oldturnvel;
-        } else {
-            Add_fuel(&(pl->fuel),-tf);
-            pl->oldturnvel = pl->turnvel;
-        }
+	tf = pl->oldturnvel - pl->turnvel;
+	tf = TURN_FUEL(tf);
+	if (pl->fuel.sum <= tf) {
+	    tf = 0;
+	    pl->turnacc = 0.0;
+	    pl->turnvel = pl->oldturnvel;
+	} else {
+	    Add_fuel(&(pl->fuel),-tf);
+	    pl->oldturnvel = pl->turnvel;
+	}
 #endif
 
 
 	pl->float_dir	+= pl->turnvel;
-        
+
 	if (pl->float_dir < 0)
 	    pl->float_dir += RES;
 	if (pl->float_dir >= RES)
@@ -594,10 +677,10 @@ void Update_objects(void)
 	 * Compute energy drainage
 	 */
 	if (BIT(pl->used, OBJ_SHIELD))
-            Add_fuel(&(pl->fuel), ED_SHIELD);
+	    Add_fuel(&(pl->fuel), ED_SHIELD);
 
 	if (BIT(pl->used, OBJ_CLOAKING_DEVICE))
-            Add_fuel(&(pl->fuel), ED_CLOAKING_DEVICE);
+	    Add_fuel(&(pl->fuel), ED_CLOAKING_DEVICE);
 
 #define UPDATE_RATE 100
 
@@ -614,41 +697,43 @@ void Update_objects(void)
 
 		pl->visibility[j].lastChange = loops;
 		pl->visibility[j].canSee
-		    = rand() % (pl->sensors + 1)
-			> (rand() % (Players[j]->cloaks + 1));
+		    = rand() % (pl->item[ITEM_SENSOR] + 1)
+			> (rand() % (Players[j]->item[ITEM_CLOAK] + 1));
 	    }
 	}
 
 	if (BIT(pl->used, OBJ_REFUEL)) {
-	    if ((Wrap_length((pl->pos.x-World.fuel[pl->fs].pos.x),
-		             (pl->pos.y-World.fuel[pl->fs].pos.y)) > 90.0)
-		|| (pl->fuel.sum >= pl->fuel.max)) {
+	    if ((Wrap_length((pl->pos.x-World.fuel[pl->fs].pix_pos.x),
+			     (pl->pos.y-World.fuel[pl->fs].pix_pos.y)) > 90.0)
+		|| (pl->fuel.sum >= pl->fuel.max)
+		|| (World.block[World.fuel[i].blk_pos.x]
+			       [World.fuel[i].blk_pos.y] != FUEL)) {
 		CLR_BIT(pl->used, OBJ_REFUEL);
 	    } else {
-                int i = pl->fuel.num_tanks;
-                int ct = pl->fuel.current;
-                
-                do {
+		int i = pl->fuel.num_tanks;
+		int ct = pl->fuel.current;
+
+		do {
 		    if (World.fuel[pl->fs].fuel > REFUEL_RATE) {
-		        World.fuel[pl->fs].fuel -= REFUEL_RATE;
+			World.fuel[pl->fs].fuel -= REFUEL_RATE;
 			World.fuel[pl->fs].conn_mask = 0;
 			World.fuel[pl->fs].last_change = loops;
-		        Add_fuel(&(pl->fuel), REFUEL_RATE);
+			Add_fuel(&(pl->fuel), REFUEL_RATE);
 		    } else {
-		        Add_fuel(&(pl->fuel), World.fuel[pl->fs].fuel);
-		        World.fuel[pl->fs].fuel = 0;
-		        World.fuel[pl->fs].conn_mask = 0;
-		        World.fuel[pl->fs].last_change = loops;
-		        CLR_BIT(pl->used, OBJ_REFUEL);
-                        break;
+			Add_fuel(&(pl->fuel), World.fuel[pl->fs].fuel);
+			World.fuel[pl->fs].fuel = 0;
+			World.fuel[pl->fs].conn_mask = 0;
+			World.fuel[pl->fs].last_change = loops;
+			CLR_BIT(pl->used, OBJ_REFUEL);
+			break;
 		    }
-                    if (pl->fuel.current == pl->fuel.num_tanks)
-                        pl->fuel.current = 0;
-                    else
-                        pl->fuel.current += 1;
-                } while (i--);
-                pl->fuel.current = ct;
-            }
+		    if (pl->fuel.current == pl->fuel.num_tanks)
+			pl->fuel.current = 0;
+		    else
+			pl->fuel.current += 1;
+		} while (i--);
+		pl->fuel.current = ct;
+	    }
 	}
 
 	/* target repair */
@@ -697,18 +782,20 @@ void Update_objects(void)
 	 * Update acceleration vector etc.
 	 */
 	if (BIT(pl->status, THRUSTING)) {
-            float power = pl->power;
-            float f = pl->power * 0.0008;	/* 1/(FUEL_SCALE*MIN_POWER) */
-            int a = (BIT(pl->used, OBJ_EMERGENCY_THRUST) ? MAX_AFTERBURNER
-		     : pl->afterburners);
+	    float power = pl->power;
+	    float f = pl->power * 0.0008;	/* 1/(FUEL_SCALE*MIN_POWER) */
+	    int a = (BIT(pl->used, OBJ_EMERGENCY_THRUST)
+		     ? MAX_AFTERBURNER
+		     : pl->item[ITEM_AFTERBURNER]);
+	    float inert = pl->mass;
 
-            if (a) {
-                power = AFTER_BURN_POWER(power, a);
-                f = AFTER_BURN_FUEL(f, a);
-            }
-	    pl->acc.x = power * tcos(pl->dir) / pl->mass;
-	    pl->acc.y = power * tsin(pl->dir) / pl->mass;
-            Add_fuel(&(pl->fuel), -f * FUEL_SCALE_FACT); /* Decrement fuel */
+	    if (a) {
+		power = AFTER_BURN_POWER(power, a);
+		f = AFTER_BURN_FUEL(f, a);
+	    }
+	    pl->acc.x = power * tcos(pl->dir) / inert;
+	    pl->acc.y = power * tsin(pl->dir) / inert;
+	    Add_fuel(&(pl->fuel), -f * FUEL_SCALE_FACT); /* Decrement fuel */
 	} else {
 	    pl->acc.x = pl->acc.y = 0.0;
 	}
@@ -718,8 +805,8 @@ void Update_objects(void)
 	if (BIT(pl->status, WARPING)) {
 	    position w;
 	    int wx, wy, proximity,
-	    	nearestFront, nearestRear,
-	    	proxFront, proxRear;
+		nearestFront, nearestRear,
+		proxFront, proxRear;
 
 	    if (World.wormHoles[pl->wormHoleHit].countdown
 		&& World.wormHoles[pl->wormHoleHit].lastplayer != i)
@@ -732,22 +819,22 @@ void Update_objects(void)
 	    else {
 		nearestFront = nearestRear = -1;
 		proxFront = proxRear = 10000000;
-		
+
 		for (j = 0; j < World.NumWormholes; j++) {
 		    if (j == pl->wormHoleHit
 			|| World.wormHoles[j].type == WORM_IN)
 			continue;
-		    
+
 		    wx = (World.wormHoles[j].pos.x -
 			  World.wormHoles[pl->wormHoleHit].pos.x) * BLOCK_SZ;
-		    wy = (World.wormHoles[j].pos.y - 
+		    wy = (World.wormHoles[j].pos.y -
 			  World.wormHoles[pl->wormHoleHit].pos.y) * BLOCK_SZ;
 		    wx = WRAP_DX(wx);
 		    wy = WRAP_DX(wy);
-		    
+
 		    proximity = pl->vel.y * wx + pl->vel.x * wy;
 		    proximity = ABS(proximity);
-		    
+
 		    if (pl->vel.x * wx + pl->vel.y * wy < 0) {
 			if (proximity < proxRear) {
 			    nearestRear = j;
@@ -758,7 +845,7 @@ void Update_objects(void)
 			proxFront = proximity;
 		    }
 		}
-		
+
 #define RANDOM_REAR_WORM
 #ifndef RANDOM_REAR_WORM
 		j = nearestFront < 0 ? nearestRear : nearestFront;
@@ -773,7 +860,7 @@ void Update_objects(void)
 #endif /* RANDOM_REAR_WORM */
 	    }
 
-            sound_play_sensors(pl->pos.x, pl->pos.y, WORM_HOLE_SOUND);
+	    sound_play_sensors(pl->pos.x, pl->pos.y, WORM_HOLE_SOUND);
 
 	    w.x = World.wormHoles[j].pos.x * BLOCK_SZ + BLOCK_SZ / 2;
 	    w.y = World.wormHoles[j].pos.y * BLOCK_SZ + BLOCK_SZ / 2;
@@ -822,7 +909,7 @@ void Update_objects(void)
 	    CLR_BIT(pl->status, WARPING);
 	    SET_BIT(pl->status, WARPED);
 
-            sound_play_sensors(pl->pos.x, pl->pos.y, WORM_HOLE_SOUND);
+	    sound_play_sensors(pl->pos.x, pl->pos.y, WORM_HOLE_SOUND);
 	}
 
 	/*
@@ -854,38 +941,20 @@ void Update_objects(void)
 	    Move_player(i);
 	}
 
-	if (BIT(pl->status, THRUSTING))
+	if (BIT(pl->status, THRUSTING)) {
 	    Thrust(i);
+	}
 #ifdef TURN_FUEL
 	if (tf)
-            Turn_thrust(i, TURN_SPARKS(tf));
+	    Turn_thrust(i, TURN_SPARKS(tf));
 #endif
 
-	/*
-	 * Updating time player has used. Only used when timing
-	 */
-	if (pl->round != 0)
-	    pl->time++;
-
-	if (pl->round == 4) {
-	    if ((pl->time < pl->best_run) || (pl->best_run == 0)) 
-		pl->best_run = pl->time;
-	    Players[i]->last_lap_time = Players[i]->time 
-		- Players[i]->last_lap;
-	    Kill_player(i);
-	    pl->round = 0;
-	    pl->check = 0;
-	    pl->time = 0;
-	}
-
 	if (BIT(pl->lock.tagged, LOCK_PLAYER)) {
-	    pl->lock.distance = Wrap_length(pl->pos.x -
-				     Players[GetInd[pl->lock.pl_id]]->pos.x,
-				     pl->pos.y -
-				     Players[GetInd[pl->lock.pl_id]]->pos.y);
-	    pl->sensor_range = MAX(pl->fuel.sum * ENERGY_RANGE_FACTOR,
-				   VISIBILITY_DISTANCE);
+	    pl->lock.distance =
+		Wrap_length(pl->pos.x - Players[GetInd[pl->lock.pl_id]]->pos.x,
+			    pl->pos.y - Players[GetInd[pl->lock.pl_id]]->pos.y);
 	}
+	Compute_sensor_range(pl);
 
 	pl->used &= pl->have;
     }
@@ -893,7 +962,7 @@ void Update_objects(void)
     for (i=0; i<World.NumWormholes; i++)
        if (World.wormHoles[i].countdown)
 	   --World.wormHoles[i].countdown;
-	    
+
     for (i = 0; i < NumPlayers; i++) {
 	player *pl = Players[i];
 
@@ -922,21 +991,17 @@ void Update_objects(void)
      * Update tanks, Kill players that ought to be killed.
      */
     for (i=NumPlayers-1; i>=0; i--) {
-        player *pl = Players[i];
-        
+	player *pl = Players[i];
+
 	if (BIT(pl->status, PLAYING|PAUSE|GAME_OVER|KILLED) == PLAYING)
 	    Update_tanks(&(pl->fuel));
 	if (BIT(pl->status, KILLED)) {
 	    Throw_items(pl);
-            if (pl->robot_mode != RM_OBJECT) {
-	        Kill_player(i);
-            } else {
-                NumPseudoPlayers--;
-                Explode(i);
-                Delete_player(i);
-		updateScores = true;
-            }
-        }
+
+	    Detonate_items(i);
+
+	    Kill_player(i);
+	}
     }
 
     /*
@@ -960,3 +1025,4 @@ void Update_objects(void)
     if (updateScores && loops % UPDATE_SCORE_DELAY == 0)
 	Update_score_table();
 }
+
