@@ -1,6 +1,6 @@
-/* $Id: widget.c,v 3.15 1994/02/13 15:49:19 bert Exp $
+/* $Id: widget.c,v 3.20 1995/01/24 17:27:40 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -21,7 +21,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <X11/Xproto.h>
 #include <X11/Xlib.h>
 #include <X11/Xos.h>
 #include <X11/Xutil.h>
@@ -34,14 +33,16 @@
 #include <limits.h>
 
 #include "version.h"
-#include "client.h"
+#include "config.h"
+#include "const.h"
 #include "paint.h"
-#include "draw.h"
 #include "xinit.h"
 #include "bit.h"
 #include "error.h"
 #include "widget.h"
 
+char widget_version[] = VERSION;
+
 static void Widget_resize_viewer(XEvent *event, int ind);
 
 static widget_t		*widgets;
@@ -307,7 +308,7 @@ int Widget_resize(int widget_desc, int width, int height)
 static void Widget_draw_button(widget_t *widget, bool inverse, char *label)
 {
     int			x, y;
-    Pixel		fg, bg;
+    unsigned long	fg, bg;
 
     if (mono) {
 	if (inverse) {
@@ -508,16 +509,16 @@ static void Widget_viewer_draw_lines(widget_t *widget, int start, int end)
 	    XSetForeground(dpy, motdGC, colors[BLACK].pixel);
 	    XDrawString(dpy, widget->window, motdGC,
 			x+1, y+1,
-			viewerw->line[i].txt, viewerw->line[i].len);
+			(char *) viewerw->line[i].txt, viewerw->line[i].len);
 	    XSetForeground(dpy, motdGC, colors[WHITE].pixel);
 	    XDrawString(dpy, widget->window, motdGC,
 			x-1, y-1,
-			viewerw->line[i].txt, viewerw->line[i].len);
+			(char *) viewerw->line[i].txt, viewerw->line[i].len);
 	}
 	else {
 	    XDrawString(dpy, widget->window, motdGC,
 			x, y,
-			viewerw->line[i].txt, viewerw->line[i].len);
+			(char *) viewerw->line[i].txt, viewerw->line[i].len);
 	}
 	y += text_height;
     }
@@ -595,7 +596,8 @@ static void Widget_draw_expose(int widget_desc, XExposeEvent *expose)
 	if (expose && expose->count > 0)
 	    break;
 	boolw = (widget_bool_t *) widget->sub;
-	Widget_draw_button(widget, boolw->pressed && boolw->inside,
+	Widget_draw_button(widget,
+			   (boolw->pressed && boolw->inside) ? true : false,
 			   (boolw->state == true) ? "Yes" : "No");
 	break;
 
@@ -610,7 +612,8 @@ static void Widget_draw_expose(int widget_desc, XExposeEvent *expose)
 	if (expose && expose->count > 0)
 	    break;
 	activw = (widget_activate_t *) widget->sub;
-	Widget_draw_button(widget, activw->pressed && activw->inside,
+	Widget_draw_button(widget,
+			   (activw->pressed && activw->inside) ? true : false,
 			   activw->str);
 	break;
 
@@ -1038,6 +1041,9 @@ static void Widget_button(XEvent *event, int widget_desc, bool pressed)
 		    }
 		}
 		break;
+	    default:
+		/*NOTREACHED*/
+		break;
 	    }
 	}
 	break;
@@ -1161,6 +1167,9 @@ int Widget_event(XEvent *event)
 			Widget_button(event, i, false);
 		    }
 		    break;
+		default:
+		    /*NOTREACHED*/
+		    break;
 		}
 	    }
 	    if (count > 0) {
@@ -1646,7 +1655,8 @@ int Widget_create_label(int parent_desc,
     return widget_desc;
 }
 
-static int Widget_create_arrow(int type, int parent_desc, int x, int y,
+static int Widget_create_arrow(widget_type_t type, int parent_desc,
+			       int x, int y,
 			       int width, int height,
 			       int border, int related_desc)
 {
@@ -1776,7 +1786,7 @@ int Widget_create_confirm(char *confirm_str,
 			button_desc;
 
     popup_desc = Widget_create_popup(popup_width, popup_height, 1,
-    				"Confirm", "Confirm");
+				"Confirm", "Confirm");
     if (popup_desc == NO_WIDGET) {
 	return NO_WIDGET;
     }
@@ -1792,7 +1802,7 @@ int Widget_create_confirm(char *confirm_str,
     button_desc = Widget_create_activate(popup_desc,
 					 (popup_width - button_width) / 2,
 					 popup_height - button_height
-					 	- button_space,
+						- button_space,
 					 button_width, button_height,
 					 0, button_str,
 					 callback,
@@ -2009,7 +2019,7 @@ static int Widget_viewer_calculate_text(int viewer_desc)
 	}
 	line[count].len = &viewerw->buf[i] - line[count].txt;
 	line[count].txt_width = XTextWidth(viewerw->font,
-					   line[count].txt,
+					   (char *) line[count].txt,
 					   line[count].len);
 	if (line[count].txt_width > viewerw->max_width) {
 	    viewerw->max_width = line[count].txt_width;
@@ -2031,7 +2041,7 @@ static void Widget_resize_viewer(XEvent *event, int ind)
     widget_t		*viewer_widget = Widget_pointer(viewer_desc);
     widget_viewer_t	*viewer_sub = (widget_viewer_t *)viewer_widget->sub;
     XFontStruct		*font = viewer_sub->font;
-    const int		
+    const int
 			close_width = 2*8 + XTextWidth(font, "CLOSE", 5),
 			close_height = 6 + font->ascent + font->descent,
 			close_x_offset = (width - close_width) / 2,
@@ -2062,7 +2072,7 @@ static void Widget_resize_viewer(XEvent *event, int ind)
 		vert_slider_x, vert_slider_y);
     Widget_resize(formw->children[3], close_width, close_height);
     XMoveWindow(dpy, Widget_pointer(formw->children[3])->window,
-	        close_x_offset, viewer_height + close_y_offset);
+		close_x_offset, viewer_height + close_y_offset);
     popup->width = width;
     popup->height = height;
     viewer_widget->width = viewer_width;
@@ -2077,7 +2087,7 @@ int Widget_create_viewer(const char *buf, int len,
 			 char *window_name, char *icon_name,
 			 XFontStruct *font)
 {
-    const int		
+    const int
 			close_width = 2*8 + XTextWidth(font, "CLOSE", 5),
 			close_height = 6 + font->ascent + font->descent,
 			close_x_offset = (width - close_width) / 2,
@@ -2151,8 +2161,8 @@ int Widget_create_viewer(const char *buf, int len,
     Widget_window_gravity(window, NorthWestGravity);
     XMapWindow(dpy, window);
     viewer_desc = Widget_create(WIDGET_VIEWER, "viewer", window,
-			        viewer_width, viewer_height,
-			        viewerw);
+				viewer_width, viewer_height,
+				viewerw);
     if (viewer_desc == NO_WIDGET) {
 	Widget_destroy(popup_desc);
 	return NO_WIDGET;
@@ -2227,7 +2237,7 @@ int Widget_update_viewer(int popup_desc, const char *buf, int len)
     widget_t		*viewer_widget;
     widget_viewer_t	*viewer_sub;
     int			text_height,
-			first_visible, 
+			first_visible,
 			last_visible,
 			start,
 			end;
