@@ -1,6 +1,6 @@
-/* $Id: xinit.c,v 3.61 1994/02/17 11:02:17 bert Exp $
+/* $Id: xinit.c,v 3.86 1995/02/04 15:03:46 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -21,7 +21,6 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <X11/Xproto.h>
 #include <X11/Xlib.h>
 #include <X11/Xos.h>
 #include <X11/Xutil.h>
@@ -36,72 +35,26 @@
 #include <errno.h>
 
 #include "version.h"
+#include "config.h"
 #include "const.h"
 #include "rules.h"
-#include "draw.h"
 #include "icon.h"
-#include "client.h"
 #include "paint.h"
 #include "xinit.h"
 #include "bit.h"
+#include "keys.h"
 #include "setup.h"
 #include "widget.h"
 #include "configure.h"
 #include "error.h"
 #include "netclient.h"
+#include "dbuff.h"
 
-#ifndef	lint
-static char sourceid[] =
-    "@(#)$Id: xinit.c,v 3.61 1994/02/17 11:02:17 bert Exp $";
-#endif
-
-#if defined(__cplusplus)
-#define class c_class
-#endif
-
-#define MAX_VISUAL_CLASS	6
-
-extern message_t	*Msg[];
-extern int		RadarHeight;
-
-/*
- * Globals.
- */
-int	ButtonHeight;
-Atom	ProtocolAtom, KillAtom;
-bool	talk_mapped;
-int	buttonColor, windowColor, borderColor, hudColor;
-int	quitting = false;
-char	visualName[MAX_VISUAL_NAME];
-Visual	*visual;
-int	dispDepth;
-bool	mono;
-bool	colorSwitch;
-int	top_width, top_height, top_x, top_y, top_posmask;
-int	draw_width, draw_height;
-char	*geometry;
-bool	autoServerMotdPopup;
-
-
-static message_t	*MsgBlock;
-static char		*keyHelpList = NULL, *keyHelpDesc = NULL;
-static int		KeyDescOffset;
-static bool		about_created;
-static bool		keys_created;
-static bool		talk_created;
-static char		talk_str[MAX_CHARS];
-static struct {
-    bool		visible;
-    short		offset;
-    short		point;
-} talk_cursor;
-
+char xinit_version[] = VERSION;
 
 /*
  * Item structures
  */
-#include "item.h"
-
 #include "items/itemRocketPack.xbm"
 #include "items/itemCloakingDevice.xbm"
 #include "items/itemEnergyPack.xbm"
@@ -117,71 +70,21 @@ static struct {
 #include "items/itemEmergencyThrust.xbm"
 #include "items/itemTractorBeam.xbm"
 #include "items/itemAutopilot.xbm"
+#include "items/itemEmergencyShield.xbm"
 
-/* NB!  Is dependent on the order of the items in item.h */
-static struct {
-    char*	data;
-    char*	keysText;
-} itemBitmapData[NUM_ITEMS] = {
-    {	itemEnergyPack_bits,
-	    "Extra energy/fuel"					},
-    {	itemWideangleShot_bits,
-	    "Extra front cannons"				},
-    {	itemRearShot_bits,
-	    "Extra rear cannon"					},
-    {	itemAfterburner_bits,
-	    "Afterburner; makes your engines more powerful"	},
-    {	itemCloakingDevice_bits,
-	    "Cloaking device; "
-	    "makes you almost invisible, both on radar and on screen"	},
-    {	itemSensorPack_bits,
-	    "Sensor; "
-	    "enables you to see cloaked opponents more easily"	},
-    {	itemTransporter_bits,
-	    "Transporter; enables you to steal equipment from "
-	    "other players"					},
-    {	itemTank_bits,
-	    "Tank; "
-	    "makes refueling quicker, increases maximum fuel "
-	    "capacity and can be jettisoned to confuse enemies"	},
-    {	itemMinePack_bits,
-	    "Mine; "
-	    "can be dropped as a bomb or as a stationary mine"	},
-    {	itemRocketPack_bits,
-	    "Rocket; can be utilized as smart missile, "
-	    "heat seaking missile, nuclear missile or just a "
-	    "plain unguided missile (torpedo)"			},
-    {	itemEcm_bits,
-	    "ECM (Electronic Counter Measures); "
-	    "can be used to disturb electronic equipment, for instance "
-	    "can it be used to confuse smart missiles and reprogram "
-	    "robots to seak certain players"			},
-    {	itemLaser_bits,
-	    "Laser; "
-	    "limited range laser beam, costs a lot of fuel, "
-	    "having more laser items increases the range of the laser, "
-	    "they can be irrepairably damaged by ECMs" },
-    {	itemEmergencyThrust_bits,
-	    "Emergency Thrust; "
-	    "gives emergency thrust capabilities for a limited period" },
-    {   itemTractorBeam_bits,
-	    "Tractor Beam; "
-	    "gives mutual attractive force to currently locked on ship, "
-	    "this means the heavier your ship, the less likely you will move "
-	    "when being tractored or using a tractor" },
-    {	itemAutopilot_bits,
-	    "Autopilot; "
-	    "when on, the ship will turn and thrust against the "
-            "direction of travel" },
-};
-Pixmap	itemBitmaps[NUM_ITEMS];		/* Bitmaps for the items */
-    
-char dashes[NUM_DASHES] = { 8, 4 };
-char cdashes[NUM_CDASHES] = { 3, 9 };
+#ifndef	lint
+static char sourceid[] =
+    "@(#)$Id: xinit.c,v 3.86 1995/02/04 15:03:46 bert Exp $";
+#endif
+
+/* Kludge for visuals under C++ */
+#if defined(__cplusplus)
+#define class c_class
+#endif
 
 /* How far away objects should be placed from each other etc... */
-#define BORDER	10
-#define BTN_BORDER 4
+#define BORDER			10
+#define BTN_BORDER		4
 
 /* Information window dimensions */
 #define ABOUT_WINDOW_WIDTH	600
@@ -197,8 +100,142 @@ char cdashes[NUM_CDASHES] = { 3, 9 };
 
 #define CTRL(c)			((c) & 0x1F)
 
+#define MAX_VISUAL_CLASS	6
+
+extern message_t	*TalkMsg[], *GameMsg[];
+extern int		RadarHeight;
+
+/*
+ * Globals.
+ */
+int			ButtonHeight;
+Atom			ProtocolAtom, KillAtom;
+bool			talk_mapped;
+int			buttonColor, windowColor, borderColor;
+int			quitting = false;
+char			visualName[MAX_VISUAL_NAME];
+Visual			*visual;
+int			dispDepth;
+bool			mono;
+bool			colorSwitch;
+int			top_width, top_height, top_x, top_y, top_posmask;
+int			draw_width, draw_height;
+char			*geometry;
+bool			autoServerMotdPopup;
+Cursor			pointerControlCursor;
+char			sparkColors[MSG_LEN];
+int			spark_color[MAX_COLORS];
+int			num_spark_colors;
+
+static message_t	*MsgBlock;
+static bool		about_created;
+static bool		talk_created;
+static char		talk_str[MAX_CHARS];
+static struct {
+    bool		visible;
+    short		offset;
+    short		point;
+} talk_cursor;
+
+
+/*
+ * NB!  Is dependent on the order of the items in item.h!
+ */
+static struct {
+    char*	data;
+    char*	keysText;
+} itemBitmapData[NUM_ITEMS] = {
+    {
+	itemEnergyPack_bits,
+	"Extra energy/fuel"
+    },
+    {
+	itemWideangleShot_bits,
+	"Extra front cannons"
+    },
+    {
+	itemRearShot_bits,
+	"Extra rear cannon"
+    },
+    {
+	itemAfterburner_bits,
+	"Afterburner; makes your engines more powerful"
+    },
+    {
+	itemCloakingDevice_bits,
+	"Cloaking device; "
+	"makes you almost invisible, both on radar and on screen"
+    },
+    {
+	itemSensorPack_bits,
+	"Sensor; "
+	"enables you to see cloaked opponents more easily"
+    },
+    {
+	itemTransporter_bits,
+	"Transporter; enables you to steal equipment from "
+	"other players"
+    },
+    {
+	itemTank_bits,
+	"Tank; "
+	"makes refueling quicker, increases maximum fuel "
+	"capacity and can be jettisoned to confuse enemies"
+    },
+    {
+	itemMinePack_bits,
+	"Mine; "
+	"can be dropped as a bomb or as a stationary mine"
+    },
+    {
+	itemRocketPack_bits,
+	"Rocket; can be utilized as smart missile, "
+	"heat seaking missile, nuclear missile or just a "
+	"plain unguided missile (torpedo)"
+    },
+    {
+	itemEcm_bits,
+	"ECM (Electronic Counter Measures); "
+	"can be used to disturb electronic equipment, for instance "
+	"can it be used to confuse smart missiles and reprogram "
+	"robots to seak certain players"
+    },
+    {
+	itemLaser_bits,
+	"Laser; "
+	"limited range laser beam, costs a lot of fuel, "
+	"having more laser items increases the range of the laser, "
+	"they can be irrepairably damaged by ECMs"
+    },
+    {
+	itemEmergencyThrust_bits,
+	"Emergency Thrust; "
+	"gives emergency thrust capabilities for a limited period"
+    },
+    {   itemTractorBeam_bits,
+	"Tractor Beam; "
+	"gives mutual attractive force to currently locked on ship, "
+	"this means the heavier your ship, the less likely you will move "
+	"when being tractored or using a tractor"
+    },
+    {
+	itemAutopilot_bits,
+	"Autopilot; "
+	"when on, the ship will turn and thrust against the "
+	"direction of travel"
+    },
+    {
+	itemEmergencyShield_bits,
+	"EmergencyShield; "
+	"gives emergency shield capabilities for a limited period"
+    },
+};
+Pixmap	itemBitmaps[NUM_ITEMS];		/* Bitmaps for the items */
+
+char dashes[NUM_DASHES] = { 8, 4 };
+char cdashes[NUM_CDASHES] = { 3, 9 };
+
 static void createAboutWindow(void);
-static void createKeysWindow(void);
 static void createTalkWindow(void);
 static int Quit_callback(int, void *, char **);
 static int About_callback(int, void *, char **);
@@ -229,15 +266,15 @@ static struct visual_class_name {
 char		color_names[MAX_COLORS][MAX_COLOR_LEN];
 static char	*color_defaults[MAX_COLORS] = {
     "#000000", "#FFFFFF", "#4E7CFF", "#FF3A27",
-    "#440000", "#992200", "#BB7700", "#EE9900",
+    "#33BB44", "#992200", "#BB7700", "#EE9900",
     "#770000", "#CC4400", "#DD8800", "#FFBB11",
-    "", "", "", ""
+    "#9f9f9f", "#5f5f5f", "#dfdfdf", "#202020"
 };
 static char	*gray_defaults[MAX_COLORS] = {
     "#000000", "#FFFFFF", "#AAAAAA", "#CCCCCC",
-    "#666666", "#888888", "#AAAAAA", "#CCCCCC",
+    "#BBBBBB", "#888888", "#AAAAAA", "#CCCCCC",
     "#777777", "#999999", "#BBBBBB", "#DDDDDD",
-    "", "", "", ""
+    "#9f9f9f", "#5f5f5f", "#dfdfdf", "#202020"
 };
 
 
@@ -319,7 +356,7 @@ static void Fill_colormap(void)
 			cells_needed,
 			max_fill;
     unsigned long	pixels[256];
-    XColor		acolor[256];
+    XColor		mycolors[256];
 
     if (colormap == 0 || colorSwitch != true) {
 	return;
@@ -327,7 +364,7 @@ static void Fill_colormap(void)
     cells_needed = (maxColors == 16) ? 256
 	: (maxColors == 8) ? 64
 	: 16;
-    max_fill = MAX(256, visual->map_entries) - cells_needed - 2;
+    max_fill = MAX(256, visual->map_entries) - cells_needed;
     if (max_fill <= 0) {
 	return;
     }
@@ -337,12 +374,22 @@ static void Fill_colormap(void)
 	error("Can't pre-alloc color cells");
 	return;
     }
+    /* Check for misunderstanding of X colormap stuff. */
     for (i = 0; i < max_fill; i++) {
-	acolor[i].pixel = pixels[i];
+	if (pixels[i] != i) {
+	    errno = 0;
+	    error("Can't pre-fill color map, got %d'th pixel %lu",
+		  i, pixels[i]);
+	    XFreeColors(dpy, colormap, pixels, max_fill, 0);
+	    return;
+	}
+    }
+    for (i = 0; i < max_fill; i++) {
+	mycolors[i].pixel = pixels[i];
     }
     XQueryColors(dpy, DefaultColormap(dpy, DefaultScreen(dpy)),
-		 acolor, max_fill);
-    XStoreColors(dpy, colormap, acolor, max_fill);
+		 mycolors, max_fill);
+    XStoreColors(dpy, colormap, mycolors, max_fill);
 }
 
 
@@ -391,16 +438,16 @@ void List_visuals(void)
     vinfo_ptr = XGetVisualInfo(dpy, mask, &my_vinfo, &num);
     printf("Listing all visuals:\n");
     for (i = 0; i < num; i++) {
-	printf("Visual class       %12s\n",
+	printf("Visual class    %12s\n",
 	       Visual_class_name(vinfo_ptr[i].class));
-	printf("    id                     0x%02x\n", vinfo_ptr[i].visualid);
-	printf("    screen             %8d\n", vinfo_ptr[i].screen);
-	printf("    depth              %8d\n", vinfo_ptr[i].depth);
-	printf("    red_mask           0x%06x\n", vinfo_ptr[i].red_mask);
-	printf("    green_mask         0x%06x\n", vinfo_ptr[i].green_mask);
-	printf("    blue_mask          0x%06x\n", vinfo_ptr[i].blue_mask);
-	printf("    colormap_size      %8d\n", vinfo_ptr[i].colormap_size);
-	printf("    bits_per_rgb       %8d\n", vinfo_ptr[i].bits_per_rgb);
+	printf("    id              0x%02x\n", (unsigned)vinfo_ptr[i].visualid);
+	printf("    screen          %8d\n", vinfo_ptr[i].screen);
+	printf("    depth           %8d\n", vinfo_ptr[i].depth);
+	printf("    red_mask        0x%06x\n", (unsigned)vinfo_ptr[i].red_mask);
+	printf("    green_mask      0x%06x\n", (unsigned)vinfo_ptr[i].green_mask);
+	printf("    blue_mask       0x%06x\n", (unsigned)vinfo_ptr[i].blue_mask);
+	printf("    colormap_size   %8d\n", vinfo_ptr[i].colormap_size);
+	printf("    bits_per_rgb    %8d\n", vinfo_ptr[i].bits_per_rgb);
     }
     XFree((void *) vinfo_ptr);
 }
@@ -448,8 +495,8 @@ static void Get_visual_info(void)
 	}
     }
     if (visual_class < 0 && visual_id < 0) {
-    	visual = DefaultVisual(dpy, DefaultScreen(dpy));
-    	if (visual->class == TrueColor || visual->class == DirectColor) {
+	visual = DefaultVisual(dpy, DefaultScreen(dpy));
+	if (visual->class == TrueColor || visual->class == DirectColor) {
 	    visual_class = PseudoColor;
 	}
     }
@@ -505,9 +552,132 @@ static void Get_visual_info(void)
 }
 
 
+/*
+ * Convert a string of color numbers into an array
+ * of "colors[]" indices stored by "spark_color[]".
+ * Initialize "num_spark_colors".
+ */
+static void Init_spark_colors(void)
+{
+    char		buf[MSG_LEN];
+    char		*src, *dst;
+    unsigned		col;
+    int			i;
+
+    num_spark_colors = 0;
+    /*
+     * The sparkColors specification may contain
+     * any possible separator.  Only look at numbers.
+     */
+    for (src = sparkColors; *src; src++) {
+	if (isascii(*src) && isdigit(*src)) {
+	    dst = &buf[0];
+	    do {
+		*dst++ = *src++;
+	    } while (*src && isascii(*src) && isdigit(*src));
+	    *dst = '\0';
+	    src--;
+	    if (sscanf(buf, "%u", &col) == 1) {
+		if (col < maxColors) {
+		    spark_color[num_spark_colors++] = col;
+		}
+	    }
+	}
+    }
+    if (num_spark_colors == 0) {
+	if (maxColors <= 8) {
+	    /* 3 colors ranging from 5 up to 7 */
+	    for (i = 5; i < maxColors; i++) {
+		spark_color[num_spark_colors++] = i;
+	    }
+	}
+	else {
+	    /* 7 colors ranging from 5 till 11 */
+	    for (i = 5; i < 12; i++) {
+		spark_color[num_spark_colors++] = i;
+	    }
+	}
+	/* default spark colors always include RED. */
+	spark_color[num_spark_colors++] = RED;
+    }
+    for (i = num_spark_colors; i < MAX_COLORS; i++) {
+	spark_color[i] = spark_color[num_spark_colors - 1];
+    }
+}
+
+
+/*
+ * Initialize miscellaneous window hints and properties.
+ */
+void Init_disp_prop(Display *d, Window win, int w, int h, int x, int y,
+		   int flags)
+{
+    extern char		**Argv;
+    extern int		Argc;
+    XClassHint		xclh;
+    XWMHints		xwmh;
+    XSizeHints		xsh;
+    char		msg[256];
+
+    xwmh.flags	   = InputHint|StateHint|IconPixmapHint;
+    xwmh.input	   = True;
+    xwmh.initial_state = NormalState;
+    xwmh.icon_pixmap   = XCreateBitmapFromData(d, win,
+					       (char *)icon_bits,
+					       icon_width, icon_height);
+
+    xsh.flags = (flags|PMinSize|PMaxSize|PBaseSize|PResizeInc);
+    xsh.width = w;
+    xsh.base_width =
+    xsh.min_width = MIN_TOP_WIDTH;
+    xsh.max_width = MAX_TOP_WIDTH;
+    xsh.width_inc = 1;
+    xsh.height = h;
+    xsh.base_height =
+    xsh.min_height = MIN_TOP_HEIGHT;
+    xsh.max_height = MAX_TOP_HEIGHT;
+    xsh.height_inc = 1;
+    xsh.x = x;
+    xsh.y = y;
+
+    xclh.res_name = NULL;	/* NULL: Automatically uses Argv[0], */
+    xclh.res_class = "XPilot"; /* stripped of directory prefixes. */
+
+    /*
+     * Set the above properties.
+     */
+    XSetWMProperties(d, win, NULL, NULL, Argv, Argc,
+		     &xsh, &xwmh, &xclh);
+
+    /*
+     * Now initialize icon and window title name.
+     */
+    if (titleFlip)
+	sprintf(msg, "Successful connection to server at \"%s\".",
+		servername);
+    else
+	sprintf(msg, "%s -- Server at \"%s\".", TITLE, servername);
+    XStoreName(d, win, msg);
+
+    sprintf(msg, "%s:%s", name, servername);
+    XSetIconName(d, win, msg);
+
+    if (d != dpy)
+	    return;
+
+    /*
+     * Specify IO error handler and the WM_DELETE_WINDOW atom in
+     * an attempt to catch 'nasty' quits.
+     */
+    ProtocolAtom = XInternAtom(d, "WM_PROTOCOLS", False);
+    KillAtom = XInternAtom(d, "WM_DELETE_WINDOW", False);
+    XSetWMProtocols(d, win, &KillAtom, 1);
+    XSetIOErrorHandler(FatalError);
+}
+
 /*
  * The following function initializes a player window.
- * It returns 0 if the initialization was successful, 
+ * It returns 0 if the initialization was successful,
  * or -1 if it couldn't initialize the double buffering routine.
  */
 int Init_window(void)
@@ -519,12 +689,12 @@ int Init_window(void)
 				x, y;
     unsigned			w, h;
     XGCValues			xgc;
-    char			msg[256];
     int				button_form,
 				menu_button;
     XSetWindowAttributes	sattr;
     unsigned long		mask;
-
+    Pixmap			pix;
+    GC				cursorGC;
 
     colormap = 0;
 
@@ -575,7 +745,16 @@ int Init_window(void)
 		    "Network Computing Devices Inc.") == 0
 	    && ProtocolVersion (dpy) == 11)
 	    shieldDrawMode = 1;
+
+#ifdef ERASE
+	/*
+	 * The NeWS X server doesn't orrectly erase shields.
+	 */
+	if (!strcmp(ServerVendor(dpy), "X11/NeWS - Sun Microsystems Inc."))
+	    shieldDrawMode = 1;
+#endif
     }
+
     shieldDrawMode = shieldDrawMode ? LineSolid : LineOnOffDash;
     radarPlayerRectFN = (mono ? XDrawRectangle : XFillRectangle);
 
@@ -628,10 +807,24 @@ int Init_window(void)
     for (i = maxColors; i < MAX_COLORS; i++) {
 	colors[i] = colors[i % maxColors];
     }
+    if (hudColor >= maxColors || hudColor <= 0) {
+	hudColor = BLUE;
+    }
+    if (hudLockColor >= maxColors || hudLockColor <= 0) {
+	hudLockColor = hudColor;
+    }
+    if (wallColor >= maxColors || wallColor <= 0) {
+	wallColor = BLUE;
+    }
+    if (targetRadarColor >= maxColors
+	|| (targetRadarColor != 8
+	    && targetRadarColor != 4
+	    && targetRadarColor != 2)) {
+	targetRadarColor = BLUE;
+    }
 
 
     about_created = false;
-    keys_created = false;
     talk_created = false;
     talk_mapped = false;
     talk_str[0] = '\0';
@@ -710,7 +903,14 @@ int Init_window(void)
 			0, dispDepth,
 			InputOutput, visual,
 			mask, &sattr);
-
+    if (kdpy) {
+	int scr = DefaultScreen(kdpy);
+	keyboard = XCreateSimpleWindow(kdpy,
+				       DefaultRootWindow(kdpy),
+				       top_x, top_y,
+				       top_width, top_height,
+				       0, 0, BlackPixel(dpy, scr));
+    }
 
     /*
      * Create item bitmaps
@@ -745,8 +945,6 @@ int Init_window(void)
 	= XCreateGC(dpy, top, values, &xgc);
     talkGC
 	= XCreateGC(dpy, top, values, &xgc);
-    keyListGC
-	= XCreateGC(dpy, top, values, &xgc);
     motdGC
 	= XCreateGC(dpy, top, values, &xgc);
     gc
@@ -770,8 +968,6 @@ int Init_window(void)
 	= Set_font(dpy, textGC, textFontName, "textFont");
     talkFont
 	= Set_font(dpy, talkGC, talkFontName, "talkFont");
-    keyListFont
-	= Set_font(dpy, keyListGC, keyListFontName, "keyListFont");
     motdFont
 	= Set_font(dpy, motdGC, motdFontName, "motdFont");
 
@@ -796,7 +992,7 @@ int Init_window(void)
 	      BlackPixel(dpy, DefaultScreen(dpy)),
 	      GXcopy, AllPlanes);
 
-    
+
     if (colorSwitch)
 	XSetPlaneMask(dpy, gc, dbuf_state->drawing_planes);
 
@@ -824,12 +1020,10 @@ int Init_window(void)
 	buttonColor = BLACK;
 	windowColor = BLACK;
 	borderColor = WHITE;
-	hudColor    = WHITE;
     } else {
 	windowColor = BLUE;
 	buttonColor = RED;
 	borderColor = WHITE;
-	hudColor    = 2;
     }
 
     draw_width = top_width - (256 + 2);
@@ -892,74 +1086,17 @@ int Init_window(void)
     XSelectInput(dpy, top,
 		 KeyPressMask | KeyReleaseMask
 		 | FocusChangeMask | StructureNotifyMask);
+    if (kdpy)
+	XSelectInput(kdpy, keyboard,
+		     KeyPressMask | KeyReleaseMask | FocusChangeMask);
     XSelectInput(dpy, radar, ExposureMask);
     XSelectInput(dpy, players, ExposureMask);
     XSelectInput(dpy, draw, 0);
 
-
-    /*
-     * Initialize miscellaneous window hints and properties.
-     */
-    {
-	extern char	**Argv;
-	extern int	Argc;
-	XClassHint	xclh;
-	XWMHints	xwmh;
-	XSizeHints	xsh;
-
-	xwmh.flags	   = InputHint|StateHint|IconPixmapHint; 
-	xwmh.input	   = True;
-	xwmh.initial_state = NormalState;
-	xwmh.icon_pixmap   = XCreateBitmapFromData(dpy, top,
-						   (char *)icon_bits,
-						   icon_width, icon_height);
-
-	xsh.flags = (top_flags|PMinSize|PMaxSize|PBaseSize|PResizeInc);
-	xsh.width = top_width;
-	xsh.base_width =
-	xsh.min_width = MIN_TOP_WIDTH;
-	xsh.max_width = MAX_TOP_WIDTH;
-	xsh.width_inc = 1;
-	xsh.height = top_height;
-	xsh.base_height =
-	xsh.min_height = MIN_TOP_HEIGHT;
-	xsh.max_height = MAX_TOP_HEIGHT;
-	xsh.height_inc = 1;
-	xsh.x = top_x;
-	xsh.y = top_y;
-
-	xclh.res_name = NULL;	/* NULL: Automatically uses Argv[0], */
-	xclh.res_class = "XPilot"; /* stripped of directory prefixes. */
-
-	/*
-	 * Set the above properties.
-	 */
-	XSetWMProperties(dpy, top, NULL, NULL, Argv, Argc,
-			 &xsh, &xwmh, &xclh);
-
-	/*
-	 * Now initialize icon and window title name.
-	 */
-	if (titleFlip)
-	    sprintf(msg, "Successful connection to server at \"%s\".",
-		    servername);
-	else
-	    sprintf(msg, "%s -- Server at \"%s\".", TITLE, servername);
-	XStoreName(dpy, top, msg);
-
-	sprintf(msg, "%s:%s", name, servername);
-	XSetIconName(dpy, top, msg);
-
-	/*
-	 * Specify IO error handler and the WM_DELETE_WINDOW atom in
-	 * an attempt to catch 'nasty' quits.
-	 */
-	ProtocolAtom = XInternAtom(dpy, "WM_PROTOCOLS", False);
-	KillAtom = XInternAtom(dpy, "WM_DELETE_WINDOW", False);
-	XSetWMProtocols(dpy, top, &KillAtom, 1);
-	XSetIOErrorHandler(FatalError);
-    }
-
+    Init_disp_prop(dpy, top, top_width, top_height, top_x, top_y, top_flags);
+    if (kdpy)
+	Init_disp_prop(kdpy, keyboard, top_width, top_height,
+		       top_x, top_y, top_flags);
 
     /*
      * Initialize misc. pixmaps if we're not color switching.
@@ -977,36 +1114,60 @@ int Init_window(void)
     }
 
     XAutoRepeatOff(dpy);	/* We don't want any autofire, yet! */
+    if (kdpy)
+	    XAutoRepeatOff(kdpy);
 
+    /*
+     * Define a blank cursor for use with pointer control
+     */
+    XQueryBestCursor(dpy, draw, 1, 1, &w, &h);
+    pix = XCreatePixmap(dpy, draw, w, h, 1);
+    cursorGC = XCreateGC(dpy, pix, 0, NULL);
+    XSetForeground(dpy, gc, 0);
+    XFillRectangle(dpy, pix, cursorGC, 0, 0, w, h);
+    XFreeGC(dpy, cursorGC);
+    pointerControlCursor = XCreatePixmapCursor(dpy, pix, pix, &colors[BLACK],
+					       &colors[BLACK], 0, 0);
+    XFreePixmap(dpy, pix);
 
     /*
      * Maps the windows, makes the visible. Voila!
      */
     XMapSubwindows(dpy, top);
     XMapWindow(dpy, top);
-
-
     XSync(dpy, False);
 
+    if (kdpy) {
+	XMapWindow(kdpy, keyboard);
+	XSync(kdpy, False);
+    }
+
+    Init_spark_colors();
+
     return 0;
 }
 
 
-int Alloc_msgs(int number)
+int Alloc_msgs(void)
 {
-    message_t *x;
-    int i;
+    message_t		*x;
+    int			i;
 
-    if ((x = (message_t *)malloc(number * sizeof(message_t))) == NULL) {
+    if ((x = (message_t *)malloc(2 * MAX_MSGS * sizeof(message_t))) == NULL) {
 	error("No memory for messages");
 	return -1;
     }
     MsgBlock = x;
-    for (i=0; i<number; i++) {
-	Msg[i]=x;
+    for (i = 0; i < 2 * MAX_MSGS; i++) {
+	if (i < MAX_MSGS) {
+	    TalkMsg[i] = x;
+	} else {
+	    GameMsg[i - MAX_MSGS] = x;
+	}
 	x->txt[0] = '\0';
+	x->len = 0;
 	x->life = 0;
-        x++;
+	x++;
     }
     return 0;
 }
@@ -1019,7 +1180,8 @@ void Free_msgs(void)
 
 
 void ShadowDrawString(Display* dpy, Window w, GC gc,
-		      int x, int y, char* str, Pixel fg, Pixel bg)
+		      int x, int y, char* str,
+		      unsigned long fg, unsigned long bg)
 {
     if (HaveColor(dpy)) {
 	XSetForeground(dpy, gc, bg);
@@ -1037,8 +1199,8 @@ void ShadowDrawString(Display* dpy, Window w, GC gc,
  * vertical position it ended at.
  */
 int DrawShadowText(Display* dpy, Window w, GC gc,
-		    int x_border, int y_start,
-		    char *str, Pixel fg, Pixel bg)
+		    int x_border, int y_start, char *str,
+		    unsigned long fg, unsigned long bg)
 {
     XFontStruct*	font = XQueryFont(dpy, XGContextFromGC(gc));
     int			y, x;
@@ -1173,7 +1335,8 @@ void Expose_about_window(void)
 	     * remove this item, set itemsplit to previous item and
 	     * stop adding more items.
 	     */
-	    if (about_page == 0 && itemsplit == -1
+	    if (about_page == 0
+		&& itemsplit == -1
 		&& box_end >= (ABOUT_WINDOW_HEIGHT - BORDER * 2 - 4
 			       - (2*BTN_BORDER + buttonFont->ascent
 				  + buttonFont->descent))) {
@@ -1188,7 +1351,7 @@ void Expose_about_window(void)
 
 	    y = box_end;
 	    box_start = box_end;
-		
+
 	}
 	/*
 	 * No page split, obviously font is small enough or not enough
@@ -1198,12 +1361,18 @@ void Expose_about_window(void)
 	    itemsplit = NUM_ITEMS-1;
 	}
 	break;
-	    
+
     case 2:
 	DrawShadowText(dpy, about_w, textGC,
 	BORDER, BORDER,
 	"GAME OBJECTIVE\n"
 	"\n"
+	"XPilot is a multi-player 2D space game.  "
+	"Some features are borrowed from classics like the Atari coin-ups "
+	"Asteriods and Gravitar, and the home-computer games "
+	"Thrust (Commdore 64) and Gravity Force, but XPilot has many "
+	"new features as well.\n"
+	"\n"
 	"The primary goal of the game is to collect points and increase "
 	"your rating by destroying enemy fighters and cannons.  "
 	"You are equipped with a machine gun when you start the game, "
@@ -1227,15 +1396,21 @@ void Expose_about_window(void)
 	BORDER, BORDER,
 	"ABOUT XPILOT\n"
 	"\n"
-	"XPilot is still not a finished product, so apologies for "
-	"any bugs etc.  However, if you find any, we would greatly "
-	"appreciate that you reported to us.\n"
+	"The game was conceived in its orignal form at the "
+	"University of Troms (Norway) by Ken Ronny Schouten and "
+	"Bjrn Stabell during the fall of 1991, but much of the game today "
+	"is the result of hard efforts by Bert Gsbers of the "
+	"molecular cytology lab at the University of Amsterdam (The Netherlands).  "
+	"Bert joined the team in the spring of 1993.\n"
 	"\n"
-	"New versions are continuously being developed, but at a random "
-	"rate.  Currently, this isn't very fast at all, mainly due to the "
-	"mandatory work in conjunction with our studies (really!!).\n"
+	"A large number of features have been contributed by XPilot fans from "
+	"all around the world.  See the CREDITS file for details.\n"
 	"\n"
-	"For more info, read the man pages for xpilot(6) and xpilots(6).\n"
+	"For more information, "
+	"read the XPilot FAQ (Frequently Asked Questions), "
+	"and the on-line manual pages for xpilot(6) and xpilots(6).\n"
+	"\n"
+	"Bugs should be reported to <xpilot@cs.uit.no>.\n"
 	"\n\n"
 	"Good luck as a future xpilot,\n"
 	"Bjrn Stabell, Ken Ronny Schouten & Bert Gsbers",
@@ -1252,7 +1427,7 @@ void Expose_about_window(void)
 static void createAboutWindow(void)
 {
     const int			windowWidth = ABOUT_WINDOW_WIDTH,
-    				buttonWindowHeight = 2*BTN_BORDER
+				buttonWindowHeight = 2*BTN_BORDER
 				    + buttonFont->ascent + buttonFont->descent,
 				windowHeight = ABOUT_WINDOW_HEIGHT;
     int				textWidth;
@@ -1331,147 +1506,11 @@ static void createAboutWindow(void)
 }
 
 
-static void createKeysWindow(void)
-{
-    const int			buttonWindowHeight = 2*BTN_BORDER
-				    + buttonFont->ascent + buttonFont->descent;
-    int				windowWidth = 0,
-				windowHeight = 0,
-				maxKeyNameLen = 0,
-				maxKeyDescLen = 0;
-    XSetWindowAttributes	sattr;
-    unsigned long		mask;
-    extern char*		Get_keyhelpstring(keys_t);
-
-    if (keyHelpList == NULL) {
-	int	sizeList = 1, sizeDesc = 1, i = 0;
-
-	/*
-	 * Make sure the strings are empty and null terminated.
-	 */
-	keyHelpList = (char *)malloc(1);
-	keyHelpList[0] = '\0';
-	keyHelpDesc = (char *)malloc(1);
-	keyHelpDesc[0] = '\0';
-
-	/*
-	 * Build the key help string, and while we're at it find
-	 * the extent (physical :) of the text.
-	 */
-	for (i = 0; i < maxKeyDefs; i++) {
-	    int len;
-	    char* str;
-
-	    /*
-	     * Key name
-	     */
-	    str = XKeysymToString(keyDefs[i].keysym);
-	    sizeList += strlen(str) + 1;
-	    keyHelpList = (char *)realloc(keyHelpList, sizeList);
-	    strcat(keyHelpList, str);
-	    keyHelpList[sizeList-2] = '\n';
-	    keyHelpList[sizeList-1] = '\0';
-
-	    /* Store longest keysym name */
-	    len = XTextWidth(keyListFont, str, strlen(str));
-	    if (len > maxKeyNameLen)
-		maxKeyNameLen = len;
-
-	    /*
-	     * Description of action invoked for this key
-	     */
-	    if (!(str = Get_keyhelpstring(keyDefs[i].key))) continue;
-	    sizeDesc += strlen(str) + 1;
-	    keyHelpDesc = (char *)realloc(keyHelpDesc, sizeDesc);
-	    strcat(keyHelpDesc, str);
-	    keyHelpDesc[sizeDesc-2] = '\n';
-	    keyHelpDesc[sizeDesc-1] = '\0';
-
-	    /* Store longest desc */
-	    len = XTextWidth(keyListFont, str, strlen(str));
-	    if (len > maxKeyDescLen)
-		maxKeyDescLen = len;
-
-	    windowHeight += keyListFont->ascent + keyListFont->descent + 1;
-	}
-    }
-
-    /*
-     * Now calculate window dimensions and the offset we need to
-     * put the description (the x coordinate it should begin on).
-     */
-    windowHeight += buttonWindowHeight + 4 + 3*BORDER;
-    windowWidth = 4*BORDER + maxKeyNameLen + maxKeyDescLen;
-    KeyDescOffset = 3*BORDER + maxKeyNameLen;
-
-    /*
-     * Create the window and initialize window name.
-     */
-    mask = 0;
-    sattr.background_pixel = colors[windowColor].pixel;
-    mask |= CWBackPixel;
-    sattr.border_pixel = colors[borderColor].pixel;
-    mask |= CWBorderPixel;
-    if (colormap != 0) {
-	sattr.colormap = colormap;
-	mask |= CWColormap;
-    }
-    sattr.backing_store = Always;
-    mask |= CWBackingStore;
-
-    keys_w
-	= XCreateWindow(dpy,
-			DefaultRootWindow(dpy),
-			0, 0,
-			windowWidth, windowHeight,
-			2, dispDepth,
-			InputOutput, visual,
-			mask, &sattr);
-    XStoreName(dpy, keys_w, "XPilot - key reference");
-    XSetIconName(dpy, keys_w, "XPilot/keys");
-    XSetTransientForHint(dpy, keys_w, top);
-
-    /*
-     * Create buttons.
-     */
-    keys_close_b
-	= XCreateSimpleWindow(dpy, keys_w,
-			      BORDER,
-			      (windowHeight - BORDER - buttonWindowHeight - 4),
-			      (XTextWidth(buttonFont,
-					  "CLOSE", 5) + 2*BTN_BORDER),
-			      buttonWindowHeight,
-			      0, 0,
-			      colors[buttonColor].pixel);
-    XSelectInput(dpy, keys_close_b,
-		 ExposureMask | ButtonPressMask | ButtonReleaseMask);
-    XSelectInput(dpy, keys_w, ExposureMask);
-
-    Expose_keys_window();
-
-    XMapSubwindows(dpy, keys_w);
-}
-
-    
-void Expose_keys_window(void)
-{
-    XClearWindow(dpy, keys_w);
-
-    DrawShadowText(dpy, keys_w, keyListGC,
-		   BORDER, BORDER, keyHelpList,
-		   colors[WHITE].pixel, colors[BLACK].pixel);
-    DrawShadowText(dpy, keys_w, keyListGC,
-		   KeyDescOffset, BORDER, keyHelpDesc,
-		   colors[WHITE].pixel, colors[BLACK].pixel);
-}
-
-
 void Expose_button_window(int color, Window w)
 {
     if (w != about_close_b
 	&& w != about_next_b
-	&& w != about_prev_b
-	&& w != keys_close_b) {
+	&& w != about_prev_b) {
 	return;
     }
 
@@ -1500,12 +1539,6 @@ void Expose_button_window(int color, Window w)
 			 BTN_BORDER, buttonFont->ascent + BTN_BORDER,
 			 "PREV",
 			 colors[WHITE].pixel, colors[BLACK].pixel);
-
-    if (w == keys_close_b)
-	ShadowDrawString(dpy, w, buttonGC,
-			 BTN_BORDER, buttonFont->ascent + BTN_BORDER,
-			 "CLOSE",
-			 colors[WHITE].pixel, colors[BLACK].pixel);
 }
 
 
@@ -1547,25 +1580,60 @@ static int About_callback(int widget_desc, void *data, char **str)
 }
 
 
-void Keys(Window w)
+static int Keys_callback(int widget_desc, void *data, char **unused)
 {
+    static bool		keys_created = false;
+    static int		keys_viewer;
+
     if (keys_created == false) {
-	createKeysWindow();
+	unsigned	bufsize = 1;
+	char		*buf = (char *)calloc(1, 1),
+			*end,
+			*help,
+			*str;
+	int		i,
+			len,
+			maxkeylen = 0;
+	extern char* Get_keyHelpString(keys_t key);
+
+	for (i = 0; i < maxKeyDefs; i++) {
+	    if ((str = XKeysymToString(keyDefs[i].keysym)) != NULL
+		&& (len = strlen(str)) > maxkeylen) {
+		maxkeylen = len;
+	    }
+	}
+	for (i = 0; i < maxKeyDefs; i++) {
+	    if (!(str = XKeysymToString(keyDefs[i].keysym))
+		|| !(help = Get_keyHelpString(keyDefs[i].key))) {
+		continue;
+	    }
+	    bufsize += strlen(str) + strlen(help) + maxkeylen + 4;
+	    if (!(buf = (char *)realloc(buf, bufsize))) {
+		error("No memory for key list");
+		return 0;
+	    }
+	    end = &buf[strlen(buf)];
+	    sprintf(end, "%-*s  %s\n", maxkeylen, str, help);
+	}
+	keys_viewer =
+	    Widget_create_viewer(buf,
+				 strlen(buf),
+				 2*DisplayWidth(dpy, DefaultScreen(dpy))/3,
+				 4*DisplayHeight(dpy, DefaultScreen(dpy))/5,
+				 2,
+				 "XPilot - key reference", "XPilot:keys",
+				 motdFont);
+	if (keys_viewer == NO_WIDGET) {
+	    errno = 0;
+	    error("Can't create key viewer");
+	    return 0;
+	}
+
 	keys_created = true;
     }
-    if (w == keys_close_b) {
-	XUnmapWindow(dpy, keys_w);
+    else if (keys_viewer != NO_WIDGET) {
+	Widget_map(keys_viewer);
     }
-}
-
-
-static int Keys_callback(int widget_desc, void *data, char **str)
-{
-    if (keys_created == false) {
-	createKeysWindow();
-	keys_created = true;
-    }
-    XMapWindow(dpy, keys_w);
     return 0;
 }
 
@@ -1654,7 +1722,14 @@ void Talk_cursor(bool visible)
 
 void Talk_map_window(bool map)
 {
+    static Window	root;
+    static int		root_x, root_y;
+
     if (map == true) {
+	Window child;
+	int win_x, win_y;
+	unsigned int keys_buttons;
+
 	if (talk_created == false) {
 	    createTalkWindow();
 	    talk_created = true;
@@ -1662,6 +1737,9 @@ void Talk_map_window(bool map)
 	XMapWindow(dpy, talk_w);
 	talk_mapped = true;
 
+	XQueryPointer(dpy, DefaultRootWindow(dpy),
+		      &root, &child, &root_x, &root_y, &win_x, &win_y,
+		      &keys_buttons);
 	XWarpPointer(dpy, None, talk_w,
 		     0, 0, 0, 0,
 		     TALK_WINDOW_WIDTH - 2,
@@ -1669,20 +1747,30 @@ void Talk_map_window(bool map)
     }
     else if (talk_created == true) {
 	XUnmapWindow(dpy, talk_w);
+	XWarpPointer(dpy, None, root, 0, 0, 0, 0, root_x, root_y);
 	talk_mapped = false;
     }
     talk_cursor.visible = false;
 }
 
 
-void Talk_event(XEvent *event)
+int Talk_do_event(XEvent *event)
 {
     char		ch;
     bool		cursor_visible = talk_cursor.visible;
     int			count, oldlen, newlen, onewidth, oldwidth;
     KeySym		keysym;
-    XComposeStatus	compose;
     char		new_str[MAX_CHARS];
+    bool		result = true;
+
+    /* according to Ralf Berger <gotan@physik.rwth-aachen.de>
+     * compose should be static:
+     * the value of 'compose' has to be preserved across calls to
+     * 'XLookupString' (the man page also mentioned that a portable program 
+     * should pass NULL but i don't know if that's specific to dec-alpha ?).
+     * To fix this declare 'compose' static (can't hurt anyway).
+     */
+    static XComposeStatus	compose;
 
     switch (event->type) {
 
@@ -1729,7 +1817,7 @@ void Talk_event(XEvent *event)
 		talk_cursor.point = 0;
 		talk_str[0] = '\0';
 	    }
-	    Talk_map_window(false);
+	    result = false;
 	    break;
 
 	case '\033':
@@ -1738,7 +1826,7 @@ void Talk_event(XEvent *event)
 	     */
 	    talk_str[0] = '\0';
 	    talk_cursor.point = 0;
-	    Talk_map_window(false);
+	    result = false;
 	    break;
 
 	case CTRL('A'):
@@ -1933,6 +2021,8 @@ void Talk_event(XEvent *event)
     default:
 	break;
     }
+
+    return result;	/* keep on talking if true, no more talking if false */
 }
 
 
@@ -1986,6 +2076,11 @@ void Quit(void)
 	}
 	XCloseDisplay(dpy);
 	dpy = NULL;
+	if (kdpy) {
+	    XAutoRepeatOn(kdpy);
+	    XCloseDisplay(kdpy);
+	    kdpy = NULL;
+	}
     }
 }
 
@@ -2071,7 +2166,7 @@ int Handle_motd(long off, char *buf, int len, long filesize)
 	    Widget_create_viewer(motd_buf,
 				 (off || len) ? (off + len) : strlen(motd_buf),
 				 2*DisplayWidth(dpy, DefaultScreen(dpy))/3,
-				 4*DisplayHeight(dpy, DefaultScreen(dpy))/5,
+				 4*DisplayHeight(dpy, DefaultScreen(dpy))/8,
 				 2,
 				 title, "XPilot:motd",
 				 motdFont);
