@@ -1,6 +1,6 @@
-/* $Id: xpilot.c,v 3.42 1994/04/05 20:27:38 bert Exp $
+/* $Id: xpilot.c,v 3.61 1995/03/06 09:53:20 bert Exp $
  *
- * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-94 by
+ * XPilot, a multiplayer gravity war game.  Copyright (C) 1991-95 by
  *
  *      Bjrn Stabell        (bjoerns@staff.cs.uit.no)
  *      Ken Ronny Schouten   (kenrsc@stud.cs.uit.no)
@@ -21,6 +21,9 @@
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
+#ifdef _IBMESA
+# define _SOCKADDR_LEN
+#endif
 #ifdef VMS
 #include <unixio.h>
 #include <unixlib.h>
@@ -52,7 +55,7 @@
 #else
 #include <sys/time.h>
 #endif
-#ifndef LINUX
+#ifndef LINUX0
 #include <net/if.h>
 #endif
 #include <netdb.h>
@@ -60,48 +63,62 @@
 
 #include "version.h"
 #include "config.h"
+#include "const.h"
 #include "types.h"
 #include "pack.h"
 #include "bit.h"
 #include "error.h"
+#include "socklib.h"
 #include "net.h"
 #ifdef SUNCMW
 #include "cmw.h"
 #endif /* SUNCMW */
 
+char xpilot_version[] = VERSION;
+
 #ifndef	lint
 static char versionid[] = "@(#)$" TITLE " $";
 static char sourceid[] =
-    "@(#)$Id: xpilot.c,v 3.42 1994/04/05 20:27:38 bert Exp $";
+    "@(#)$Id: xpilot.c,v 3.61 1995/03/06 09:53:20 bert Exp $";
 #endif
 
-#if defined(LINUX) || defined(VMS)
+#if defined(LINUX0) || defined(VMS)
 # ifndef QUERY_FUDGED
 #  define QUERY_FUDGED
 # endif
 #endif
 
+#ifdef	LIMIT_ACCESS
+extern bool		Is_allowed(char *);
+#endif
+extern void Parse_options(int *argcp, char **argvp, char *realName, int *port,
+			  int *my_team, int *list, int *join, int *noLocalMotd,
+			  char *nickName, char *dispName, char *shut_msg);
+extern int Join(char *server_addr, char *server_name, int port,
+		char *real, char *nick, int my_team,
+		char *display, unsigned version);
+extern int micro_delay(unsigned usec);
+
+
 #define MAX_LINE	256	/* should not be smaller than MSG_LEN */
 
 
-static int		socket_c,		/* Contact socket */
-    			contact_port = SERVER_PORT,
-    			server_port,
-    			login_port;
+static int		contact_port = SERVER_PORT,
+			server_port,
+			login_port;
 static char		nick_name[MAX_NAME_LEN],
 			real_name[MAX_NAME_LEN],
-    			server_addr[MAXHOSTNAMELEN],
-    			server_name[MAXHOSTNAMELEN],
-    			hostname[MAXHOSTNAMELEN],
-    			display[MAX_DISP_LEN],
+			server_addr[MAXHOSTNAMELEN],
+			server_name[MAXHOSTNAMELEN],
+			display[MAX_DISP_LEN],
 			shutdown_reason[MAX_CHARS];
+char			hostname[MAXHOSTNAMELEN];
 static int		auto_connect = false,
-    			list_servers = false,
+			list_servers = false,
 			noLocalMotd = false,
-    			auto_shutdown = false;
+			auto_shutdown = false;
 static unsigned		server_version;
 static int		team = TEAM_NOT_SET;
-static sockbuf_t	sbuf;			/* contact buffer */
 
 char			**Argv;
 int			Argc;
@@ -109,8 +126,7 @@ int			Argc;
 
 static int Query_all(int sockfd, int port, char *msg, int msglen);
 static int Query_fudged(int sockfd, int port, char *msg, int msglen);
-
-extern void usleep(unsigned long);
+static void Check_client_versions(void);
 
 
 static void printfile(char *filename)
@@ -131,26 +147,26 @@ static void printfile(char *filename)
 
 
 
-static bool Get_contact_message(void)
+static bool Get_contact_message(sockbuf_t *sbuf)
 {
     int			len;
     unsigned		magic;
     unsigned char	reply_to, status;
     bool		readable = false;
 
-    SetTimeout(3, 0);
-    while (readable == false && SocketReadable(socket_c) > 0) {
+    SetTimeout(2, 0);
+    while (readable == false && SocketReadable(sbuf->sock) > 0) {
 
-	Sockbuf_clear(&sbuf);
-	len = DgramReceiveAny(sbuf.sock, sbuf.buf, sbuf.size);
+	Sockbuf_clear(sbuf);
+	len = DgramReceiveAny(sbuf->sock, sbuf->buf, sbuf->size);
 	if (len <= 0) {
 	    if (len == 0) {
 		continue;
 	    }
 	    error("DgramReceiveAny, contact message");
-	    exit(-1);
+	    exit(1);
 	}
-	sbuf.len = len;
+	sbuf->len = len;
 
 	/*
 	 * Get server's host and port.
@@ -159,7 +175,7 @@ static bool Get_contact_message(void)
 	server_port = DgramLastport();
 	strcpy(server_name, DgramLastname());
 
-	if (Packet_scanf(&sbuf, "%u%c%c", &magic, &reply_to, &status) <= 0) {
+	if (Packet_scanf(sbuf, "%u%c%c", &magic, &reply_to, &status) <= 0) {
 	    errno = 0;
 	    error("Incomplete contact reply message (%d)", len);
 	}
@@ -200,7 +216,7 @@ static int Get_reply_message(sockbuf_t *ibuf)
 	if ((len = read(ibuf->sock, ibuf->buf, ibuf->size)) == -1) {
 	    error("Can't read reply message from %s/%d",
 		  server_addr, server_port);
-	    exit(-1);
+	    exit(1);
 	}
 
 	ibuf->len = len;
@@ -224,12 +240,38 @@ static int Get_reply_message(sockbuf_t *ibuf)
 	}
     } else
 	return (0);
-    
+
     return (len);
 }
 
 
 
+static void Command_help(void)
+{
+    printf("Supported commands are:\n"
+	   "H/?  -   Help - this text.\n"
+	   "N    -   Next server, skip this one.\n"
+	   "S    -   list Status.\n"
+	   "T    -   set Team.\n"
+	   "Q    -   Quit.\n"
+	   "K    -   Kick a player.                (only owner)\n"
+	   "M    -   send a Message.               (only owner)\n"
+	   "L    -   Lock/unLock server access.    (only owner)\n"
+	   "D(*) -   shutDown/cancel shutDown.     (only owner)\n"
+	   "R(#) -   set maximum number of Robots. (only owner)\n"
+	   "O    -   Modify a server option.       (only owner)\n"
+	   "V    -   View the server options.\n"
+	   "J(&) or just Return enters the game.\n"
+	   "(*) If you don't specify any delay, you will signal that\n"
+	   "    the server should stop an ongoing shutdown.\n"
+	   "(#) Not specifying the maximum number of robots is\n"
+	   "    the same as specifying 0 robots.\n"
+	   "(&) You may specify a team number after the J.\n"
+	  );
+}
+
+
+
 /*
  * This is the routine that interactively (if not auto_connect) prompts
  * the user on his/her next action.  Returns true if player joined this
@@ -241,6 +283,12 @@ static bool Process_commands(sockbuf_t *ibuf)
     int			i, len, retries, delay, max_robots;
     char		c, status, reply_to, str[MAX_LINE];
     unsigned short	port;
+    int			has_credentials = 0;
+    int			cmd_credentials = 0;
+    int			privileged_cmd;
+    int			max_replies;
+    long		key = 0;
+    static char		localhost[] = "127.0.0.1";
 
 
     if (auto_connect && !list_servers && !auto_shutdown) {
@@ -249,17 +297,26 @@ static bool Process_commands(sockbuf_t *ibuf)
 
     for (;;) {
 
+	max_replies = 1;
+
 	/*
 	 * Now, what do you want from the server?
 	 */
-	if (!auto_connect) {
+	if (cmd_credentials) {
+	    c = cmd_credentials;
+	    cmd_credentials = 0;
+	}
+	else if (!auto_connect) {
 	    printf("*** Server on %s. Enter command> ", server_name);
 
 	    gets(str);
-	    c = str[0];
 	    if (feof(stdin)) {
 		puts("");
 		c = 'Q';
+	    } else {
+		c = str[0];
+		if (c == '\0')
+		    c = 'J';
 	    }
 	    CAP_LETTER(c);
 	} else {
@@ -271,162 +328,213 @@ static bool Process_commands(sockbuf_t *ibuf)
 		c = 'J';
 	}
 
+	/*
+	 * For each command to the server create a new socket and bind
+	 * the socket to the server's address and destination port.
+	 * This assures us that we only get replies to the last command sent.
+	 */
+	if (ibuf->sock != -1) {
+	    SocketClose(ibuf->sock);
+	    ibuf->sock = -1;
+	}
+
+	privileged_cmd = (strchr("DKLMOR", c) != NULL);
+	if (privileged_cmd) {
+	    if (!has_credentials) {
+		if ((ibuf->sock = CreateDgramAddrSocket(server_addr, 0)) == -1) {
+		    printf("Server %s is not local, privileged command not possible.\n",
+			   server_addr);
+		    continue;
+		}
+		close(ibuf->sock);
+	    }
+	    if ((ibuf->sock = CreateDgramAddrSocket(localhost, 0)) == -1) {
+		error("Could not create localhost socket");
+		exit(1);
+	    }
+	    if (DgramConnect(ibuf->sock, localhost, server_port) == -1) {
+		error("Can't connect to server %s on port %d\n",
+		      localhost, server_port);
+		return (false);
+	    }
+	} else {
+	    if ((ibuf->sock = CreateDgramSocket(0)) == -1) {
+		error("Could not create socket");
+		exit(1);
+	    }
+	    if (DgramConnect(ibuf->sock, server_addr, server_port) == -1) {
+		error("Can't connect to server %s on port %d\n",
+		      server_addr, server_port);
+		return (false);
+	    }
+	}
+
 	Sockbuf_clear(ibuf);
 	Packet_printf(ibuf, "%u%s%hu", VERSION2MAGIC(server_version),
 		      real_name, GetPortNum(ibuf->sock));
 
-	switch (c) {
+	if (privileged_cmd && !has_credentials) {
+	    Packet_printf(ibuf, "%c%ld", CREDENTIALS_pack, 0L);
+	} else {
 
-	    /*
-	     * Owner only commands:
-	     */
-	case 'K':
-	    printf("Enter name of victim: ");
-	    fflush(stdout);
-	    gets(str);
-	    str[MAX_NAME_LEN - 1] = '\0';
-	    Packet_printf(ibuf, "%c%s", KICK_PLAYER_pack, str);
-	    break;
+	    switch (c) {
 
-	case 'R':
-	    printf("Enter maximum number of robots: ");
-	    fflush(stdout);
-	    gets(str);
-	    if (sscanf(str, "%d", &max_robots) <= 0) {
-		max_robots = 0;
-	    } else
-		if (max_robots < 0)
-		    max_robots = 0;
-	    Packet_printf(ibuf, "%c%d", MAX_ROBOT_pack, max_robots);
-	    break;
+		/*
+		 * Owner only commands:
+		 */
 
-	case 'M':				/* Send a message to server. */
-	    printf("Enter message: ");
-	    fflush(stdout);
-	    gets(str);
-	    str[MAX_CHARS - 1] = '\0';
-	    Packet_printf(ibuf, "%c%s", MESSAGE_pack, str);
-	    break;
+	    case 'K':
+		printf("Enter name of victim: ");
+		fflush(stdout);
+		if (!gets(str)) {
+		    printf("Nothing changed.\n");
+		    continue;
+		}
+		str[MAX_NAME_LEN - 1] = '\0';
+		Packet_printf(ibuf, "%c%ld%s", KICK_PLAYER_pack, key, str);
+		break;
 
-	    /*
-	     * Public commands:
-	     */
-	case 'N':				/* Next server. */
-	    return (false);
-	    break;
+	    case 'R':
+		printf("Enter maximum number of robots: ");
+		fflush(stdout);
+		if (!gets(str)) {
+		    printf("Nothing changed.\n");
+		    continue;
+		}
+		if (sscanf(str, "%d", &max_robots) <= 0 || max_robots < 0) {
+		    printf("Invalid number of robots \"%s\".\n", str);
+		    continue;
+		}
+		Packet_printf(ibuf, "%c%ld%d", MAX_ROBOT_pack, key, max_robots);
+		break;
 
-	case 'O':				/* Tune an option. */
-	    printf("Enter option: ");
-	    fflush(stdout);
-	    if (!gets(str) || (len = strlen(str)) == 0) {
-		printf("Nothing changed.\n");
-		continue;
-	    }
-	    printf("Enter new value for %s: ", str);
-	    fflush(stdout);
-	    strcat(str, ":"); len++;
-	    if (!gets(&str[len]) || str[len] == '\0') {
-		printf("Nothing changed.\n");
-		continue;
-	    }
-	    Packet_printf(ibuf, "%c%S", OPTION_TUNE_pack, str);
-	    break;
+	    case 'M':				/* Send a message to server. */
+		printf("Enter message: ");
+		fflush(stdout);
+		if (!gets(str) || !str[0]) {
+		    printf("No message sent.\n");
+		    continue;
+		}
+		str[MAX_CHARS - 1] = '\0';
+		Packet_printf(ibuf, "%c%ld%s", MESSAGE_pack, key, str);
+		break;
 
-	case 'V':				/* View options. */
-	    Packet_printf(ibuf, "%c", OPTION_LIST_pack);
-	    break;
+	    case 'L':				/* Lock the game. */
+		Packet_printf(ibuf, "%c%ld", LOCK_GAME_pack, key);
+		break;
 
-	case 'T':				/* Set team. */
-	    printf("Enter team: ");
-	    fflush(stdout);
-	    if (!gets(str) || (len = strlen(str)) == 0) {
-		printf("Nothing changed.\n");
-	    }
-	    else {
-		int newteam;
-		if (sscanf(str, " %d", &newteam) != 1
-		    || newteam < 0 || newteam > 9) {
-		    printf("Invalid team specification: %s.\n", str);
+	    case 'D':				/* Shutdown */
+		if (!auto_shutdown) {
+		    printf("Enter delay in seconds or return for cancel: ");
+		    gets(str);
+		    /*
+		     * No argument = cancel shutdown = arg_int=0
+		     */
+		    if (sscanf(str, "%d", &delay) <= 0) {
+			delay = 0;
+		    } else
+			if (delay <= 0)
+			    delay = 1;
+
+		    printf("Enter reason: ");
+		    gets(str);
+		} else {
+		    strcpy(str, shutdown_reason);
+		    delay = 60;
+		}
+		str[MAX_CHARS - 1] = '\0';
+		Packet_printf(ibuf, "%c%ld%d%s", SHUTDOWN_pack, key, delay, str);
+		break;
+
+	    case 'O':				/* Tune an option. */
+		printf("Enter option: ");
+		fflush(stdout);
+		if (!gets(str) || (len = strlen(str)) == 0) {
+		    printf("Nothing changed.\n");
+		    continue;
+		}
+		printf("Enter new value for %s: ", str);
+		fflush(stdout);
+		strcat(str, ":"); len++;
+		if (!gets(&str[len]) || str[len] == '\0') {
+		    printf("Nothing changed.\n");
+		    continue;
+		}
+		Packet_printf(ibuf, "%c%ld%S", OPTION_TUNE_pack, key, str);
+		break;
+
+		/*
+		 * Public commands:
+		 */
+
+	    case 'J':				/* Trying to enter game. */
+		if (str[1] >= '0' && str[1] <= '9') {
+		    team = str[1] - '0';
+		    printf("Joining team %d\n", team);
+		}
+		else if (str[1] == '-') {
+		    team = TEAM_NOT_SET;
+		    printf("Team set to unspecified\n");
+		}
+		Packet_printf(ibuf, "%c%s%s%s%d", ENTER_GAME_pack,
+			      nick_name, display, hostname, team);
+		break;
+
+	    case 'S':				/* Report status. */
+		Packet_printf(ibuf, "%c", REPORT_STATUS_pack);
+		break;
+
+	    case 'V':				/* View options. */
+		Packet_printf(ibuf, "%c", OPTION_LIST_pack);
+		max_replies = 5;
+		break;
+
+		/*
+		 * User interface commands:
+		 */
+
+	    case 'N':				/* Next server. */
+		return (false);
+
+	    case 'T':				/* Set team. */
+		printf("Enter team: ");
+		fflush(stdout);
+		if (!gets(str) || (len = strlen(str)) == 0) {
+		    printf("Nothing changed.\n");
 		}
 		else {
-		    team = newteam;
-		    printf("Team set to %d\n", team);
+		    int newteam;
+		    if (sscanf(str, " %d", &newteam) != 1) {
+			printf("Invalid team specification: %s.\n", str);
+		    }
+		    else if (newteam >= 0 && newteam <= 9) {
+			team = newteam;
+			printf("Team set to %d\n", team);
+		    }
+		    else {
+			team = TEAM_NOT_SET;
+			printf("Team set to unspecified\n");
+		    }
 		}
-	    }
-	    continue;
+		continue;
 
-	case 'S':				/* Report status. */
-	    Packet_printf(ibuf, "%c", REPORT_STATUS_pack);
-	    break;
+	    case 'Q':
+		exit (0);
+		break;
+
+	    case '?':
+	    case 'H':				/* Help. */
+	    default:
+		Command_help();
 
-	case 'D':				/* Shutdown */
-	    if (!auto_shutdown) {
-		printf("Enter delay in seconds or return for cancel: ");
-		gets(str);
 		/*
-		 * No argument = cancel shutdown = arg_int=0
+		 * Next command.
 		 */
-		if (sscanf(str, "%d", &delay) <= 0) {
-		    delay = 0;
-		} else
-		    if (delay <= 0)
-			delay = 1;
-
-		printf("Enter reason: ");
-		gets(str);
-	    } else {
-		strcpy(str, shutdown_reason);
-		delay = 60;
+		continue;
 	    }
-	    str[MAX_CHARS - 1] = '\0';
-	    Packet_printf(ibuf, "%c%d%s", SHUTDOWN_pack, delay, str);
-	    break;
-
-	case 'Q':
-	    exit (0);
-	    break;
-
-	case 'L':				/* Lock the game. */
-	    Packet_printf(ibuf, "%c", LOCK_GAME_pack);
-	    break;
-
-	case '\0':
-	    c = 'J';
-	case 'J':				/* Trying to enter game. */
-	    Packet_printf(ibuf, "%c%s%s%s%d", ENTER_GAME_pack,
-			  nick_name, display, hostname, team);
-	    break;
-
-	case '?':
-	case 'H':				/* Help. */
-	default:
-	    printf("CLIENT VERSION...: %s\n", TITLE);
-	    printf("Supported commands are:\n"
-		   "H/?  -   Help - this text.\n"
-		   "N    -   Next server, skip this one.\n"
-		   "S    -   list Status.\n"
-		   "T    -   set Team.\n"
-		   "Q    -   Quit.\n"
-		   "K    -   Kick a player.                (only owner)\n"
-		   "M    -   send a Message.               (only owner)\n"
-		   "L    -   Lock/unLock server access.    (only owner)\n"
-		   "D(*) -   shutDown/cancel shutDown.     (only owner)\n"
-		   "R(#) -   set maximum number of Robots. (only owner)\n"
-		   "O    -   Modify a server option.       (only owner)\n"
-		   "V    -   View the server options.\n"
-		   "J or just Return enters the game.\n"
-		   "(*) If you don't specify any delay, you will signal that\n"
-		   "    the server should stop an ongoing shutdown.\n"
-		   "(#) Not specifying the maximum number of robots is\n"
-		   "    the same as specifying 0 robots.\n");
-
-	    /*
-	     * Next command.
-	     */
-	    continue;
 	}
 
-	retries = (c == 'J') ? 2 : 0;
+	retries = (c == 'J' || c == 'S') ? 2 : 0;
 	for (i = 0; i <= retries; i++) {
 	    if (i > 0) {
 		SetTimeout(1, 0);
@@ -436,7 +544,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    }
 	    if (write(ibuf->sock, ibuf->buf, ibuf->len) != ibuf->len) {
 		error("Couldn't send request to server.");
-		exit(-1);
+		exit(1);
 	    }
 	}
 
@@ -445,6 +553,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 	 */
 	SetTimeout(3, 0);
 	do {
+	    Sockbuf_clear(ibuf);
 	    if (Get_reply_message(ibuf) <= 0) {
 		errno = 0;
 		error("No answer from server");
@@ -466,7 +575,7 @@ static bool Process_commands(sockbuf_t *ibuf)
 		/*
 		 * Oh glorious success.
 		 */
-		switch (reply_to) {
+		switch (reply_to & 0xFF) {
 
 		case OPTION_LIST_pack:
 		    while (Packet_scanf(ibuf, "%S", str) > 0) {
@@ -515,6 +624,18 @@ static bool Process_commands(sockbuf_t *ibuf)
 		    }
 		    break;
 
+		case CREDENTIALS_pack:
+		    if (Packet_scanf(ibuf, "%ld", &key) <= 0) {
+			errno = 0;
+			error("Incomplete credentials reply from server");
+		    }
+		    else {
+			has_credentials++;
+			cmd_credentials = c;
+			continue;
+		    }
+		    break;
+
 		default:
 		    puts("*** Operation successful.");
 		    break;
@@ -552,6 +673,12 @@ static bool Process_commands(sockbuf_t *ibuf)
 	    case E_VERSION:
 		error("We have an incompatible version says the server");
 		break;
+	    case E_NOENT:
+		error("No such variable, says the server");
+		break;
+	    case E_UNDEFINED:
+		error("Requested operation is undefined, says the server");
+		break;
 	    default:
 		error("Wrong status '%d'", status);
 		break;
@@ -564,10 +691,10 @@ static bool Process_commands(sockbuf_t *ibuf)
 		return (false);
 
 	    /*
-	     * If we wanted to enter the game and we were allowed to, return true
-	     * (we are done).  If we weren't allowed, either return false (get next
-	     * server) if we are auto_connecting or get next command if we aren't
-	     * auto_connecting (interactive).
+	     * If we wanted to enter the game and we were allowed to, return
+	     * true (we are done).  If we weren't allowed, either return false
+	     * (get next server) if we are auto_connecting or get next command
+	     * if we aren't auto_connecting (interactive).
 	     */
 	    if (reply_to == ENTER_GAME_pack) {
 		if (status == SUCCESS && login_port > 0) {
@@ -578,8 +705,8 @@ static bool Process_commands(sockbuf_t *ibuf)
 		}
 	    }
 
-	     SetTimeout(0, 500*1000);
-	} while (SocketReadable(ibuf->sock));
+	    SetTimeout(0, 500*1000);
+	} while (--max_replies > 0 && SocketReadable(ibuf->sock));
 
 	/*
 	 * Get next command.
@@ -598,49 +725,104 @@ static bool Process_commands(sockbuf_t *ibuf)
  */
 static bool Connect_to_server(void)
 {
-    int			socket_i;		/* Info socket */
     sockbuf_t		ibuf;			/* info buffer */
     bool		result;
 
-    if ((socket_i = CreateDgramSocket(0)) == -1) {
-	error("Could not create info socket");
-	exit(-1);
-    }
-    if (DgramConnect(socket_i, server_addr, server_port) == -1) {
-	error("Can't connect to server %s on port %d\n",
-	      server_addr, server_port);
-	SocketClose(socket_i);
-	return (false);
-    }
-    if (Sockbuf_init(&ibuf, socket_i, CLIENT_RECV_SIZE,
+    if (Sockbuf_init(&ibuf, -1, CLIENT_RECV_SIZE,
 		     SOCKBUF_READ | SOCKBUF_WRITE | SOCKBUF_DGRAM) == -1) {
 	error("No memory for info buffer");
-	SocketClose(socket_i);
-	exit(-1);
+	exit(1);
     }
     result = Process_commands(&ibuf);
-    SocketClose(socket_i);
+    if (ibuf.sock != -1) {
+	SocketClose(ibuf.sock);
+    }
     Sockbuf_cleanup(&ibuf);
 
     return result;
 }
 
 
+static bool Contact_servers(int count, char **servers)
+{
+    bool		connected = false;
+    const int		max_retries = 2;
+    int			i;
+    int			fd;
+    int			retries;
+    int			contacted;
+    sockbuf_t		sbuf;			/* contact buffer */
+
+
+    if ((fd = CreateDgramSocket(0)) == -1) {
+	error("Could not create connection socket");
+	exit(1);
+    }
+    if (Sockbuf_init(&sbuf, fd, CLIENT_RECV_SIZE,
+		     SOCKBUF_READ | SOCKBUF_WRITE | SOCKBUF_DGRAM) == -1) {
+	error("No memory for contact buffer");
+	exit(1);
+    }
+    if (!count) {
+	retries = 0;
+	contacted = 0;
+	do {
+	    Sockbuf_clear(&sbuf);
+	    Packet_printf(&sbuf, "%u%s%hu%c", MAGIC,
+			  real_name, GetPortNum(sbuf.sock), CONTACT_pack);
+	    if (Query_all(sbuf.sock, contact_port, sbuf.buf, sbuf.len) == -1) {
+		error("Couldn't send contact requests");
+		exit(1);
+	    }
+	    if (retries) {
+		printf("Searching again...\n");
+	    }
+	    while (Get_contact_message(&sbuf)) {
+		contacted++;
+		if ((connected = Connect_to_server()) != 0) {
+		    break;
+		}
+	    }
+	} while (!contacted && retries++ < max_retries);
+    }
+    else {
+	for (i = 0; i < count && !connected; i++) {
+	    retries = 0;
+	    contacted = 0;
+	    do {
+		Sockbuf_clear(&sbuf);
+		Packet_printf(&sbuf, "%u%s%hu%c", MAGIC,
+			      real_name, GetPortNum(sbuf.sock), CONTACT_pack);
+		if (DgramSend(sbuf.sock, servers[i], contact_port,
+			      sbuf.buf, sbuf.len) == -1) {
+		    error("Can't contact server at %s on port %d",
+			  servers[i], contact_port);
+		}
+		if (retries) {
+		    printf("Retrying %s...\n", servers[i]);
+		}
+		if (Get_contact_message(&sbuf)) {
+		    contacted++;
+		    if ((connected = Connect_to_server()) != 0) {
+			break;
+		    }
+		}
+	    } while (!contacted && retries++ < max_retries);
+	}
+    }
+    Sockbuf_cleanup(&sbuf);
+    close(fd);
+
+    return (connected ? true : false);
+}
+
 
 /*
  * Oh glorious main(), without thee we cannot exist.
  */
 int main(int argc, char *argv[])
 {
-    int			i, n;
     struct passwd	*pwent;
-    bool		connected = false;
-#ifdef	LIMIT_ACCESS
-    extern bool		Is_allowed(char *);
-#endif
-    void Parse_options(int *argcp, char **argvp, char *realName, int *port,
-		       int *my_team, int *list, int *join, int *noLocalMotd,
-		       char *nickName, char *dispName, char *shut_msg);
 
 
     /*
@@ -664,20 +846,10 @@ int main(int argc, char *argv[])
     cmw_priv_init();
 #endif /* CMW */
     init_error(argv[0]);
+    srand(time((time_t *)0) * getpid());
+    Check_client_versions();
     GetLocalHostName(hostname, sizeof hostname);
 
-    if ((socket_c = CreateDgramSocket(0)) == -1) {
-	error("Could not create connection socket");
-	SocketClose(socket_c);
-	exit(-1);
-    }
-    if (Sockbuf_init(&sbuf, socket_c, CLIENT_RECV_SIZE,
-		     SOCKBUF_READ | SOCKBUF_WRITE | SOCKBUF_DGRAM) == -1) {
-	error("No memory for contact buffer");
-	SocketClose(socket_c);
-	exit(-1);
-    }
-
     /*
      * --- Setup core of pack ---
      */
@@ -692,9 +864,6 @@ int main(int argc, char *argv[])
     strncpy(real_name, pwent->pw_name, sizeof(real_name) - 1);
 #endif
     nick_name[0] = '\0';
-    Sockbuf_clear(&sbuf);
-    Packet_printf(&sbuf, "%u%s%hu%c", MAGIC,
-		  real_name, GetPortNum(sbuf.sock), CONTACT_pack);
 
 
     /*
@@ -729,60 +898,13 @@ int main(int argc, char *argv[])
 	exit (-1);
 #endif
 
-    /*
-     * --- Try to contact server ---
-     */
-    if (argc > 1) {		/* Server specified on command line? */
-	n = 0;
-	for (i = 1; i < argc; i++) {
-	    if (DgramSend(sbuf.sock, argv[i], contact_port,
-			  sbuf.buf, sbuf.len) == -1) {
-		error("Can't send contact request to server at %s port %d",
-		      argv[i], contact_port);
-	    } else {
-		n++;
-	    }
-	}
-	if (n == 0) {
-	    exit(1);
-	}
-
-	while (Get_contact_message()) {
-	    if ((connected = Connect_to_server()) != 0 || --n == 0) {
-		break;
-	    }
-	}
-
-    } else {				/* Search for servers... */
-	SetTimeout(3, 0);
-	if (Query_all(sbuf.sock, contact_port, sbuf.buf, sbuf.len) == -1) {
-	    error("Couldn't send contact requests");
-	    exit(1);
-	}
-
-	/*
-	 * Wait for answer.
-	 */
-	while (Get_contact_message()) {
-	    if ((connected = Connect_to_server()) != 0) {
-		break;
-	    }
-	}
-    }
-
-    close(socket_c);
-    Sockbuf_cleanup(&sbuf);
-    if (connected) {
-	extern int Join(char *server_addr, char *server_name, int port,
-			char *real, char *nick, int my_team,
-			char *display, unsigned version);
-
+    if (Contact_servers(argc - 1, &argv[1])) {
 	Join(server_addr, server_name, login_port,
 	     real_name, nick_name, team,
 	     display, server_version);
+	return 0;
     }
-
-    return ((connected == true) ? 0 : 1);
+    return 1;
 }
 
 
@@ -795,6 +917,10 @@ int main(int argc, char *argv[])
 #ifndef MAX_INTERFACE
 #define MAX_INTERFACE    16	/* Max. number of network interfaces. */
 #endif
+#ifdef DEBUGBROADCAST
+#undef D
+#define D(x)	x
+#endif
 
 
 /*
@@ -866,7 +992,7 @@ static int Query_subnet(int sockfd,
 	 * are still transmitting packets for non-existing servers
 	 * and the server packet colliding with one of our packets.
 	 */
-	usleep(10000);
+	micro_delay((unsigned)10000);
     }
 
     return 0;
@@ -931,7 +1057,7 @@ static int Query_fudged(int sockfd, int port, char *msg, int msglen)
 static int Query_all(int sockfd, int port, char *msg, int msglen)
 {
 #ifdef QUERY_FUDGED
-    return Query_fudged(sbuf.sock, contact_port, sbuf.buf, sbuf.len);
+    return Query_fudged(sockfd, contact_port, msg, msglen);
 #else
 
     int         	fd, len, ifflags, count = 0, broadcasts = 0, haslb = 0;
@@ -973,7 +1099,7 @@ static int Query_all(int sockfd, int port, char *msg, int msglen)
 	D( printf("\taddress family %d\n", ifreqp->ifr_addr.sa_family); );
 
 	len += sizeof(struct ifreq);
-#if BSD >= 199006 || HAVE_SA_LEN
+#if BSD >= 199006 || HAVE_SA_LEN || defined(_SOCKADDR_LEN) || defined(_AIX)
 	/*
 	 * Recent TCP/IP implementations have a sa_len member in the socket
 	 * address structure in order to support protocol families that have
@@ -1017,8 +1143,10 @@ static int Query_all(int sockfd, int port, char *msg, int msglen)
 	    /*
 	     * Only send on the loopback if we don't broadcast.
 	     */
-	    loopback = *(struct sockaddr_in *)&ifreq.ifr_addr;
-	    haslb = 1;
+	    if (haslb == 0) {
+		loopback = *(struct sockaddr_in *)&ifreq.ifr_addr;
+		haslb = 1;
+	    }
 	    continue;
 	} else if ((ifflags & IFF_POINTOPOINT) != 0) {
 	    D( printf("\tpoint-to-point interface\n"); );
@@ -1077,7 +1205,7 @@ static int Query_all(int sockfd, int port, char *msg, int msglen)
 		 */
 		continue;
 	    }
-	    
+
 	    /*
 	     * Broadcasting failed.
 	     * Try it in a different (kludgy) manner.
@@ -1129,3 +1257,80 @@ static int Query_all(int sockfd, int port, char *msg, int msglen)
 
 #endif	/* QUERY_FUDGED */
 }
+
+/*
+ * Verify that all source files making up this program have been
+ * compiled for the same version.  Too often bugs have been reported
+ * for incorrectly compiled programs.
+ */
+static void Check_client_versions(void)
+{
+#ifdef SOUND
+    extern char		audio_version[];
+#endif
+    extern char		caudio_version[],
+			client_version[],
+			configure_version[],
+			dbuff_version[],
+			default_version[],
+			error_version[],
+			join_version[],
+			math_version[],
+			net_version[],
+			netclient_version[],
+			paint_version[],
+			record_version[],
+			socklib_version[],
+			texture_version[],
+			widget_version[],
+			xevent_version[],
+			xinit_version[],
+			xpilot_version[],
+			xpmread_version[];
+    static struct file_version {
+	char		filename[16];
+	char		*versionstr;
+    } file_versions[] = {
+#ifdef SOUND
+	{ "*audio", audio_version },
+#endif
+	{ "caudio", caudio_version },
+	{ "client", client_version },
+	{ "configure", configure_version },
+	{ "dbuff", dbuff_version },
+	{ "default", default_version },
+	{ "error", error_version },
+	{ "join", join_version },
+	{ "math", math_version },
+	{ "net", net_version },
+	{ "netclient", netclient_version },
+	{ "paint", paint_version },
+	{ "record", record_version },
+	{ "socklib", socklib_version },
+	{ "texture", texture_version },
+	{ "widget", widget_version },
+	{ "xevent", xevent_version },
+	{ "xinit", xinit_version },
+	{ "xpilot", xpilot_version },
+	{ "xpmread", xpmread_version },
+    };
+    int			i;
+    int			oops = 0;
+
+    for (i = 0; i < NELEM(file_versions); i++) {
+	if (strcmp(VERSION, file_versions[i].versionstr)) {
+	    oops++;
+	    error("Source file %s.c (\"%s\") is not compiled "
+		  "for the current version (\"%s\")!",
+		  file_versions[i].filename,
+		  file_versions[i].versionstr,
+		  VERSION);
+	}
+    }
+    if (oops) {
+	error("%d version inconsistency errors, cannot continue.", oops);
+	error("Please recompile this program properly.");
+	exit(1);
+    }
+}
+
